using System;
using System.Runtime.InteropServices;
using Cake3;
using Cake3.Ace;

namespace AceDllWrapped
{
	/// <summary>
	/// 
	/// C# wrapper for ACE Dlls.
	/// 
	/// The wrapper is written for console applications. 
	/// Modify the input/output according to your needs!
	/// 
	/// </summary>

	public class AceWrap
	{
		///////////////// global varliables ///////////////////////////////////////

		public const int COMMENTBUFSIZE = 32768;		// comments may be up to 32k in size;
														// decrease it if you don't want to put that
		// large comments to archives and if you don't want to receive all of such large
		// comments (ACE_COMMENT_SMALLBUF is returned if comment does not fit into buffer)


		[StructLayout(LayoutKind.Sequential, Pack=1)]	// allows for copying ANSI-strings to
			public struct CommentStrHelperStruc			// native comment buffer
		{
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=COMMENTBUFSIZE)]
			public string Str;
		}


		public static UnAceFnc.sACEInitDllStruc			// must be global to prevent delegates from being destroyed
			DllData = new UnAceFnc.sACEInitDllStruc();	// set up struc (all fields initialized to zero)

		public static IntPtr DllDataPtr;				// must be global to keep native string buffers alive while using Dll


		///////////////// helper methods //////////////////////////////////////////

		public static void SetCommentBuf(string CommentStr)
		{
			CommentStrHelperStruc CommentStruc = new CommentStrHelperStruc();
		
			CommentStruc.Str = CommentStr;		
			Marshal.StructureToPtr(CommentStruc, DllData.GlobalData.Comment.Buf, false);
		}


		public static void AceDone()
		{
			Marshal.FreeHGlobal(DllData.GlobalData.Comment.Buf);
			Marshal.DestroyStructure(DllDataPtr, typeof(UnAceFnc.sACEInitDllStruc));
		}

		public static void OutFunctionCall(string Function)
		{
			Console.WriteLine("calling {0}\n======================================", Function);
		}

		public static bool OutFunctionResult(string Function, int Result)
		{
			Console.WriteLine("--------------------------------------\n{0} returned: {1}\n\n", Function, Result);
			
			return Result==UnAceFnc.ACE_ERROR_NOERROR;
		}

		public static int UserPrompt(string PromptStr, bool IsRequest)
		{
			return Callback.ACE_CALLBACK_RETURN_OK;
//			char Ch;
//
//			Console.Write(IsRequest ? "Request: " : "Error: ");
//			Console.Write(PromptStr);
//			Console.Write(IsRequest ? "? (Yes, No, Cancel)" : "! (Ok, Cancel)");
//
//			do
//			{
//				Ch = Char.ToUpper((char)Console.Read());
//
//			} while (!(IsRequest ? (Ch=='Y' || Ch=='N' || Ch=='C') :	(Ch=='O' || Ch=='C')));
//
//			switch (Ch)
//			{
//				case 'O':
//				case 'Y':
//				{
//					return Callback.ACE_CALLBACK_RETURN_OK;
//				}
//
//				case 'N':
//				{
//					return Callback.ACE_CALLBACK_RETURN_NO;
//				}
//			}
//
//			return Callback.ACE_CALLBACK_RETURN_CANCEL;
		}


		public static bool FileExists(string FileName)
		{
			return (new System.IO.FileInfo(FileName)).Exists;
		}



		///////////////// handle info callbacks ///////////////////////////////////

		public static int InfoProc(IntPtr Info)
		{																	  
			string InfoStr;

			// make data from native memory accessible by marshalling to managed structures
			sACECallbackGlobalStruc Global      = (sACECallbackGlobalStruc)Marshal.PtrToStructure(Info, typeof(sACECallbackGlobalStruc));
			sACECallbackArchiveStruc Archive    = (sACECallbackArchiveStruc)Marshal.PtrToStructure(Info, typeof(sACECallbackArchiveStruc));
			sACECallbackRealFileStruc RealFile  = (sACECallbackRealFileStruc)Marshal.PtrToStructure(Info, typeof(sACECallbackRealFileStruc));
			sACECallbackCopyStruc Copy			= (sACECallbackCopyStruc)Marshal.PtrToStructure(Info, typeof(sACECallbackCopyStruc));

			switch (Global.Code)
			{
					/* .. add more codes to handle here ..*/

				case Callback.ACE_CALLBACK_INFO_FILELISTCREATE:
				{
					InfoStr = "creating file list";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_FILELISTCREATEEND:
				{
					InfoStr = "finished creating file list";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_FILELISTADD:
				{
					InfoStr = "adding file to file list";
					break;
				}


				case Callback.ACE_CALLBACK_INFO_GENERALKEY:
				{
					InfoStr = "key is general key! get your personalized one from www.winace.com";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_TMPARCCREATE:
				{
					InfoStr = "creating temporary archive";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_TMPARCCREATEEND:
				{
					InfoStr = "finished creating temporary archive";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_ADDRECREC:
				{
					InfoStr = "adding recovery record";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_ADDRECRECEND:
				{
					InfoStr = "finished adding recovery record";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_RECREC:
				{
					InfoStr = "trying to recover archive by recovery record";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_NORECREC:
				{
					InfoStr = "found no recovery record";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_RECOVERED:
				{
					InfoStr = "archive could be recovered";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_NODAMAGE:
				{
					InfoStr = "archive not damaged, repair operation aborted";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_FNDMAINHEAD:
				{
					InfoStr = "found main header";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_FILESORT:
				{
					InfoStr = "sorting file list";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_FILESORTEND:
				{
					InfoStr = "finished sorting file list";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_COPYEND:
				{
					InfoStr = "finished copying file";
					break;
				}

				case Callback.ACE_CALLBACK_INFO_COPY:
				{
					InfoStr = "copying file";
					break;
				}

				default:
				{
					InfoStr = "";
					break;
				}
			}

			Console.WriteLine("Info: " + InfoStr);

			return Callback.ACE_CALLBACK_RETURN_OK;
		}



		///////////////// handle error callbacks //////////////////////////////////

//		public static int HandleErrorGlobal(sACECallbackGlobalStruc Global)
//		{
//			string ErrorStr;
//			int Result;
//
//			switch (Global.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_ERROR_MEMORY:
//				{
//					ErrorStr = "not enough memory";
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_UNCSPACE:
//				{
//					ErrorStr = "could not detect available space on network drive";
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;
//					break;
//				}
//			}
//
//			Console.WriteLine("\nError: " + ErrorStr);
//
//			return Result;
//		}
//
//		
//		public static int HandleErrorArchive(sACECallbackArchiveStruc Archive)
//		{
//			string ErrorStr;
//			int Result;
//			bool DoPrompt;
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(Archive.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(Archive.ArchiveData, typeof(sACEArchiveDataStruc));
//
//			string ArcName = ArchiveData.ArchiveName;
//			
//			Result   = Callback.ACE_CALLBACK_RETURN_OK;
//			DoPrompt = false;
//
//			switch (Archive.Code)
//			{
//					/* .. add more codes to handle here ..*/
//			
//				case Callback.ACE_CALLBACK_ERROR_AV:
//				{
//					ErrorStr = "AV of archive " + ArcName + " invalid or does not match to yours";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_OPENARCHIVEREAD:
//				{
//					ErrorStr = "could not open archive " + ArcName + " for reading";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_READARCHIVE:
//				{
//					ErrorStr = "error reading from archive " + ArcName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ARCHIVEBROKEN:
//				{
//					ErrorStr = "archive " + ArcName + " is broken";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_NOFILES:
//				{
//					ErrorStr = "no files specified";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ISNOTANARCHIVE:
//				{
//					ErrorStr = "file is not an ACE archive";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_HIGHERVERSION:
//				{
//					ErrorStr = "this Dll version is not able to handle the archive";
//					break;
//				}
//
//
//				case Callback.ACE_CALLBACK_ERROR_MODIFYVOLUME:
//				{
//					ErrorStr = "volumes can not be modified";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_MODIFYLOCKEDARCHIVE:
//				{
//					ErrorStr = "locked archive " + ArcName + " can not be modified";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_TOODAMAGED:
//				{
//					ErrorStr = "recovery record of " + ArcName + " too damaged, continue with normal repair";
//					DoPrompt = true;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ARCHIVEEXISTS:
//				{
//					ErrorStr = ArcName + " already exists";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_OPENREPAIRARCHIVE:
//				{
//					ErrorStr = "could not create archive " + ArcName + " for repairing";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_OPENARCHIVEWRITE:
//				{
//					ErrorStr = "could not open archive " + ArcName + " for writing";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_WRITEARCHIVE:
//				{
//					ErrorStr = "error writing to archive " + ArcName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ALREADYSFX:
//				{
//					ErrorStr = "archive " + ArcName + " is already selfextracting";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ADDSFXTOVOLUME:
//				{
//					ErrorStr = "can not add SFX module to volumes";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ARCHIVESAVE:
//				{
//					ErrorStr = "not enough space to save archive";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_TEMPDIRCREATE:
//				{
//					ErrorStr = "could not create file in temp directory";
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;
//					break;
//				}
//			}
//
//			if (DoPrompt)
//			{
//				return UserPrompt(ErrorStr, false);
//			}
//			else
//			{
//				Console.Write("\nError: ");
//				Console.WriteLine(ErrorStr);
//
//				return Result;
//			}
//		}
//
//		
//		public static int HandleErrorArchivedFile(sACECallbackArchivedFileStruc ArchivedFile)
//		{
//			string ErrorStr;
//			int Result;
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(ArchivedFile.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(ArchivedFile.ArchiveData, typeof(sACEArchiveDataStruc));
//			sACEFileDataStruc FileData       = (sACEFileDataStruc)Marshal.PtrToStructure(ArchivedFile.FileData, typeof(sACEFileDataStruc));
//
//			string FileName = FileData.SourceFileName;
//
//			Result   = Callback.ACE_CALLBACK_RETURN_OK;
//
//			switch (ArchivedFile.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_ERROR_CREATIONNAMEINUSE:
//				{
//					ErrorStr = "could not extract " + FileName + ": name used by directory";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_WRITE:
//				{
//					ErrorStr = "error writing " + FileName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_OPENWRITE:
//				{
//					ErrorStr = "error opening " + FileName + " for writing";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_METHOD:
//				{
//					ErrorStr = "compression method not known to this Dll version";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_EXTRACTSPACE:
//				{
//					ErrorStr = "not enough space to extract " + FileName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_CREATION:
//				{
//					ErrorStr = "creation of " + FileName + " failed (write-protection?)";
//					break;
//				}
//
//
//				case Callback.ACE_CALLBACK_ERROR_ENCRYPTIONCRC:
//				{
//					ErrorStr = "encryption of " + FileName + " failed because of CRC-check error";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_READ:
//				{
//					ErrorStr = "error reading " + FileName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_OPENREAD:
//				{
//					ErrorStr = "error opening " + FileName + " for reading";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_FILENAMETOOLONG:
//				{
//					ErrorStr = "resulting file name too long (" + FileName + ")";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_REPACKCRC:
//				{
//					ErrorStr = "CRC-check error recompressing " + FileName + " (archive broken or wrong password)";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_EXCLUDEPATH:
//				{
//					ErrorStr = "could not exclude paths; two or more files would have the same name";
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;
//					break;
//				}
//			}
//
//			Console.Write("\nError: " + ErrorStr);
//
//			return Result;
//		}
//
//
//		public static int HandleErrorRealFile(sACECallbackRealFileStruc ErrorRealFile)
//		{
//			string ErrorStr;
//			int Result;
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(ErrorRealFile.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(ErrorRealFile.ArchiveData, typeof(sACEArchiveDataStruc));
//
//			switch (ErrorRealFile.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_ERROR_MOVEDELETE:
//				{
//					ErrorStr = "deletion of file or directory failed";
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;					
//					break;
//				}
//			}
//
//			Console.Write("\nError: " + ErrorStr/* + Error->FileName*/);
//
//			return Result;
//		}
//
//
//		public static int HandleErrorSpace(ref sACECallbackSpaceStruc ErrorSpace)
//		{
//			string ErrorStr;
//			int Result;
//
//			switch (ErrorSpace.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_ERROR_TEMPDIRSPACE:
//				{
//					ErrorSpace.Directory = "C:\\TEMP"; //change temporary directory to C:\TEMP
//					ErrorStr = "not enough space at current temp directory (" + ErrorSpace.Directory + ")";
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_ERROR_ARCHIVESPACE:
//				{
//					ErrorSpace.Directory = "C:\\";     // save archive at C:\
//					ErrorStr = "not enough space at destination path (" + ErrorSpace.Directory + ")";
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;
//
//					break;
//				}
//			}
//
//			Console.WriteLine("\nError: " + ErrorStr);
//
//			return Result;
//		}
//
//
//		public static int HandleErrorSFXFile(sACECallbackSFXFileStruc ErrorSFXFile)
//		{
//			string ErrorStr;
//			int Result;
//
//			switch (ErrorSFXFile.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_ERROR_READINGSFXFILE:
//				{
//					ErrorStr = "error reading SFX file " + ErrorSFXFile.SFXFileName;
//					Result   = Callback.ACE_CALLBACK_RETURN_OK;
//					break;
//				}
//
//				default:
//				{
//					ErrorStr = "unknown";
//					Result   = Callback.ACE_CALLBACK_RETURN_CANCEL;
//					break;
//				}
//			}
//
//			Console.WriteLine("\nError: " + ErrorStr);
//
//			return Result;
//		}
//
//
//		public static int ErrorProc(IntPtr Error)
//		{
//			int Result = new int();
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACECallbackGlobalStruc Global = (sACECallbackGlobalStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackGlobalStruc));
//
//			switch (Global.StructureType)
//			{
//				case Callback.ACE_CALLBACK_TYPE_GLOBAL:
//				{
//					Result = HandleErrorGlobal(Global);
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackArchiveStruc Archive  = (sACECallbackArchiveStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackArchiveStruc));
//					Result = HandleErrorArchive(Archive); 
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVEDFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackArchivedFileStruc ArchivedFile  = (sACECallbackArchivedFileStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackArchivedFileStruc));
//					Result = HandleErrorArchivedFile(ArchivedFile); 					
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_REALFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackRealFileStruc RealFile  = (sACECallbackRealFileStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackRealFileStruc));
//					Result = HandleErrorRealFile(RealFile);					
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_SPACE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackSpaceStruc Space = (sACECallbackSpaceStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackSpaceStruc));
//					Result = HandleErrorSpace(ref Space); 					
//					Marshal.StructureToPtr(Space, Error, false);	// write back structure to native memory
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_SFXFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackSFXFileStruc SFXFile = (sACECallbackSFXFileStruc)Marshal.PtrToStructure(Error, typeof(sACECallbackSFXFileStruc));
//					Result = HandleErrorSFXFile(SFXFile);					
//					break;
//				}
//
//				default:
//				{
//					Result = Callback.ACE_CALLBACK_RETURN_CANCEL;					
//					break;
//				}
//			}
//
//			return Result;
//		}



		//////////////// handle request callbacks /////////////////////////////////

//		public static int HandleRequestGlobal(sACECallbackGlobalStruc Request)
//		{
//			string RequestStr = "";
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData = (sACEGlobalDataStruc)Marshal.PtrToStructure(Request.GlobalData, typeof(sACEGlobalDataStruc));
//
//			switch (Request.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				default:
//				{
//					Console.WriteLine("\nunknown request");
//
//					return Callback.ACE_CALLBACK_RETURN_CANCEL;
//				}
//			}
//
//			return UserPrompt(RequestStr, true);
//		}
//
//
//		public static int HandleRequestArchive(sACECallbackArchiveStruc Request)
//		{
//			string RequestStr = "";
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(Request.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(Request.ArchiveData, typeof(sACEArchiveDataStruc));
//
//			switch (Request.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_REQUEST_CHANGEVOLUME:
//				{
//					RequestStr = "ready to process next volume " + ArchiveData.ArchiveName;
//					break;
//				}
//
//
//				case Callback.ACE_CALLBACK_REQUEST_MARKASSOLID:
//				{
//					RequestStr = "mark archive as solid";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_REQUEST_ARCHIVEEXISTS:
//				{
//					RequestStr = "overwrite file " + ArchiveData.ArchiveName + " with archive";
//					break;
//				}
//
//				default:
//				{
//					Console.WriteLine("\nunknown request");
//
//					return Callback.ACE_CALLBACK_RETURN_CANCEL;
//				}
//			}
//
//			return UserPrompt(RequestStr, true);
//		}
//
//
//		public static int HandleRequestArchivedFile(sACECallbackArchivedFileStruc ArchivedFile)
//		{
//			string RequestStr = "";
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(ArchivedFile.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(ArchivedFile.ArchiveData, typeof(sACEArchiveDataStruc));
//			sACEFileDataStruc FileData		 = (sACEFileDataStruc)Marshal.PtrToStructure(ArchivedFile.FileData, typeof(sACEFileDataStruc));
//
//			switch (ArchivedFile.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_REQUEST_OVERWRITE:
//				{
//					RequestStr = "overwrite existing file " + FileData.SourceFileName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_REQUEST_PASSWORD:
//				{
//					RequestStr = FileData.SourceFileName + " is encrypted, using \"testpassword\" as password";
//					GlobalData.DecryptPassword = "testpassword";					
//					Marshal.StructureToPtr(GlobalData, ArchivedFile.GlobalData, true);	// write back structure to native memory
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_REQUEST_DELARCHIVEDSYSFILE:
//				{
//					RequestStr = "delete system, hidden or read-only file " + FileData.SourceFileName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_REQUEST_ADDBROKENFILE:
//				{
//					RequestStr = "found broken file header, add " + FileData.SourceFileName;
//					break;
//				}
//
//				default:
//				{
//					Console.WriteLine("\nunknown request");
//					return Callback.ACE_CALLBACK_RETURN_CANCEL;
//				}
//			}
//
//			return UserPrompt(RequestStr, true);
//		}
//
//
//		public static int HandleRequestRealFile(sACECallbackRealFileStruc RealFile)
//		{
//			string RequestStr = "";
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(RealFile.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(RealFile.ArchiveData, typeof(sACEArchiveDataStruc));
//
//			switch (RealFile.Code)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_REQUEST_MOVEDELREALSYSFILE:
//				{
//					RequestStr = "delete hidden, system or read-only file " + RealFile.FileName;
//					break;
//				}
//
//				default:
//				{
//					Console.WriteLine("\nunknown request");
//
//					return Callback.ACE_CALLBACK_RETURN_CANCEL;
//				}
//			}
//
//			return UserPrompt(RequestStr, true);
//		}
//
//
//		public static int RequestProc(IntPtr Request)
//		{
//			// make data from native memory accessible by marshalling to managed structures
//			sACECallbackGlobalStruc Global				= (sACECallbackGlobalStruc)Marshal.PtrToStructure(Request, typeof(sACECallbackGlobalStruc));
//
//			switch (Global.StructureType)
//			{
//				case Callback.ACE_CALLBACK_TYPE_GLOBAL:
//				{
//					return HandleRequestGlobal(Global);
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackArchiveStruc Archive = (sACECallbackArchiveStruc)Marshal.PtrToStructure(Request, typeof(sACECallbackArchiveStruc));
//					return HandleRequestArchive(Archive);
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVEDFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackArchivedFileStruc ArchivedFile = (sACECallbackArchivedFileStruc)Marshal.PtrToStructure(Request, typeof(sACECallbackArchivedFileStruc));
//					return HandleRequestArchivedFile(ArchivedFile);
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_REALFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackRealFileStruc RealFile = (sACECallbackRealFileStruc)Marshal.PtrToStructure(Request, typeof(sACECallbackRealFileStruc));
//					return HandleRequestRealFile(RealFile);
//				}
//
//				default:
//				{
//					return Callback.ACE_CALLBACK_RETURN_CANCEL;
//				}
//			}
//		}
//	
//
//
//		///////////////// handle state callbacks //////////////////////////////////
//
//		public static int HandleStateStartArchive(sACECallbackArchiveStruc Archive)
//		{
//			string ActionStr;
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(Archive.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(Archive.ArchiveData, typeof(sACEArchiveDataStruc));
//
//			string ArcName = ArchiveData.ArchiveName;
//			
//			switch (Archive.Operation)
//			{
//					/* .. add more codes to handle here ..*/
//				
//				case Callback.ACE_CALLBACK_OPERATION_LIST:
//				{
//					ActionStr = "Listing " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_TEST:
//				{
//					ActionStr = "Testing " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_EXTRACT:
//				{
//					ActionStr = "Extracting " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ADD:
//				{
//					ActionStr = "Adding files to " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_DELETE:
//				{
//					ActionStr = "Deleting files from " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ENCRYPT:
//				{
//					ActionStr = "Encrypting " + ArcName + "\n";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_REPAIR:
//				{
//					ActionStr = "Trying to repair " + ArcName + " (no recovery record found)";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_SETCMT:
//				{
//					ActionStr = "Changing main comment of " + ArcName;
//					SetCommentBuf("This is the changed main comment of "  + ArcName);
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_RECOVER:
//				{
//					ActionStr = "Trying to recover archive using recovery record";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_HEADSEARCH:
//				{
//					ActionStr = "Searching for headers";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_RECRECSEARCH:
//				{
//					ActionStr = "Searching for recovery record";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ADDSFX:
//				{
//					ActionStr = "Adding SFX to " + ArcName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_LOCK:
//				{
//					ActionStr = "Locking " + ArcName;
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ADDRECOVREC:
//				{
//					ActionStr = "Adding recovery record to " + ArcName;
//					break;
//				}
//
//				default:
//				{
//					ActionStr = "unknown operation on " + ArcName;
//					break;
//				}
//			}
//
//			Console.WriteLine(ActionStr);
//
//			return Callback.ACE_CALLBACK_RETURN_OK;
//		}


//		public static int HandleStateStartFile(sACECallbackArchivedFileStruc ArchivedFile)
//		{
//			string ActionStr;
//
//			// make data from native memory accessible by marshalling to managed structures
//			sACEGlobalDataStruc GlobalData   = (sACEGlobalDataStruc)Marshal.PtrToStructure(ArchivedFile.GlobalData, typeof(sACEGlobalDataStruc));
//			sACEArchiveDataStruc ArchiveData = (sACEArchiveDataStruc)Marshal.PtrToStructure(ArchivedFile.ArchiveData, typeof(sACEArchiveDataStruc));
//			sACEFileDataStruc FileData		 = (sACEFileDataStruc)Marshal.PtrToStructure(ArchivedFile.FileData, typeof(sACEFileDataStruc));
//
//			switch (ArchivedFile.Operation)
//			{
//					/* .. add more codes to handle here ..*/
//
//				case Callback.ACE_CALLBACK_OPERATION_LIST:
//				{
//					ActionStr = "Found";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_TEST:
//				{
//					ActionStr = "Testing";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ANALYZE:
//				{
//					ActionStr = "Analyzing";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_EXTRACT:
//				{
//					ActionStr = "Extracting";
//					break;
//				}
//
//
//				case Callback.ACE_CALLBACK_OPERATION_ADD:
//				{
//					ActionStr = "Adding";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_REPACK:
//				{
//					ActionStr = "Repacking";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_DELETE:
//				{
//					ActionStr = "Deleting";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_REPAIR:
//				{
//					ActionStr = "Found";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_SETCMT:
//				{
//					ActionStr = "Changing comment";
//					SetCommentBuf("This is the file comment of " + FileData.SourceFileName);
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_ENCRYPT:
//				{
//					ActionStr = "Encrypting";
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_OPERATION_KEEP:
//				{
//					ActionStr = "Keeping";
//					break;
//				}
//
//				default:
//				{
//					ActionStr = "unknown operation on";
//					break;
//				}
//			}
//
//			Console.Write("  " + ActionStr + " " + FileData.SourceFileName + 
//				"  (" + FileData.Size + " byte uncompressed, "	+
//				FileData.CompressedSize + " byte compressed)\n");
//
//			return Callback.ACE_CALLBACK_RETURN_OK;
//		}


//		public static int StateProc(IntPtr State)
//		{
//			// make data from native memory accessible by marshalling to managed structures
//			sACECallbackArchiveStruc Archive    = (sACECallbackArchiveStruc)Marshal.PtrToStructure(State, typeof(sACECallbackArchiveStruc));
//
//			switch (Archive.StructureType)
//			{
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVE:
//				{
//					if (Archive.Code == Callback.ACE_CALLBACK_STATE_STARTARCHIVE)
//					{
//						return HandleStateStartArchive(Archive);
//					}
//
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_ARCHIVEDFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackArchivedFileStruc ArchivedFile  = (sACECallbackArchivedFileStruc)Marshal.PtrToStructure(State, typeof(sACECallbackArchivedFileStruc));
//
//					switch (ArchivedFile.Code)
//					{
//						case Callback.ACE_CALLBACK_STATE_STARTFILE:
//						{
//							return AceArchiver.HandleStateStartFile(ArchivedFile);
//						}
//
//						//case Callback.ACE_CALLBACK_STATE_ENDNOCRCCHECK: { break; }
//					}
//
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_REALFILE:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackRealFileStruc RealFile  = (sACECallbackRealFileStruc)Marshal.PtrToStructure(State, typeof(sACECallbackRealFileStruc));
//					
//					/* .. */
//
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_PROGRESS:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackProgressStruc Progress  = (sACECallbackProgressStruc)Marshal.PtrToStructure(State, typeof(sACECallbackProgressStruc));
//
//					
//					if (Progress.Code == Callback.ACE_CALLBACK_STATE_PROGRESS)
//					{
//						// make data from native memory accessible by marshalling to managed structures
//						sACEProgressDataStruc ProgressData  = (sACEProgressDataStruc)Marshal.PtrToStructure(Progress.ProgressData, typeof(sACEProgressDataStruc));
//
//						Console.WriteLine("    processed: {0} of {1} bytes ({2} of {3} bytes)", 
//							ProgressData.FileProcessedSize, ProgressData.FileSize,
//							ProgressData.TotalProcessedSize, ProgressData.TotalSize);
//					}
//
//					break;
//				}
//
//				case Callback.ACE_CALLBACK_TYPE_CRCCHECK:
//				{
//					// make data from native memory accessible by marshalling to managed structures
//					sACECallbackCRCCheckStruc CRCCheck  = (sACECallbackCRCCheckStruc)Marshal.PtrToStructure(State, typeof(sACECallbackCRCCheckStruc));
//
//					if (CRCCheck.Code == Callback.ACE_CALLBACK_STATE_ENDCRCCHECK)
//					{
//						if (CRCCheck.CRCOk) Console.WriteLine("   CRC OK");
//							else Console.WriteLine("   CRC-check error");
//					}
//
//					break;
//				}
//			}
//
//			return Callback.ACE_CALLBACK_RETURN_OK;
//		}
//	


		///////////////// calling ACE-Dll functions ///////////////////////////////

		public static bool CallAceInitDll()
		{
			int Result;

			DllData.GlobalData.MaxArchiveTestBytes = 0x2ffFF;	// search for archive header in first 256k of file
			DllData.GlobalData.MaxFileBufSize      = 0x2ffFF;	// read/write buffer size is 256k

			// set comment bufffer to receive comments of archive and/or set comments
			DllData.GlobalData.Comment.Buf         = Marshal.AllocHGlobal(COMMENTBUFSIZE);
			DllData.GlobalData.Comment.BufSize     = COMMENTBUFSIZE;

			DllData.GlobalData.TempDir			   = "C:\\TEMP";

			// set callback function pointers
			DllData.GlobalData.InfoCallbackProc    = new dACEInfoCallback(InfoProc);
			DllData.GlobalData.ErrorCallbackProc   = new dACEErrorCallback(AceArchiver.ErrorProc);
			DllData.GlobalData.RequestCallbackProc = new dACERequestCallback(AceArchiver.RequestProc);
			DllData.GlobalData.StateCallbackProc   = new dACEStateCallback(AceArchiver.StateProc);

			// create a permanent unmanaged memory block, so that strings persist (strings must be
			// valid at other function calls as well, as they will still be used by the dll)
			DllDataPtr = Marshal.AllocHGlobal(Marshal.SizeOf(DllData));
			Marshal.StructureToPtr(DllData, DllDataPtr, false);

			OutFunctionCall("ACEInitDll");

			Result = UnAceFnc.ACEInitDll(DllDataPtr);

			if (Result != UnAceFnc.ACE_ERROR_NOERROR)
				AceDone();

			return OutFunctionResult("ACEInitDll", Result);
		}
	

		public static bool CallAceReadArchiveData(string ArchiveName, ref sACEArchiveDataStruc ArchiveData)
		{
			int Result;

			UnAceFnc.sACEReadArchiveDataStruc ReadArchiveData = new UnAceFnc.sACEReadArchiveDataStruc(); 

			OutFunctionCall("ACEReadArchiveData");

			Result = UnAceFnc.ACEReadArchiveData(ArchiveName, ref ReadArchiveData);

			ArchiveData = (ReadArchiveData.ArchiveData != (IntPtr)null) ?
				(sACEArchiveDataStruc)Marshal.PtrToStructure(ReadArchiveData.ArchiveData, 
				typeof(sACEArchiveDataStruc)) :
				new sACEArchiveDataStruc();

			return OutFunctionResult("CallAceReadArchiveData", Result);
		}


		public static bool CallAceList(string ArchiveName, string FileList)
		{
			UnAceFnc.sACEListStruc
				List = new UnAceFnc.sACEListStruc();		// set up struc (all fields initialized to zero)

			List.Files.SourceDir   = "";        // archive main directory is base directory for FileList
			List.Files.FileList    = FileList;  // set FileList
			List.Files.ExcludeList = "";        // no files to exclude
			List.Files.FullMatch   = false;		// also list files partially matching (for instance: list DIR1\TEST.DAT	if FileList specifies TEST.DAT)

			OutFunctionCall("ACEList");

			return OutFunctionResult("ACEList", UnAceFnc.ACEList(ArchiveName, ref List));
		}
		

		public static bool CallAceTest(string ArchiveName, string FileList)
		{
			UnAceFnc.sACETestStruc
				Test = new UnAceFnc.sACETestStruc();			// set up struc (all fields initialized to zero)

			Test.Files.SourceDir	= "";		// archive main directory is base directory for FileList
			Test.Files.FileList		= FileList;	// set FileList
			Test.Files.ExcludeList	= "";		// no files to exclude
			Test.Files.FullMatch	= false;	// also test files partially matching

			Test.DecryptPassword	= "";		// no encrypted file expected

			OutFunctionCall("ACETest");

			return OutFunctionResult("ACETest", UnAceFnc.ACETest(ArchiveName, ref Test));;
		}


		public static bool CallAceExtract(string ArchiveName, string FileList, string DestinationDir, int DoExcludePath)
		{
			UnAceFnc.sACEExtractStruc
				Extract = new UnAceFnc.sACEExtractStruc();		// set up struc (all fields initialized to zero)

			Extract.Files.SourceDir      = "";					// archive main directory is base dir for FileList
			Extract.Files.FileList       = FileList;			// set FileList
			Extract.Files.ExcludeList    = "";					// no files to exclude
			Extract.Files.FullMatch      = false;				// also extract files partially matching

			Extract.DestinationDir       = DestinationDir;		// directory to extract to
			Extract.ExcludePath          = DoExcludePath;		// extract files with path?
			Extract.DecryptPassword      = "";					// no encrypted file expected						
			
			OutFunctionCall("ACEExtract");

			return OutFunctionResult("ACEExtract", UnAceFnc.ACEExtract(ArchiveName, ref Extract));
		}


		public static bool CallAceDelete(string ArchiveName, string FileList)
		{
			AceFnc.sACEDeleteStruc
				Delete = new AceFnc.sACEDeleteStruc();		// set up struc (all fields initialized to zero)

			Delete.Files.SourceDir  = "";       // archive main directory is base directory for FileList
			Delete.Files.FileList   = FileList; // set FileList
			Delete.Files.ExcludeList= "";       // no files to exclude
			Delete.Files.FullMatch  = false;	// also delete files partially matching

			Delete.DecryptPassword  = "";       // no encrypted file expected

			// required for solid archives for recompression:
			Delete.CompressParams.Level					= Strucs.ACE_LEVEL_BEST;// compress best
			Delete.CompressParams.Dictionary			= 20;					// 1024k dictionary
			Delete.CompressParams.V20Compression.DoUse	= true;					// use all v2.0 techniques

			Delete.TimeSet								= AceFnc.ACE_SETTIME_MODIFY;  // set archive time to current time

			OutFunctionCall("ACEDelete");

			return OutFunctionResult("ACEDelete", AceFnc.ACEDelete(ArchiveName, ref Delete));
		}


		public static bool CallAceAdd(string ArchiveName, string FileList, string DestinationDir, int AddMode)
		{
			AceFnc.sACEAddStruc
				Add = new AceFnc.sACEAddStruc();		// set up struc (all fields initialized to zero)

			Add.Files.SourceDir      = "";				// current directory is base directory for FileList
			Add.Files.FileList       = FileList;		// set FileList
			Add.Files.ExcludeList    = "";				// no files to exclude
			Add.Files.FullMatch      = true;			// filenames must fully match
			Add.Files.RecurseSubDirs = false;			// no recursive matching

			Add.DestinationDir       = DestinationDir;	// archive directory to add to
			Add.Mode                 = AddMode;			// add, move, fresh or update
			Add.SavePath             = AceFnc.ACE_SAVEPATH_RECURSED; // store paths recursively
			Add.EncryptPassword      = "";				// do not encrypt files
			Add.DecryptPassword      = "";				// no encrypted files expected

			Add.CompressParams.Level                = Strucs.ACE_LEVEL_BEST; // compress best
			Add.CompressParams.Dictionary           = 20;					// 1024k dictionary
			Add.CompressParams.V20Compression.DoUse = true;					// use all v2.0 techniques

			Add.VolumeSize           = AceFnc.ACE_VOLUME_AUTO; // auto-detect volume size

			Add.MainComment          = true;               // add main comment

			SetCommentBuf("Hello World!");

			Add.Solid                = true;               // create solid archive
			Add.Lock                 = false;               // lock archive
			Add.RecoveryRecord       = false;               // don't add recovery record
			Add.SFXName              = AceFnc.ACE_SFX_DEFAULT;			// add no SFX module
			Add.TimeSet              = AceFnc.ACE_SETTIME_MODIFY;	// set archive time to current time

			OutFunctionCall("ACEAdd");

			return OutFunctionResult("ACEAdd", AceFnc.ACEAdd(ArchiveName, ref Add));
		}


		public static bool CallAceRepair(string ArchiveName, string FileList)
		{
			AceFnc.sACERepairStruc
				Repair = new AceFnc.sACERepairStruc();		// set up struc (all fields initialized to zero)

			Repair.RepairedArchiveName = "REPAIRED.ACE";	// set name for repaired archive

			OutFunctionCall("ACERepair");

			return OutFunctionResult("ACERepair", AceFnc.ACERepair(ArchiveName, ref Repair));
		}


		public static bool CallAceSetComments(string ArchiveName, string FileList)
		{
			AceFnc.sACESetCommentsStruc
				SetComments = new AceFnc.sACESetCommentsStruc();	// set up struc (all fields initialized to zero)

			SetComments.Files.SourceDir      = "";		// archive main directory is base directory for FileList

														//change file comments if files are specified, otherwise change main comment
			SetComments.Files.FileList       = FileList!="" ? FileList : null;

			SetComments.Files.ExcludeList    = "";		// no files to exclude
			SetComments.Files.FullMatch      = false;		// also change comments of files partially matching

			SetComments.TimeSet = AceFnc.ACE_SETTIME_MODIFY;  // set archive time to current time

			OutFunctionCall("ACESetComments");

			return OutFunctionResult("ACESetComments", AceFnc.ACESetComments(ArchiveName, ref SetComments));
		}


		public static bool CallAceEncryptFiles(string ArchiveName, string FileList)
		{
			AceFnc.sACEEncryptFilesStruc
				EncryptFiles = new AceFnc.sACEEncryptFilesStruc();	// set up struc (all fields initialized to zero)

			EncryptFiles.Files.SourceDir   = "";				// current directory is base directory for FileList
			EncryptFiles.Files.FileList    = FileList;			// set FileList
			EncryptFiles.Files.ExcludeList = "";				// no files to exclude
			EncryptFiles.Files.FullMatch   = false;				// also encrypt files partially matching

			EncryptFiles.EncryptPassword   = "testpassword";	// use "testpassword" as password
			EncryptFiles.DecryptPassword   = "";				// no file expected that has to be decrypted
			EncryptFiles.TimeSet           = AceFnc.ACE_SETTIME_MODIFY; // set archive time to current time

			OutFunctionCall("ACEEncryptFiles");

			return OutFunctionResult("ACEEncryptFiles", AceFnc.ACEEncryptFiles(ArchiveName, ref EncryptFiles));
		}


		public static bool CallAceAddSFX(string ArchiveName, string FileList)
		{
			AceFnc.sACEAddSFXStruc
				AddSFX = new AceFnc.sACEAddSFXStruc();			// set up struc (all fields initialized to zero)

			AddSFX.SFXArchiveName = AceFnc.ACE_SFX_DEFAULT;		// receives name of SFX
			AddSFX.TimeSet        = AceFnc.ACE_SETTIME_MODIFY;	// set archive time stamp to current time

			OutFunctionCall("ACEAddSFX");

			return OutFunctionResult("ACEAddSFX", AceFnc.ACEAddSFX(ArchiveName, ref AddSFX));
		}

		
		public static bool CallAceLock(string ArchiveName, string FileList)
		{
			AceFnc.sACELockStruc
				Lock = new AceFnc.sACELockStruc();		// set up struc (all fields initialized to zero)

			Lock.TimeSet = AceFnc.ACE_SETTIME_MODIFY;	// set archive time to current time

			OutFunctionCall("ACELock");

			return OutFunctionResult("ACELock", AceFnc.ACELock(ArchiveName, ref Lock));
		}

		
		public static bool CallAceAddRecoveryRecord(string ArchiveName, string FileList)
		{
			AceFnc.sACEAddRecoveryRecordStruc
				AddRecoveryRecord = new AceFnc.sACEAddRecoveryRecordStruc();	// set up struc (all fields initialized to zero)

			AddRecoveryRecord.TimeSet = AceFnc.ACE_SETTIME_MODIFY;				// set archive time to current time

			OutFunctionCall("ACEAddRecoveryRecord");

			return OutFunctionResult("ACEAddRecoveryRecord", AceFnc.ACEAddRecoveryRecord(ArchiveName, ref AddRecoveryRecord));
		}
	}
}
