using System;
using System.Runtime.InteropServices;

namespace AceDllWrapped
{
	/// <summary>
	/// 
	/// Declarations for C# ACE-Dll wrapper
	/// 
	/// </summary>

	//-----------------------------------------------------------------------------
	//----------------          Part 1: different structures        ---------------
	//-----------------------------------------------------------------------------

	//---------------------------------------------------------------------------
	//
	// Herein different structures used at callback functions and
	// AceV2.Dll functions are declared.
	//
	//---------------------------------------------------------------------------
	//
	// Contents:
	//   Part 1.1: structures used in callback structures
	//   Part 1.2: structures used in function structures
	//
	//---------------------------------------------------------------------------

	//---------------------------------------------------------------------------
	//========---  Part 1.1: structures used in callback structures  ---=========
	//---------------------------------------------------------------------------

	//-----------------------  comment buffer structure  ----------------------
	// Used in sACEGlobalDataStruc. Application has to specify where the
	// comment is or should be stored.
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACECommentStruc
	{
		public IntPtr Buf;			// area where the comments are stored by the application 
									// or by the Dll comments must be zero-terminated
		public uint BufSize;		// size of this area
		public uint State;			// state of comment - see ACE_COMMENT constants below
	}


	//-----------------------  global data structure  -------------------------
	// This structure contains information for the Dll being interesting for
	// nearly all functions. The Dll has to be initialized with this
	// structure passed to sACEInitDll(). This structure is also passed
	// by the callback functions.
	//-------------------------------------------------------------------------

	//---------------------------------------------------------------
	// Forward declarations of callback function parameters
	// used in this structure.
	//---------------------------------------------------------------

	public delegate int dACEInfoCallback(IntPtr Info);

	public delegate int dACEErrorCallback(IntPtr Error);
		
	public delegate int dACERequestCallback(IntPtr Request);
		
	public delegate int dACEStateCallback(IntPtr State);


	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEGlobalDataStruc
	{
		public uint Obj;					// ---- reserved for application! ----
											// thought to be used as a pointer to an object;
											// when a callback-function is called, the object
											// pointer can be used to handle the callback in a
											// specific way; the pointer has to be initialized
											// by ACEInitDll()
			
		public uint MaxArchiveTestBytes;	// how many bytes of a file should be
											// looked upon at archive header search?

			
		public uint MaxFileBufSize;			// maximum buffer size for buffered I/O operations

		public sACECommentStruc Comment;    // used to exchange comment data
											// between application and Dll
											// using callback functions

		public string DecryptPassword;		// the DecryptPassword specified at
											// ACEInitDll() is overwritten by the
											// DecryptPassword field of sACEAddStruc and
											// other function-specific structures;
											// but this field can be used to change the
											// password at callback function calls

		public bool UseVBStructures;		//  passes structures to callback functions
											//  much better suited for Visual Basic

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=60)]
		public byte[] Reserved1;			// has to be filled with zeros

		//--------------------------- fields for ACE only ---------------------

		public int EncryptPassword;			// things stated at the description of the
											// DecryptPassword field apply here as well

		public string TempDir;				// directory to save temporary archive

		//-------- registration (ACE DLLs only, but not used at ACL) ----------

		public int KeyPath;					// necessary for reading and writing key file

		public int UserAV;					// Dll returns the AV string (if registered) in this field

		public int IsGeneralKey;			// DLL returns the key, if it is a general key

		public int OwnerWindow;				// specifies the applications window to be
											// parent of the registration reminder dialog

		//--------------------------- fields for ACE only ---------------------

		public int CompressionLevel;		// contains the currently used compression level
											//  - may be changed during compression	operation

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=56)]
		public byte[] Reserved2;			// has to be filled with zeros


		//----------------------  callback functions (ACE and UnACE) ----------

		public dACEInfoCallback InfoCallbackProc;		//  Informs application about details like 
														//  the creation of a temporary archive.
														//  See ACE_CALLBACK_INFO definitions for
														//  more information.

		public dACEErrorCallback ErrorCallbackProc;		//  Called to let the application know about
														//  errors. The Code field of Error will
														//  contain an ACE_CALLBACK_ERROR value.

		public dACERequestCallback RequestCallbackProc;	//  Called to let the application know about
														//  requests for data. Code will contain an
														//  ACE_CALLBACK_REQUEST value.

		public dACEStateCallback StateCallbackProc;		//  Called to let the application know about
														//  the progress of operations.

		//-------------------------- different new fields ---------------------
		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved3;			// has to be filled with zeros
	
	}


	//-----------------------  archive data structure  ------------------------
	// Informs the callback functions about the current archive, its volume
	// number, the archive-flags (see ACE_FLAG constants), the creation host
	// system (see ACE_HOST constants) and the AV if present in archive.
	// Also used at ACEReadArchiveData().
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEArchiveDataStruc
	{
		[MarshalAs(UnmanagedType.LPStr)] 
		public string ArchiveName;
			
		public uint 
			VolumeNumber,
			Flags,				// see ACE_ARCFLAG defines below
			HostCreated,		// see ACE_HOST defines below
			TimeCreated,		// in MS-DOS format
			VersionCreated,
			VersionExtract;		// version needed to extract files

		string AV;				// not used in ACL

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved;   // filled with zeros
	}


	//------------------------  file data structure  --------------------------
	// Contains information about an archived file.
	//-------------------------------------------------------------------------


	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEFileDataStruc
	{
		public string SourceFileName;		// relative file name
		public string DestinationFileName;	// absolute file name;
											// valid for add and extract only!

		public uint 
			Flags,				// see ACE_FILEFLAG defines below
			CRC32,
			Method,             // 0=stored, 1=LZ77, 2=V20Compression
			Dictionary;         // DictionarySize = 2^Dictionary

		public ulong
			CompressedSize,
			Size;

		public uint 
			Time,
			Attributes;

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved;        // filled with zeros
	}
			

	//------------------------  copy info structure  --------------------------
	// Is passed to ACEInfoCallbackProc with ACE_CALLBACK_INFO_COPY as code.
	// Informs application about the progress of copying either an archive to
	// a temporary archive, or a temporary archive back to a normal archive.
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACECopyInfoStruc
	{
		public string SourceFileName;		// source file
		public string DestinationFileName;	// the file copying the source to
		
		public ulong  CopiedBytes;			// bytes already copied
		public ulong  FileSize;				// source file size

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved;				// filled with zeros
	}


	//--------------------  operation progress structure  ---------------------
	// Used to state the progress of the current operation.
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEProgressDataStruc
	{
		public IntPtr Addr;					// address of uncompressed data block
		public uint Size;					// size of uncompressed data block
		
		public ulong 
			TotalProcessedSize,				// counted by Dll: total uncompressed bytes processed
			TotalCompressedSize,			// total compressed bytes processed
			TotalSize,						// total uncompressed bytes to process (sum of all original file sizes)
			FileProcessedSize,				// uncompr. bytes of file processed
			FileCompressedSize,				// compr. bytes of file processed
			FileSize;						// uncompressed file size
	}


	//---------------------------------------------------------------------------
	//========---  Part 1.2: structures used in function structures  ---=========
	//---------------------------------------------------------------------------

	//-------------------------  file list structure  -------------------------
	// This structure is used in the function specific structures.
	// The application has to use this structure to indicate which files
	// have to be processed by the DLL.
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEFilesStruc
	{
		public string  SourceDir;           // maybe a real or an archive directory

		public string  FileList;            // pointer to list of files to process;
											// zero-terminated; files have to be
											// separated by carriage-return (0xd);
											// FileList may/will be modified by the
											// Dll; if nothing is specified, "*"
											// will be used (attention at ACEDelete!!)

		public string  ExcludeList;         // list of files to exclude from process
		
		public bool FullMatch;				// specifications must fully match
											// (no files with the same name in
											//  subdirs are processed)

		public bool RecurseSubDirs;			// recurse subdirectories
											// (valid for add operations only)

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=60)]
		public byte[] Reserved ;			// has to be filled with zeros
											// for future: possibly in-/excluding
											// file attributes and date/time range
	}


	//---------------------  V2.0 compression structure -----------------------
	// Specifies whether to use v2.0 compression or not. If you use v2.0
	// compression you can also specify which v2.0 compression techniques
	// you want to use. (ACE only)
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACEV20CompressionStruc
	{
		public bool
			DoUse,					// if DoUse=1 and all other fields are
			DoUseDelta,				// zero, then all v2.0 compression
			DoUseExe,				// techniques will be used
			DoUsePic,
			DoUseSound;

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved;		// has to be filled with zeros
	}


	//-------------------  compression parameter structure  -------------------
	// Used in sACEAddStruc and sACEDeleteStruc. (ACE only)
	//-------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public struct sACECompressParamsStruc
	{
		public uint Level;					// see ACE_LEVEL constants below
		public uint Dictionary;				// 15(32k)..22(4Mb)

		public sACEV20CompressionStruc		// indicates if (or which) v2.0
			V20Compression;					// compression techniques shall be used

		public bool TestAfter;				// make a test for CRC check errors after compression

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
		public byte[] Reserved;				// has to be filled with zeros
	}


	//--------------  definition of constants used in structures  -------------

	public class Strucs
	{
		//-----------------------------------------------------------------------
		// Defines for the sACECommentStruc.State field, which tells about
		// the result of the last comment extraction.
		//-----------------------------------------------------------------------

        public const int ACE_COMMENT_OK         = 0;    // comment extraction went fine

        public const int ACE_COMMENT_SMALLBUF   = 1;    // comment buffer too small to
														// store the whole comment in

        public const int ACE_COMMENT_NONE       = 255;  // No comment present


		//-----------------------------------------------------------------------
		// Flag constants for sACEArchiveDataStruc.Flags field.
		//-----------------------------------------------------------------------

        public const int ACE_ARCFLAG_MAINCOMMENT        = 2;
        public const int ACE_ARCFLAG_SFX                = 512;
        public const int ACE_ARCFLAG_LIMITSFXJR         = 1024;		// an SFX archive that supports
																	// 256k	dictionary only
        public const int ACE_ARCFLAG_MULTIVOLUME        = 2048;
        public const int ACE_ARCFLAG_AV                 = 4096;		// not used in ACL
        public const int ACE_ARCFLAG_RECOVERYREC        = 8192;
        public const int ACE_ARCFLAG_LOCK               = 16384;
        public const int ACE_ARCFLAG_SOLID              = 32768;


		//-----------------------------------------------------------------------
		// Host system used to create an archive. Used at
		// sACEArchiveDataStruc.HostCreated field.
		//-----------------------------------------------------------------------

        public const int ACE_HOST_MSDOS                 = 0;		// archive created by
																	// MSDOS ACE archiver
        public const int ACE_HOST_OS2                   = 1;		// created by OS2 ACE
        public const int ACE_HOST_WIN32                 = 2;		// created by Win32 ACE
		

		//-----------------------------------------------------------------------
		// Flag constants for the sACEFileData.Flags field.
		//-----------------------------------------------------------------------

        public const int ACE_FILEFLAG_FILECOMMENT       = 2;		// file has comment
        public const int ACE_FILEFLAG_SPLITBEFORE       = 4096;		// continued from previous volume
        public const int ACE_FILEFLAG_SPLITAFTER        = 8192;		// continued on next volume
        public const int ACE_FILEFLAG_PASSWORD          = 16384;	// is encrypted
        public const int ACE_FILEFLAG_SOLID             = 32768;	// uses data of previous
																	// files (solid mode)

		//-----------------------------------------------------------------------
		// Tells the Dll which compression level to use. (ACE only)
		//-----------------------------------------------------------------------

        public const int ACE_LEVEL_STORE    = 0;	// save file only; do not compress
        public const int ACE_LEVEL_FASTEST  = 1;	// compress very fast
        public const int ACE_LEVEL_FAST     = 2;	// compress fast
        public const int ACE_LEVEL_NORMAL   = 3;	// good compromise between speed and compression
        public const int ACE_LEVEL_GOOD     = 4;	// achieves good compression
        public const int ACE_LEVEL_BEST     = 5;	// best compression; bit slow

	}
}
