/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 21/11/2006
 * Time: 22:47
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;

using Cake3.SharpZipLib;
using Cake3.IconLib;
using Cake3.Cmarc;
using Cake3.Ace;
using Cake3.Sqx;
using Cake3.Rar;
using Cake3.InstallBuild;
using Cake3.WcxPlugins;

namespace Cake3
{
	/// <summary>
	/// Main unit of Cake3, provide support for all archive operations.
	/// </summary>
	public class Cakdir3 
	{
		void prepareArchiver()
		{
			archiverList.Add(new AceArchiver());
			archiverList.Add(new RarArchiver());
			archiverList.Add(new SqxArchiver());
			archiverList.Add(new SZLArchiver());			
			archiverList.Add(new ICLArchiver());			
			archiverList.Add(new WcxArchiver());
			archiverList.Add(new InstBuildArchiver());
			archiverList.Add(new CmarcArchiver());			
		}
		
		CakArchiver archiver;
		string archiveName;
		bool internalList = true;
		bool applytoall;  //Extract related.
		bool overwrite;   //Extract related.
		bool firstRun;    //Extract related.		
		List<CakArchiver> archiverList = new List<CakArchiver>();
		
		
		private ContentList contents = new ContentList();
		
		/// <summary>
		/// Occurs when the Listing an archive.
		/// </summary>
		public ItemListEventHandler OnItemList;
		/// <summary>
		/// Occurs when a message received.
		/// </summary>
		public MessageEventHandler OnMessage;
		/// <summary>
		/// Occurs when there is an error.
		/// </summary>
		public ErrorEventHandler OnError;	
		/// <summary>
		/// Occurs when progress update.
		/// </summary>
		public ProgressEventHandler OnProgress;
		/// <summary>
		/// Occurs when confirm overwrite.
		/// </summary>
		public OverwriteEventHandler OnOverwrite;
		public static OverwriteEventHandler OnOverwriteDefault;
		/// <summary>
		/// Occurs when request password.
		/// </summary>
		public PasswordEventHandler OnPassword;				
		public static PasswordEventHandler OnPasswordDefault;		
		/// <summary>
		/// Occurs when start archive work.
		/// </summary>
		public EventHandler OnStartWorking;
		/// <summary>
		/// Occurs when finished archive work.
		/// </summary>
		public EventHandler OnStopWorking;
		
		/// <summary>
		/// Gets or sets the options for extract.
		/// </summary>
		public ExtractOptions ExtractOptions;
		/// <summary>
		/// Gets or sets the options for add.
		/// </summary>
		public AddOptions AddOptions;
		/// <summary>
		/// Gets or sets the options for delete.
		/// </summary>
		public DeleteOptions DeleteOptions;
		
		public SFXOptions SFXOptions;
		
		/// <summary>
		/// Gets the archivename. 
		/// If you wish to open another archive, create a new instance of Cakdir instead.
		/// </summary>
		public string ArchiveName
		{
			get
			{
				return archiveName;
			}
		}
		
		/// <summary>
		/// Gets the content of archive after List() is called, only valid if InternalList is set to true.
		/// </summary>
		public ContentList Archive_Contents 
		{
			get 
			{
				if (!internalList)
					throw new Exception("Internal Listing is not enabled!");
				return contents;
			}
				
		}
		
		/// <summary>
		/// Gets or Sets whether to use the internal contentlist when listing. (Archive_Contents)
		/// </summary>
		public bool InternalList
		{
			get { return internalList; }
			set { internalList = value; }
		}
		
		/// <summary>
		/// Gets whether cake3 can list this archive.
		/// </summary>
		public bool CanList
		{
			get { return (archiver != null) && (archiver.CanList); }
		}
		/// <summary>
		/// Gets whether cake3 can extract this archive.
		/// </summary>
		public bool CanExtract
		{
			get { return (archiver != null) && (archiver.CanExtract); }
		}
		/// <summary>
		/// Gets whether cake3 can add file to this archive.
		/// </summary>
		public bool CanAdd
		{
			get { return (archiver != null) && (archiver.CanAdd); }
		}
						
		CakArchiver locateArchiver(string anArchive)
		{
			string arcExt = Utils.ExtractFileExt(anArchive).ToLower();				
			
			foreach (CakArchiver arc in archiverList)
			{
				if (arc.Support(anArchive))
					return arc;
			}
			
//			foreach (ArchiveType at in SupportedArchiveTypes())
//			{				
//				if (at.SupportedExtensions.IndexOf(arcExt) != -1) //Support
//					if (locateArchiverName(at.Archiver).Support(anArchive))
//						return locateArchiverName(at.Archiver);
//			}
//			
			//If not found, try use SupportedExtensions list instead.
			foreach (CakArchiver arc in archiverList)
			{
				foreach (string ext in arc.SupportedExtensions())
				{
					if (arcExt == ext)
						return arc;
				}
			}
			
			//Not supported...
			return null;
		}
		
		CakArchiver locateArchiverName(string anArchiverName)
		{			
			foreach (CakArchiver arc in archiverList)
			{
				if (arc.name == anArchiverName)
					return arc;
			}
			return null;
		}
		
		/// <summary>
		/// Gets a list of archive type extension. 
		/// All extension start with a dot (e.g. ".zip")
		/// </summary>
		public static List<string> SupportedArchives()
		{
			Cakdir3 cakdir = new Cakdir3();			
			List<string> retVal = new List<string>();
						
			foreach (ArchiveType at in SupportedArchiveTypes())
			{
				string[] exts = at.Extensions.Split(',');
				foreach (string ext in exts)
					if (retVal.IndexOf("." + ext) == -1)
					retVal.Add("." + ext);
			}
					
			retVal.Sort();
			return retVal;
		}
		
		public static List<ArchiveType> SupportedArchiveTypes()
		{
			Cakdir3 cakdir = new Cakdir3();			
			List<ArchiveType> retVal = new List<ArchiveType>();
			
			foreach (CakArchiver arc in cakdir.archiverList)
			{
				retVal.AddRange(arc.SupportedArchiveTypes());
			}
			
			retVal.Sort();
			return retVal;
		}
		
		public static ArchiveType GetArchiveType(string extension)
		{
			extension = "." + extension.Trim('.');
			
			foreach (ArchiveType at in SupportedArchiveTypes())
				if (at.SupportedExtensions.IndexOf(extension) != -1)
					return at;
			
			return null;
		}
		
		public static string OpenFilter(out string[] ArchiverNameList)
		{			
			string retAll = "All Readable files|";
			string retVal = "";
			List<string> retArcList = new List<string>();
			retArcList.Add("");
			
			foreach (ArchiveType at in SupportedArchiveTypes())
			{
				if (at.CanList)
				{
					//Remove Duplicates
					foreach (string ext in at.ExtensionMasks.Split(';'))
					   if (retAll.IndexOf(ext + ";") == -1)
						retAll += ext + ";";					
					
					retVal += at.Filter;
					retArcList.Add(at.Archiver);	
				}
			}
			
			ArchiverNameList = retArcList.ToArray();			
			return retAll.Trim(';') + retVal;
		}		
		
		void setHandler()
		{
			archiver.OnItemList = new ItemListEventHandler(Cakdir3_ItemList);
			archiver.OnMessage = new MessageEventHandler(Cakdir3_Message);
			archiver.OnError = new ErrorEventHandler(Cakdir3_Error);
			archiver.OnProgress = new ProgressEventHandler(Cakdir3_Progress);
			archiver.OnOverwrite = new OverwriteEventHandler(Cakdir3_Overwrite);
			archiver.OnPassword = new PasswordEventHandler(Cakdir3_Password);
		}
		
		
		void Cakdir3_ItemList(Object sender, ItemListEventArgs e)
		{
			if (internalList)
				contents.Add(e.Content);
			else
				if (OnItemList != null)
					OnItemList(sender, e);
		}
		
		void Cakdir3_Message(Object sender, MessageEventArgs e)
		{
			if (OnMessage != null)
				OnMessage(sender, e);
		}
		
		void Cakdir3_Error(Object sender, ErrorEventArgs e)
		{
			if (OnError != null)
				OnError(sender, e);
		}
		
		void Cakdir3_Progress(Object sender, ProgressEventArgs e)
		{
			if (OnProgress != null)
				OnProgress(sender, e);
		}
			
		void Cakdir3_Overwrite(Object sender, ref OverwriteEventArgs e)
		{
			string dest;
			if (ExtractOptions.allowFolder)
				dest = Utils.AppendSlash(ExtractOptions.ExtractFolderParsed) + e.Filename;
			else dest = Utils.AppendSlash(ExtractOptions.ExtractFolderParsed) +
				Utils.ExtractFileName(e.Filename);
			
			//UPD: 08-05-07 ExtractOptions.overwriteMode support
			switch (ExtractOptions.extractOverwrite)
			{
				case ExtractOptions.overwriteMode.overwrite:
					#region Always overwrite
					e.Overwrite = true;
					#endregion
					break;
				case ExtractOptions.overwriteMode.skip:
					e.Overwrite = false;
					break;
				case ExtractOptions.overwriteMode.compareDate:
					#region Compare file time					
					if (Utils.GetFileInfo(dest).LastWriteTime < Archive_Contents[e.Filename].fileTime)
						e.Overwrite = true;
					#endregion
					break;
				case  ExtractOptions.overwriteMode.renameOld:
					#region Rename old file
					int dummy = 0;
					while (Utils.FileExists(dest + "." + dummy.ToString()))
						dummy++;
					Utils.MoveFile(dest, dest + "." + dummy.ToString());
					e.Overwrite = true;
					#endregion
					break;
				
				case ExtractOptions.overwriteMode.ask:
					if (OnOverwrite != null)
						OnOverwrite(sender, ref e);
					else
						if (Cakdir3.OnOverwriteDefault != null)
						Cakdir3.OnOverwriteDefault(sender, ref e);
					else //Internal handler
					{
						e.Overwrite = true;
						if (Archive_Contents.TotalCount == 0)
							List();
						ContentType ct = Archive_Contents[e.Filename];
						if (ct.fileName != "")
						{														
							if (!applytoall)
								Utils.AskOverwrite(ct, dest, ref overwrite);
														
							if ((firstRun) && (Archive_Contents.TotalCount > 1))
								if (overwrite)
								applytoall = (MessageBox.Show("Overwrite All?", "Overwrite", MessageBoxButtons.YesNo) == DialogResult.Yes);
							else applytoall = (MessageBox.Show("Skip All?", "Overwrite", MessageBoxButtons.YesNo) == DialogResult.Yes);
							firstRun = false;
							
							e.Overwrite = overwrite;
						}
					}
				break;
			}
		}  
		
		//UPD: 08-05-07 OnPassword handle first if assigned, then OnPasswordDefault.
		void Cakdir3_Password(Object sender, ref PasswordEventArgs e)
		{
			if (OnPassword != null)
				OnPassword(sender, ref e);
			else
			if (Cakdir3.OnPasswordDefault != null)
				Cakdir3.OnPasswordDefault(sender, ref e);					
		}
		
		Cakdir3()
		{
			prepareArchiver();			
		}
		
		void initArchiver(string anArchive, string anArchiverName)
		{
			prepareArchiver();
			archiveName = anArchive;
			
			if (anArchiverName != "")
				archiver = locateArchiverName(anArchiverName);
			if (archiver == null)
				archiver = locateArchiver(anArchive);			
			if (archiver == null)
				throw new NotSupportedException(anArchive + " is not supported.");
			
			Debug.WriteLine(archiver.name + " loaded.");
			archiver.ArchiveName = anArchive;
			
			ExtractOptions = new ExtractOptions(anArchive);
			AddOptions = new AddOptions(anArchive);
			DeleteOptions = new DeleteOptions(anArchive);
			SFXOptions = new SFXOptions(anArchive);
			
			setHandler();
		}
		
		/// <summary>
		/// Initializes a new instance of the Cakdir3 class. 
		/// </summary>
		/// <param name="anArchive">Archive to open.</param>
		public Cakdir3(string anArchive)
		{
			initArchiver(anArchive, "");					
		}
		
		public Cakdir3(string anArchive, string anArchiverName)
		{
			initArchiver(anArchive, anArchiverName);
		}
		
		void startWorking()
		{
			if (OnStartWorking != null)
				OnStartWorking(this, new EventArgs());
		}

		void stopWorking()
		{
			if (OnStopWorking != null)
				OnStopWorking(this, new EventArgs());
		}
				
		
		/// <summary>
		/// Delete file(s) from archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public bool Delete()
		{
			startWorking();
			bool retVal = archiver.Delete(DeleteOptions);
			stopWorking();
			return retVal;
		}
		/// <summary>
		/// Delete file(s) from archive.
		/// </summary>
		/// <param name="deleteFiles">File(s) to be deleted.</param>
		/// <returns>Whether operation successful.</returns>
		public bool Delete(string[] deleteFiles)
		{
			DeleteOptions.deleteFile = deleteFiles;
			return Delete();
		}
		/// <summary>
		/// Delete file(s) from archive.
		/// </summary>
		/// <param name="deleteFile">File to be deleted.</param>
		/// <returns>Whether operation successful.</returns>
		public bool Delete(string deleteFile)
		{
			DeleteOptions.deleteFile = new String[1] {deleteFile};
			return Delete();
		}
		
		public bool SFX()
		{
			return archiver.SFX(SFXOptions);
		}
				
		bool confirmOverwrite(string filename)
		{
			OverwriteEventArgs oe = new OverwriteEventArgs(archiver, filename);
			Cakdir3_Overwrite(archiver, ref oe);
			return oe.Overwrite;			
		}
		
		void confirmOverwrite()
		{
			if (!archiver.CanAlarmOverwrite)
			{
				if (Archive_Contents.TotalCount == 0)
					List();
				ContentList cloneList = (ContentList)Archive_Contents.Clone();
				foreach (string item in ExtractOptions.extractItem)
				{
					cloneList.Mask = item;					
					foreach (ContentType ct in cloneList)
					{
						string dest;
						if (ExtractOptions.allowFolder)
							dest = Utils.AppendSlash(ExtractOptions.extractFolder) + ct.fileName;
						else dest = Utils.AppendSlash(ExtractOptions.extractFolder) + 
									Utils.ExtractFileName(ct.fileName);
						
						if (Utils.FileExists(dest))
							if (confirmOverwrite(ct.fileName))
								Utils.RemoveFile(dest);
					}
				}
			}
		}
		/// <summary>
		/// Extract file(s) from archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public bool Extract()
		{
			startWorking();	
			applytoall = false;
			overwrite = false;
			firstRun = true;
			confirmOverwrite();
			bool retVal = archiver.Extract(ExtractOptions);
			stopWorking();
			return retVal;
		}
		
		public bool Extract(string extractItem, string extractTo, bool allowFolder, bool allowOverwrite)
		{
			ExtractOptions.extractItem = new string[] { extractItem };
			ExtractOptions.allowFolder = allowFolder;
			ExtractOptions.allowOverwrite = allowOverwrite;
			ExtractOptions.extractFolder = extractTo;
			return Extract();
		}
		
		public bool Extract(string extractItem, string extractTo, string password, bool allowFolder, bool allowOverwrite)
		{
			ExtractOptions.extractItem = new string[] { extractItem };
			ExtractOptions.allowFolder = allowFolder;
			ExtractOptions.allowOverwrite = allowOverwrite;
			ExtractOptions.password = password;
			ExtractOptions.extractFolder = extractTo;
			return Extract();
		}				
		
		/// <summary>
		/// Add file(s) to archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public bool Add()
		{
			startWorking();
			bool retVal = true;
			if (AddOptions.addMode != AddOptions.updateMode.add)
			{				
				List("*");
				string[] DelFile;
				AddOptions = Utils.ProcessAddMode(this.Archive_Contents, AddOptions, out DelFile);
				
//				foreach (string df in DelFile)
//					Debug.WriteLine("Del - " + df);
//				foreach (string af in AddOptions.addFile)
//					Debug.WriteLine("Add - " + af);				
				
				retVal = this.Delete(DelFile);
			}
			if (retVal) retVal = archiver.Add(AddOptions);
			stopWorking();
			return retVal;
		}		
		
		public bool Add(string addFile, AddOptions.folderMode addFolder, string baseFolder)
		{
			AddOptions.addFile = new string[] { addFile };
			AddOptions.addFolder = addFolder;
			AddOptions.baseFolder = baseFolder;
			return Add();
		}
		
		/// <summary>
		/// Add files in AddOptions.addFile to specified folder in archive.
		/// </summary>
		/// <param name="folder">Folder in archive.</param>
		/// <returns></returns>

		public bool AddToFolder(string folder)
		{			
			if ((folder == "") || (folder == @"/") || (folder == @"\"))
				return Add();
			
			startWorking();
			
			folder = Utils.AppendSlash(folder);
			string tempPath = Utils.NewTempPath("qzTemp");			
			AddOptions userAddOptions = (AddOptions)AddOptions.Clone();
			
			for (int i = 0; i < AddOptions.addFile.Length; i++)
			{
				string filename = AddOptions.addFile[i];
				string relativePath = "";
				switch (AddOptions.addFolder)
				{
					case AddOptions.folderMode.none:
						relativePath = Utils.ExtractFileName(filename);
						break;
					case AddOptions.folderMode.full:
						relativePath = Utils.RemoveFileDrive(filename);
						break;
					case AddOptions.folderMode.relative:
						if (filename.ToLower().IndexOf(AddOptions.baseFolder.ToLower()) != 0)
							relativePath = Utils.RemoveFileDrive(filename); //FullPath
						else
							relativePath = Utils.RemoveFrontSlash(filename.Substring(AddOptions.baseFolder.Length));
						break;					
				}
				
				AddOptions.addFile[i] = tempPath + Utils.RemoveFrontSlash(folder) + relativePath;
				Utils.MakeDirectory(Utils.ExtractFilePath(AddOptions.addFile[i]));
				Utils.CopyFileMask(userAddOptions.addFile[i], Utils.ExtractFilePath(AddOptions.addFile[i]), true);
//				System.IO.File.Copy(userAddOptions.addFile[i], AddOptions.addFile[i]);
//				Debug.WriteLine(userAddOptions.addFile[i] + "--->" + AddOptions.addFile[i]);
			}
			
			AddOptions.addFolder = AddOptions.folderMode.relative;
			AddOptions.baseFolder = tempPath;			
			bool retVal = Add();			
			AddOptions = userAddOptions;
			
			stopWorking();
			return retVal;
		}
		/// <summary>
		/// List the opened archive.
		/// </summary>
		/// <param name="mask">Only file that match the mask will be listed.</param>
		/// <returns>Whether operation successful.</returns>
		public bool List(string mask)
		{
			startWorking();
			if (internalList)
				contents.Clear();
			
			bool retVal = false;
			
			if (archiver.Open())
			{
				retVal = archiver.List(mask);
				archiver.Close();
			}
			stopWorking();
			return retVal;	
		}
		/// <summary>
		/// List the opened archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public bool List()
		{
			return List("*");
		}
		/// <summary>
		/// Test integrity of the archive
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public bool Test()
		{			
			startWorking();
			bool retVal = archiver.Test();				
			stopWorking();
			return retVal;
		}
		
		
	}
}
