/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 20/11/2006
 * Time: 22:19
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using Cake3;

namespace Cake3.Cmarc
{				
	/// <summary>
	/// Description of CmarcArchiver.
	/// </summary>
	public class CmarcArchiver : CakArchiver
	{
		internal override string name {	get { return "cmarc"; }}
		Cake3.Cmarc.ArchiverBase arcBase;
		IntPtr harc;
		
		public CmarcArchiver() : base()
		{			
		}
		
		internal static string ModifySlashToCak(string input)
		{
			return Utils.AppendFrontSlash( input.Replace('/','\\') );
		}
		
		internal static string ModifySlashFromCak(string input)
		{
			return Utils.RemoveFrontSlash(input).Replace('\\','/');
		}
		
		
		
		internal override void setArchiveName(string anArchive)
		{			
			base.setArchiveName(anArchive);
			
			string extension = Cake3.Utils.ExtractFileExt(anArchive).ToLower();
			if (LhaBase.SupportedExtensions().IndexOf(extension) != -1)
			{ arcBase = new LhaBase(); canAdd = true; }
			else
				if (SevenZipBase.SupportedExtensions().IndexOf(extension) != -1)
				{ arcBase = new SevenZipBase(); canAdd = true; } 
				else
					if (ArjBase.SupportedExtensions().IndexOf(extension) != -1)
						arcBase = new ArjBase();
					else
						if (BhBase.SupportedExtensions().IndexOf(extension) != -1)
							arcBase = new BhBase();
						else
							if (XacRettBase.SupportedExtensions().IndexOf(extension) != -1)
								arcBase = new XacRettBase();
							else throw new NotSupportedException();		
						
			canList = true;
			canExtract = true;			
		}
				
		public override List<ArchiveType> SupportedArchiveTypes()
		{
			List<ArchiveType> retVal = new List<ArchiveType>();
			retVal.AddRange(LhaBase.SupportedArchiveTypes(this.name));
			retVal.AddRange(SevenZipBase.SupportedArchiveTypes(this.name));
			retVal.AddRange(ArjBase.SupportedArchiveTypes(this.name));
			retVal.AddRange(BhBase.SupportedArchiveTypes(this.name));	
			retVal.AddRange(XacRettBase.SupportedArchiveTypes(this.name));
			return retVal;
		}
		
		public override bool Open()
		{ 
			if (archiveOpened) 
				return false;
			
			if (Utils.FileExists(archiveName))				
				harc = arcBase._ArchiverOpenArchive(IntPtr.Zero, archiveName, 0);
			
			archiveOpened = true;
			return true;		
		}
		
		public override bool Close()
		{
			if (!archiveOpened) 
				return false;
			
			if (harc != IntPtr.Zero)
				arcBase._ArchiverCloseArchive(harc);
			
			harc = IntPtr.Zero;
			archiveOpened = false;
			return true;
		}
				
		
		public override bool List(string mask) 
		{ 
			if(OnItemList == null) return false;
			if(harc == IntPtr.Zero) return true;
			
			DispMessage(Defines.ErrorMessage("PROCESSLIST"), archiveName);	
			
			ArchiverBase.INDIVIDUALINFO subInfo = new ArchiverBase.INDIVIDUALINFO();			
			
			if (arcBase._ArchiverFindFirst(harc, mask, ref subInfo) == 0)
			{
				//06-17-07 Hide all folder entries
				if (!subInfo.szFileName.EndsWith("\\")) //Folder
				OnItemList(this, new ItemListEventArgs(this, 
				                                       new ContentType(ArchiveName, subInfo.dwOriginalSize, subInfo.dwCompressedSize,
				                                       Cake3.Utils.DosDateToDateTime(subInfo.wDate, subInfo.wTime), subInfo.dwCRC,
				                						ModifySlashToCak(subInfo.szFileName), false)) );
				while (arcBase._ArchiverFindNext(harc, ref subInfo) == 0)
					if (!subInfo.szFileName.EndsWith("\\")) //Folder
					OnItemList(this, new ItemListEventArgs(this, new ContentType(ArchiveName, subInfo.dwOriginalSize, subInfo.dwCompressedSize,
				                						Cake3.Utils.DosDateToDateTime(subInfo.wDate, subInfo.wTime), subInfo.dwCRC, 
				                						ModifySlashToCak(subInfo.szFileName), false)) );
			}						
			
			return true;
		}
		
		public void OutputMessage(string message)
		{
			string[] output	= message.ToString().Split((Char)(10));
			foreach (string s in output)
			{
				string text = s.Trim();
				if (text != "")
					OnMessage(this, new MessageEventArgs(null, text));
			}
		}
		
		public override bool Extract(Cake3.ExtractOptions options) 
		{ 						
			DispMessage(Defines.ErrorMessage("PROCESSEXTR"), archiveName);				
			for (int i = 0; i < options.extractItem.Length; i++)
			{
				string file = options.extractItem[i];
				string command = arcBase.GetExtractCommand(options, ModifySlashFromCak(file));
				Debug.WriteLine(command);				
				if (command == "")
					return false;
				StringBuilder output = new StringBuilder(4096);
				Int32 retVal = arcBase._ArchiverExec(IntPtr.Zero, command, output, (UInt32)output.Capacity);			
				ErrorMessage ec = Defines.ErrorMessage(retVal);
				
				if ((OnProgress != null))
					OnProgress(this, new ProgressEventArgs(this, (ushort)(i/options.extractItem.Length*100), file));				
				OutputMessage(output.ToString());										
			
				if (ec.ErrorName != "NOERROR")
				{
					OnError(this, new ErrorEventArgs(this, ec));
					return false;
				}
				
			}
			return true; 
		}
		
		public override bool Add(AddOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSADD"), archiveName);	
			
			for (int i = 0; i < options.addFile.Length; i++)
			{		
				string file = options.addFile[i];
				string command = arcBase.GetAddCommand(options, file);
				Debug.WriteLine(command);
				if (command == "")
					return false;
				
				StringBuilder output = new StringBuilder(4096);
				Int32 retVal = arcBase._ArchiverExec(IntPtr.Zero, command, output, (UInt32)output.Capacity);
				ErrorMessage ec = Defines.ErrorMessage(retVal);
				
				if ((OnProgress != null))
					OnProgress(this, new ProgressEventArgs(this, (ushort)(i/options.addFile.Length*100), file));
				OutputMessage(output.ToString());
				
				if (ec.ErrorName != "NOERROR")
				{
					OnError(this, new ErrorEventArgs(this, ec));
					return false;
				}
			}
			
			return true;
		}
		
		public override bool Delete(DeleteOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSDEL"), archiveName);	
			foreach (string file in options.deleteFile)
			{				
				string command = arcBase.GetDeleteCommand(options, file);
				Debug.WriteLine(command);
				if (command == "")
					return false;
				
				StringBuilder output = new StringBuilder(4096);
				Int32 retVal = arcBase._ArchiverExec(IntPtr.Zero, command, output, (UInt32)output.Capacity);
				ErrorMessage ec = Defines.ErrorMessage(retVal);
				
				OutputMessage(output.ToString());
				
				if (ec.ErrorName != "NOERROR")
				{
					OnError(this, new ErrorEventArgs(this, ec));
					return false;
				}
			}
			
			return true;
		}
		
		public override bool Test()
		{
			DispMessage(Defines.ErrorMessage("PROCESSTEST"), archiveName);	
			string command = arcBase.GetTestCommand(archiveName);
			if (command == "")
				return false;
			StringBuilder output = new StringBuilder(4096);
			Int32 retVal = arcBase._ArchiverExec(IntPtr.Zero, command, output, (UInt32)output.Capacity);
			ErrorMessage ec = Defines.ErrorMessage(retVal);
			
			OutputMessage(output.ToString());
			
			if (ec.ErrorName != "NOERROR")
			{
				OnError(this, new ErrorEventArgs(this, ec));
				return false;
			}
			return true;				
		}
		
		public override bool SFX(SFXOptions options)
		{
			string command = arcBase.GetSFXCommand(options);
			if (command == "") return false;
			
			StringBuilder output = new StringBuilder(4096);
			Int32 retVal = arcBase._ArchiverExec(IntPtr.Zero, command, output, (UInt32)output.Capacity);
			ErrorMessage ec = Defines.ErrorMessage(retVal);
			
			OutputMessage(output.ToString());
			
			if (ec.ErrorName != "NOERROR")
			{
				OnError(this, new ErrorEventArgs(this, ec));
				return false;
			}
			return true;		
		}
	}
}
