/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 20/11/2006
 * Time: 22:21
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Cake3.Cmarc
{
	/// <summary>
	/// Description of LhaBase.
	/// </summary>
	public class LhaBase : ArchiverBase
	{		
		private const string DLLPATH  = "UnLHA32.dll";
		private const string ARCHIVER = "Unlha";
		
		public LhaBase() : base()
		{
			
		}
		
		public static List<string> SupportedExtensions()
		{ 
			List<string> retVal = new List<string>();
			
			foreach (ArchiveType at in SupportedArchiveTypes(""))
				foreach (string ext in at.SupportedExtensions.Split(';'))
				         retVal.Add(ext);

			return retVal;		
		}
		
		public static List<ArchiveType> SupportedArchiveTypes(string archiver)
		{
			List<ArchiveType> retVal = new List<ArchiveType>();
			if (Utils.verifyDllPath(DLLPATH))
			{
				retVal.Add(new ArchiveType(archiver, "lha,lzh,lzs", 0, true, true, true, true));
			}								
			return retVal;
		}
		
		protected override void SetArchiverName()
        {
            archiverName = ARCHIVER;
            dllPath = DLLPATH;
        }
		
		internal override string GetExtractCommand(Cake3.ExtractOptions options, string filename)
		{
			if (options.archiveName == "") 
				throw new Exception("No archive specified!");
						
			StringBuilder sb = new StringBuilder();
			
			
			sb.Append("e ");			
						
			sb.Append('"' + options.archiveName + '"' + ' ');
			sb.Append('"' + Utils.AppendSlash(options.ExtractFolderParsed) + '"' + ' ');
			sb.Append('"' + CmarcArchiver.ModifySlashFromCak(filename) + '"' + ' ');			
			
			if (options.allowOverwrite)
				sb.Append("-c1 ");
			else
				sb.Append("-c0 ");						
			
			if (options.allowFolder)
				sb.Append("-x1m1 ");
			
			if (options.dialogless)
					sb.Append("-gm1 -go1 -gp1 -gq1 -gn2"); //Dialogless			
			
			return sb.ToString();
		}
		
		
		internal override string GetSFXCommand(Cake3.SFXOptions options)
		{
			StringBuilder sb = new StringBuilder();
			
			sb.Append("s ");							
			
			sb.Append('"' + options.archiveName + '"' + ' ');			
			sb.Append('"' + Utils.AppendSlash(Utils.ExtractFilePath(options.archiveName)) + '"' + ' ');
			sb.Append("-gw3e ");
			
			return sb.ToString();
		}
		
		internal override string GetAddCommand(Cake3.AddOptions options, string filename)
		{
			if (options.archiveName == "") 
				throw new Exception("No archive specified!");
						
			StringBuilder sb = new StringBuilder();
			
			//Command
			sb.Append("a ");
			
			//Switches
			//TODO: LhaBase.cs - Compression Level is not implemented.			
			sb.Append("-a1 ");			
						
			switch (options.addFolder)
			{
				case (AddOptions.folderMode.full):
					sb.Append("-x1 -r2 ");
					break;					
				case (AddOptions.folderMode.relative):
					sb.Append("-x1 -r2 ");
					break;					
			}	
			
			//UPD: 08-09-07 Fixed Lha Add Bug.
			//Dialogless
			if (options.dialogless)
				sb.Append("-gm1 -go1 -gp1 -gq1 -gn2 "); //Dialogless			

							
			//ArchiveName
			sb.Append('"' + options.archiveName + '"' + ' ');	
			
			//BaseDir
			
			
			string baseFolder = options.baseFolder;
			string item = "";
			switch (options.addFolder)
			{
				case (AddOptions.folderMode.relative):
					item = getRelativePath(filename, ref baseFolder);
					break;
				case (AddOptions.folderMode.full):
					item = getFullPath(filename, ref baseFolder);
					break;
				case (AddOptions.folderMode.none):
					baseFolder = Utils.ExtractFilePath(filename);
					item = Utils.ExtractFileName(filename);
					break;					
			}			
		
			sb.Append('"' + baseFolder + "\\" + '"' + ' ');																	
			sb.Append('"' + item + '"' + ' ');
			
			return sb.ToString();
		
		}	
		
		internal override string GetDeleteCommand(Cake3.DeleteOptions options, string filename)
		{
			if (options.archiveName == "")
				throw new Exception("No archive specified!");
			
			StringBuilder sb = new StringBuilder();
			
			//Command
			sb.Append("d ");
			
			//Switches
			sb.Append("-jn ");	 //Ignore error if not exist
//			sb.Append("-jyd1 "); //Confirm when delete			
			
			//ArchiveName
			sb.Append('"' + options.archiveName + '"' + ' ');
			
			//Filename, mask supported
			sb.Append('"' + CmarcArchiver.ModifySlashFromCak(filename) + '"' + ' ');
			
			return sb.ToString();
		}
	}
	
	
}
