/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 23/11/2006
 * Time: 22:50
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Text;

namespace Cake3.Cmarc
{	
	/// <summary>
	/// Description of SevenZipBase.
	/// </summary>
	public class SevenZipBase : ArchiverBase
	{	
		private const string DLLPATH  = "7-zip32.dll";
		private const string ARCHIVER = "SevenZip";
		
		public SevenZipBase() : base()
		{
			
		}
		
		public static List<string> SupportedExtensions()
		{ 
			List<string> retVal = new List<string>();
			
			foreach (ArchiveType at in SupportedArchiveTypes(""))
				foreach (string ext in at.SupportedExtensions.Split(';'))
				         retVal.Add(ext);

			return retVal;		
		}
		
		public static List<ArchiveType> SupportedArchiveTypes(string archiver)
		{
			List<ArchiveType> retVal = new List<ArchiveType>();
			if (Utils.verifyDllPath(DLLPATH))
			{
				retVal.Add(new ArchiveType(archiver, "7z,7zip", 0, true, true, true, true));
			}								
			return retVal;
		}
		
		protected override void SetArchiverName()
        {
            archiverName = ARCHIVER;
            dllPath = DLLPATH;
        }
				
		
		internal override string GetExtractCommand(Cake3.ExtractOptions options, string filename)
		{
			if (options.archiveName == "") 
				throw new Exception("No archive specified!");
						
			StringBuilder sb = new StringBuilder();
			//7z <command> [<switches>...] <archive_name> [<file_names>...][<@listfiles...>]
			
			//Command
			if (options.allowFolder)
				sb.Append("x ");
			else sb.Append("e ");

			//Switches
			sb.Append("-o" + '"' + options.ExtractFolderParsed + '"' + ' ');			
			if (options.allowOverwrite)
				sb.Append("-aoa ");
			else
				sb.Append("-aos ");											
			if (options.password != "")
				sb.Append("-p"+options.password + ' ');
			
			sb.Append("-y "); //No confirm dialog.		
			if (options.dialogless)
					sb.Append("-hide ");
			
			//ArchiveName
			sb.Append('"' + options.archiveName + '"' + ' ');		
			
			//Mask
			sb.Append('"' + filename + '"' + ' ');
						
			return sb.ToString();
		}
		
		internal override string GetAddCommand(Cake3.AddOptions options, string filename)
		{
			if (options.archiveName == "") 
				throw new Exception("No archive specified!");
						
			StringBuilder sb = new StringBuilder();
			
			//Command
			sb.Append("a ");
			
			//Switches
			sb.Append("-t7z ");			

			if (options.dialogless)
					sb.Append("-hide ");
			
			if ((options.addOptionsEx != null) && (options.addOptionsEx is SevenZipAddOptions))
			{
				SevenZipAddOptions so = (SevenZipAddOptions)options.addOptionsEx;				
			}
						
			//TODO: SevernZipBase.cs - Compression Level is not implemented.
			sb.Append("-m0=BCJ -m1=LZMA:d=21 -ms=off ");
			
			if (options.password != "")
				sb.Append("-p" + options.password + " ");
			
//			switch (options.addFolder)
//			{
//				case (AddOptions.folderMode.full):
					sb.Append("-r0 ");
//					break; //06-17-07 -r0 required for all mask operations.
//			}
			
			//ArchiveName
			sb.Append('"' + options.archiveName + '"' + ' ');
			
			//BaseDir
			
			string baseFolder = options.baseFolder;
			string item = "";
			switch (options.addFolder)
			{
				case (AddOptions.folderMode.relative):
					item = getRelativePath(filename, ref baseFolder);
					break;
				case (AddOptions.folderMode.full):
					item = getFullPath(filename, ref baseFolder);
					break;
				case (AddOptions.folderMode.none):
					baseFolder = Utils.ExtractFilePath(filename);
					item = Utils.ExtractFileName(filename);
					break;
			}
			
			sb.Append('"' + baseFolder + "\\" + '"' + ' ');
			sb.Append('"' + item + '"' + ' ');
			
			return sb.ToString();
		}
		
		
		
		internal override string GetDeleteCommand(Cake3.DeleteOptions options, string filename)
		{
			if (options.archiveName == "")
				throw new Exception("No archive specified!");
			
			StringBuilder sb = new StringBuilder();
			
			//Command
			sb.Append("d ");
			
			//ArchiveName
			sb.Append('"' + options.archiveName + '"' + ' ');
			
			//Filename, mask supported
			sb.Append('"' + CmarcArchiver.ModifySlashFromCak(filename) + '"' + ' ');
			
			//Switches
			sb.Append("-ms=off "); 
			sb.Append("-r0 "); //06-12-07 delete subdir if masked.
			
			return sb.ToString();
		}
		
		
		
	}
	
	public class SevenZipAddOptions : AddOptionsEx
	{
		
		public SevenZipAddOptions()
		{
//			dialogless = false;
		}
		
//		public SevenZipAddOptions(bool isDialogless)
//		{
//			dialogless = isDialogless;
//		}
		
//		public static AddOptions NewAddOptions(string anArchiveName, string[] anAddFile,
//		                                       string aPassword, UInt16 aCompressLv, AddOptions.folderMode isAddFolder,
//		                                       bool isDialogless)
//		{
//			AddOptions retVal = new AddOptions(anArchiveName, anAddFile, aPassword, aCompressLv, isAddFolder);
//			SevenZipAddOptions szo = new SevenZipAddOptions();
//			szo.dialogless = isDialogless;
//			retVal.addOptionsEx = szo;
//			return retVal;
//		}
//			blockSize = 0;
//			solidMode = true;
//			headerCompress = true;
//			headerCompressFully = true;
//			headerEncrypt = false;
//			allowFolder = false;
		
		
		//-t (type)
		//-t[7z|zip]
		
		//-m (Compression Method)
		
		//-mzip
		
		//-m7z
		//f=[off|on]      								| on | Enable Filtering.
		//s=[off|on|[e][{N}f][{N}b|{N}k|{N}m|{N}g)]  	| on | Solid mode
		//f = limit file in same block
		//b,k,m,g = solid block size (byte, kb, mb, gb)
		//e.g. -s=1, -s=100f10m
		//hc=[off|on]									| on | Header compress
		//hcf=[off|on]									| on | Header compress fully, required  7-Zip 2.30 beta 25(2.30.25.01)
		//he=[off|on]									| off| Header encrypt
		//b{C1}[s{S1}]:{C2}[s{S2}]						|    |
		//{N}={MethodID}[:param1][:param2] ... [:paramN]|lzma| Compression method (LZMA, PPMd, BZip2, Deflate, BCJ, BCJ2, Copy)
		//mt=[off|on|{N}]								| on | Multi thread mode
		
		//-r (Recrusive dir)
		//-r[-| |0]										|    | - = no recrusive dir,
		
		//-p (Password)
		
		//-scs (Encoding)
		//-scs{UTF-8 | WIN | DOS}
		
		//-sfx (sfx module)
		//-sfx
		
//		public Int32 blockSize;
//		public bool solidMode;
//		public bool headerCompress;
//		public bool headerCompressFully;
//		public bool headerEncrypt;
//		public bool allowFolder;
//
//
//
	}
}
