using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Cake3
{	

	#region OuterContentList
	public class ContentList2 : IEnumerable
	{
		public EventHandler OnItemChanged;	
		internal ContentList baseContentList = new ContentList();
		internal ContentList filterContentList = new ContentList();
		int revision = 0;     		
		string directory = "";
		bool subdir = true;
        
		#region Filter Properties		
        public bool Reverse
        {  	get { return filterContentList.Reverse; }
        	set { filterContentList.Reverse = value; Reset(); } }         
		
        public string Directory
        {
        	get { return directory; }
        	set { directory = value;
        		  Reset();           		  
        		}
        }
        
        public bool Subdir
        {
        	get { return subdir; }
        	set { subdir = value;
        		  Reset();           		  
        		}

        }
        #endregion
        
        #region Directory methods       
        public List<string> GetDirectoryList()
		{        	
        	return baseContentList.GetDirectoryList();
		}
        
        public List<string> GetSubdirectoryList()
        {
        	return baseContentList.GetSubdirectoryList(directory);
        }
        
        public List<string> GetSubdirectoryList(string directory)
        {
        	return baseContentList.GetSubdirectoryList(directory);
        }        
        #endregion
        
        #region Obtain Entry        
        public ContentType this[int index]
        {
        	get
        	{        		
        		return filterContentList[index];
        	}
        }
        
        public int TotalCount
        {
			get
			{
				return( filterContentList.Count );
			}
        }
                
        public int Count
		{
			get
			{
				return( filterContentList.Count );
			}
		}   
        #endregion
        
        internal int Revision
		{
			get
			{
				return( revision );
			}
		}
		
		public bool FileExists(string filename)
		{
			return baseContentList.FileExists(filename);
		}
		
		public void Add( ContentType content )
		{
			baseContentList.Add(content);			
		}
		
		public void Clear()
		{
			baseContentList.Clear();
		}        
                 
		
		public void Sort()
		{
			baseContentList.Sort();
			Reset();
		}

		public void Sort( ContentType.SortMetric sortMetric )
		{
			baseContentList.Sort(sortMetric);
			Reset();
		}
		
		#region IEnumerable Members
		public IEnumerator GetEnumerator()
		{
			if (filterContentList.Count != 0)
				return( filterContentList.GetEnumerator() );
			return( baseContentList.GetEnumerator() );
		}
						
        void Reset()
        {  	
        	revision++;  
        	filterContentList = baseContentList.Filter(directory, subdir);
        	this.GetEnumerator().Reset();   

        	if (OnItemChanged != null)
        		OnItemChanged(this, new EventArgs());
        }         
        #endregion
	}	
	
//	public class ContentList2Enumerator : IEnumerator
//	{
//		
//	}
	#endregion

	#region BaseContentList	
	public class ContentList : IEnumerable
	{
		public EventHandler OnItemChanged;		
		internal ContentType[] contentList = new ContentType[1];
		internal List<string> dirList = new List<string>();		
				
		int count = 0;
        int revision = 0;                 
        bool reverse = false;
        
        public bool Reverse
        {
        	get { return reverse; }
        	set { reverse = value; Reset(); }
        }                        
               
        public List<string> GetDirectoryList()
		{  return dirList;	}
                
        public List<string> GetSubdirectoryList(string directory)
        {
        	List<string> retVal = new List<string>();
        	List<string> dirList = GetDirectoryList();        	
        	
			foreach (string s in dirList)
			{
				if (s.IndexOf(directory) == 0)
				{
					string subdir = Utils.ExtractFirstDir(s, directory.Length);
				
					if (retVal.IndexOf(subdir) == -1)
        				retVal.Add(subdir);																				
				}
			}
			
			return retVal;
        }
        
        public ContentList Filter(string mask)
        {
        	ContentList result = new ContentList();
        	
        	foreach (ContentType ct in this)
        	{
        		if (Utils.MatchMask(ct.fileName, mask))
        		    result.Add(ct);
        	}
        	
        	return result;
        }
                
        public ContentList Filter(string path, bool subdir)
        {
        	ContentList result = new ContentList();
        	
        	foreach (ContentType ct in this)
        	{
        		if (Utils.MatchPath(path, ct.fileName, subdir))
        			result.Add(ct);
        	}
        	
        	return result;
        }
        
        void Reset()
        {
        	revision++;        	
        	this.GetEnumerator().Reset();   

        	if (OnItemChanged != null)
        		OnItemChanged(this, new EventArgs());
        }
        
        public ContentType this[int index]
        {
        	get
        	{        		
        		return contentList[index];
        	}
        }
                
        public int Count
        {
			get
			{
				return( count );
			}
        }
                       
		internal int Revision
		{
			get
			{
				return( revision );
			}
		}
		
		public bool FileExists(string filename)
		{
			foreach (ContentType ct in contentList)
			{
				if (ct.fileName == filename)
					return true;
			}
			
			return false;
		}
		
		public void Add( ContentType content )
		{
			if( count >= contentList.Length )
			{
				contentList = (ContentType[]) Resize( contentList, contentList.Length * 2 );
			}
			contentList[count++] = content;
			
			string path = Utils.ExtractFilePath(content.fileName);			 	
     		if (dirList.IndexOf(path) == -1)
        		dirList.Add(path);
			
		}
		
		public void Clear()
		{
			count = 0;
			dirList.Clear();
		}

		public static Array Resize( Array array, int newSize )
		{
			Type type = array.GetType().GetElementType();
			Array newArray = Array.CreateInstance( type, newSize );
			Array.Copy( array, 0, newArray, 0, Math.Min( array.Length, newSize ));
			return newArray;
		}


        internal void CheckIndex( int index )
        {
            if( index >= count )
                throw new ArgumentOutOfRangeException( 
                                 @"Index value out of range" );
        }
        
        #region IComparable Members   
        
        internal static IComparer SortByFilePathName
        {
            get
            {
                return( (IComparer) new ContentType.SortByFilePathNameClass());
            }
        }
		
		internal static IComparer SortByFileName
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileNameClass());
            }
        }
		
		internal static IComparer SortByFileExt
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileExtClass());
            }
        }
		
		internal static IComparer SortByFileSize
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileSizeClass());
            }
        }
		
		internal static IComparer SortByFilePackedSize
        {
            get
            {
                return( (IComparer) new ContentType.SortByFilePackedSizeClass());
            }
        }
		
		internal static IComparer SortByFileTime
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileTimeClass());
            }
        }
		
		internal static IComparer SortByFileRatio
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileRatioClass());
            }
        }
		
		internal static IComparer SortByFileCRC
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileCRCClass());
            }
        }
		
		internal static IComparer SortByFilePathNameR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFilePathNameClassR());
            }
        }
		
		internal static IComparer SortByFileNameR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileNameClassR());
            }
        }
		
		internal static IComparer SortByFileExtR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileExtClassR());
            }
        }
		
		internal static IComparer SortByFileSizeR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileSizeClassR());
            }
        }
		
		internal static IComparer SortByFilePackedSizeR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFilePackedSizeClassR());
            }
        }
		
		internal static IComparer SortByFileTimeR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileTimeClassR());
            }
        }
		
		internal static IComparer SortByFileRatioR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileRatioClassR());
            }
        }
		
		internal static IComparer SortByFileCRCR
        {
            get
            {
                return( (IComparer) new ContentType.SortByFileCRCClassR());
            }
        }
		
		
		#endregion
		
		public void Sort()
		{
			Array.Sort( contentList, 0, count );
			Reset();
		}

		public void Sort( ContentType.SortMetric sortMetric )
		{
			if (!this.reverse)							
			switch( sortMetric )
            {
				case ContentType.SortMetric.FilePathName:				
					Array.Sort( contentList, 0, count, SortByFilePathName );
                    break;
                case ContentType.SortMetric.FileName:				
					Array.Sort( contentList, 0, count, SortByFileName );
                    break;
                case ContentType.SortMetric.FileExt:				
					Array.Sort( contentList, 0, count, SortByFileExt );
                    break;
                case ContentType.SortMetric.FileSize:				
					Array.Sort( contentList, 0, count, SortByFileSize );
                    break;
                case ContentType.SortMetric.FilePackedSize:				
					Array.Sort( contentList, 0, count, SortByFilePackedSize );
                    break;
                case ContentType.SortMetric.FileTime:				
					Array.Sort( contentList, 0, count, SortByFileTime );
                    break;
                case ContentType.SortMetric.FileRatio:				
					Array.Sort( contentList, 0, count, SortByFileRatio );
                    break;
                case ContentType.SortMetric.FileCRC:				
					Array.Sort( contentList, 0, count, SortByFileCRC );
                    break;
			} else
			switch( sortMetric )
            {
				case ContentType.SortMetric.FilePathName:				
					Array.Sort( contentList, 0, count, SortByFilePathNameR );
                    break;
                case ContentType.SortMetric.FileName:				
					Array.Sort( contentList, 0, count, SortByFileNameR );
                    break;
                case ContentType.SortMetric.FileExt:				
					Array.Sort( contentList, 0, count, SortByFileExtR );
                    break;
                case ContentType.SortMetric.FileSize:				
					Array.Sort( contentList, 0, count, SortByFileSizeR );
                    break;
                case ContentType.SortMetric.FilePackedSize:				
					Array.Sort( contentList, 0, count, SortByFilePackedSizeR );
                    break;
                case ContentType.SortMetric.FileTime:				
					Array.Sort( contentList, 0, count, SortByFileTimeR );
                    break;
                case ContentType.SortMetric.FileRatio:				
					Array.Sort( contentList, 0, count, SortByFileRatioR );
                    break;
                case ContentType.SortMetric.FileCRC:				
					Array.Sort( contentList, 0, count, SortByFileCRCR );
                    break;
			}
			Reset();
		}
		
		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return( new ContentListEnumerator( this ));
		}

		#endregion
		
		
	}
	
	public class ContentListEnumerator : IEnumerator
    {
        ContentList contentList;
        int index;
        int revision;

        #region IEnumerator Members

        public void Reset()
        {
            index = -1;            	        	
        	revision = contentList.Revision;
        }

        public object Current
        {
            get
            {
                if( revision != contentList.Revision )
                    throw new InvalidOperationException( 
                            @"Collection modified while enumerating." );
                return( contentList[index] );
            }
        }              

		public bool MoveNext()
		{
			index += 1;
			
			while (( index < contentList.Count ))
			{
				index += 1;
			}
											
			return ( index < contentList.Count );			
		}
	
		#endregion
		
		internal ContentListEnumerator( ContentList contentList )
		{
			this.contentList = contentList;
			Reset();
		}

	}
	#endregion
}
