/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 20/11/2006
 * Time: 22:00
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using NaturalSorting;

namespace Cake3
{
	/// <summary>
	/// Represent a file in an archive.
	/// </summary>
	public struct ContentType : ICloneable, IComparable 
	{		
		#region SortMetric		
		/// <summary>
		/// Specifies the sort method.
		/// </summary>
		public enum SortMetric 
		{
			/// <summary>
			/// Sort by file name with path.
			/// </summary>
			FilePathName,
			/// <summary>
			/// Sort by file name.
			/// </summary>
			FileName,
			/// <summary>
			/// Sort by file extension.
			/// </summary>
			FileExt,
			/// <summary>
			/// Sort by file size.
			/// </summary>
			FileSize,
			/// <summary>
			/// Sort by file packed size.
			/// </summary>
			FilePackedSize,
			/// <summary>
			/// Sort by file time.
			/// </summary>
			FileTime,
			/// <summary>
			/// Sort by file compress ratio.
			/// </summary>
			FileRatio,
			/// <summary>
			/// Sort by file CRC.
			/// </summary>
			FileCRC 							
		};
		#endregion
		
		/// <summary>
		/// Gets the file size.
		/// </summary>
		public UInt32 fileSize;
		/// <summary>
		/// Gets the file packed size.
		/// </summary>
		public UInt32 filePackedSize;
		/// <summary>
		/// Get the file time.
		/// </summary>
		public DateTime fileTime;
		/// <summary>
		/// Gets the file CRC.
		/// </summary>
		public Int64 fileCRC;
		/// <summary>
		/// Gets the file name.
		/// </summary>
		public string fileName;
		/// <summary>
		/// Gets whether the file is encrypted.
		/// </summary>
		public bool fileEncrypted;
		/// <summary>
		/// Gets the archive name.
		/// </summary>
		public string archiveName;
		/// <summary>
		/// Returns a new ContentType object whose value is the same as the current ContentType object. 
		/// </summary>
		/// <returns></returns>
		public object Clone() { 
			ContentType content = new ContentType(archiveName, fileSize, filePackedSize, fileTime, fileCRC, fileName, fileEncrypted);
    		return content; 
  		}	
		
		/// <summary>
		/// Initalize a new instance of ContentType object.
		/// </summary>
		/// <param name="aFileSize">File size.</param>
		/// <param name="aFilePackedSize">File packed size.</param>
		/// <param name="aFileTime">File time.</param>
		/// <param name="aFileCRC">File CRC.</param>
		/// <param name="aFileName">File name.</param>
		/// <param name="aFileEncrypted">File encrypted.</param>
		public ContentType(string anArchiveName, UInt32 aFileSize, UInt32 aFilePackedSize, DateTime aFileTime, Int64 aFileCRC, 
		                   string aFileName, bool aFileEncrypted)
		{
			archiveName = anArchiveName;
			fileSize = aFileSize;
			filePackedSize = aFilePackedSize;
			fileTime = aFileTime;
			fileCRC = aFileCRC;
			fileName = Utils.ParsePChar(aFileName);
			fileEncrypted = aFileEncrypted;			
		}
		/// <summary>
		/// Initalize a new instance of ContentType object.
		/// </summary>
		/// <param name="aFileSize">File size.</param>
		/// <param name="aFilePackedSize">File packed size.</param>
		/// <param name="aFileTime">File time.</param>
		/// <param name="aFileCRC">File CRC.</param>
		/// <param name="aFileName">File name.</param>
		/// <param name="aFileEncrypted">File encrypted.</param>		
		public ContentType(string anArchiveName, UInt32 aFileSize, UInt32 aFilePackedSize, DateTime aFileTime, Int32 aFileCRC, 
		                   string aFileName, bool aFileEncrypted)
		{
			archiveName = anArchiveName;
			fileSize = aFileSize;
			filePackedSize = aFilePackedSize;
			fileTime = aFileTime;
			fileCRC = (Int64)aFileCRC;
			fileName = Utils.ParsePChar(aFileName);			
			fileEncrypted = aFileEncrypted;
		}
		/// <summary>
		/// Initalize a new instance of ContentType object.
		/// </summary>
		/// <param name="aFileSize">File size.</param>
		/// <param name="aFilePackedSize">File packed size.</param>
		/// <param name="aFileTime">File time.</param>
		/// <param name="aFileCRC">File CRC.</param>
		/// <param name="aFileName">File name.</param>
		/// <param name="aFileEncrypted">File encrypted.</param>
		public ContentType(string anArchiveName, UInt32 aFileSize, UInt32 aFilePackedSize, DateTime aFileTime, UInt32 aFileCRC, 
		                   string aFileName, bool aFileEncrypted)
		{
			archiveName = anArchiveName;
			fileSize = aFileSize;
			filePackedSize = aFilePackedSize;
			fileTime = aFileTime;
			fileCRC = (Int64)aFileCRC;
			fileName = Utils.ParsePChar(aFileName);			
			fileEncrypted = aFileEncrypted;
		}
		
		#region Sort related	
		
		private static NaturalComparer nc = new NaturalComparer();		
		/// <summary>
		/// Compares this ContentType object to the supplied object and returns an indication of their relative values.
		/// </summary>
		/// <param name="obj1"></param>
		/// <returns></returns>		
		public Int32 CompareTo(Object obj1)
		{
			return nc.Compare(fileName, ((ContentType)obj1).fileName);
//			return fileName.CompareTo(((ContentType)obj1).fileName);
		}
		
		internal class SortByFilePathNameClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			string p1 = Utils.ExtractFilePath(ct1.fileName).ToLower();
    			string p2 = Utils.ExtractFilePath(ct2.fileName).ToLower();
    			string f1 = Utils.ExtractFileName(ct1.fileName).ToLower();
    			string f2 = Utils.ExtractFileName(ct2.fileName).ToLower();
    			
    			if (p1 != p2)
    				return nc.Compare(p1, p2); 
    			else return nc.Compare(f1, f2);    		
        	}            
   		}	
		
		internal class SortByFileNameClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			
    			
    			string f1 = Utils.ExtractFileName(ct1.fileName).ToLower();
    			string f2 = Utils.ExtractFileName(ct2.fileName).ToLower();
    		    		
    			
    			return nc.Compare(f1, f2);   		
        	}            
   		}	
		

		
		internal class SortByFileExtClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			string f1 = Utils.ExtractFileExt(ct1.fileName);
    			string f2 = Utils.ExtractFileExt(ct2.fileName);
    		
        		return f1.CompareTo(f2);    		
        	}            
   		}	
		
		internal class SortByFileSizeClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct1.fileSize.CompareTo(ct2.fileSize);    		
        	}            
   		}	
		
		internal class SortByFilePackedSizeClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct1.filePackedSize.CompareTo(ct2.filePackedSize);    		
        	}            
   		}	
		
		internal class SortByFileTimeClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct1.fileTime.CompareTo(ct2.fileTime);    		
        	}            
   		}	
		
		internal class SortByFileCRCClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct1.fileCRC.CompareTo(ct2.fileCRC);    		
        	}            
   		}	
		
		internal class SortByFileRatioClass : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			float ratio1 = (float)ct1.filePackedSize / (float)ct1.fileSize;
    			float ratio2 = (float)ct2.filePackedSize / (float)ct2.fileSize;
    		
        		return ratio1.CompareTo(ratio2);    		
        	}            
   		}
		
		
		
		internal class SortByFilePathNameClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			string p1 = Utils.ExtractFilePath(ct1.fileName).ToLower();
    			string p2 = Utils.ExtractFilePath(ct2.fileName).ToLower();
    			string f1 = Utils.ExtractFileName(ct1.fileName).ToLower();
    			string f2 = Utils.ExtractFileName(ct2.fileName).ToLower();
    			
    			if (p1 != p2)
    				return nc.Compare(p2, p1); 
    			else return nc.Compare(f2, f1);    		
        	}     		     
   		}	

		internal class SortByFileNameClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			string f1 = Utils.ExtractFileName(ct1.fileName).ToLower();
    			string f2 = Utils.ExtractFileName(ct2.fileName).ToLower();
    		
        		return nc.Compare(f2, f1);    		
        	}            
   		}		

		internal class SortByFileExtClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			string f1 = Utils.ExtractFileExt(ct1.fileName);
    			string f2 = Utils.ExtractFileExt(ct2.fileName);
    		
        		return f2.CompareTo(f1);    		
        	}            
   		}	
		
		internal class SortByFileSizeClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct2.fileSize.CompareTo(ct1.fileSize);    		
        	}            
   		}	
		
		internal class SortByFilePackedSizeClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct2.filePackedSize.CompareTo(ct1.filePackedSize);    		
        	}            
   		}	
		
		internal class SortByFileTimeClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct2.fileTime.CompareTo(ct1.fileTime);    		
        	}            
   		}	
		
		internal class SortByFileCRCClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    		
        		return ct2.fileCRC.CompareTo(ct1.fileCRC);    		
        	}            
   		}	
		
		internal class SortByFileRatioClassR : IComparer
    	{
    		public int Compare(object x, object y)
        	{
    			ContentType ct1 = (ContentType)(x);
    			ContentType ct2 = (ContentType)(y);
    			float ratio1 = (float)ct1.filePackedSize / (float)ct1.fileSize;
    			float ratio2 = (float)ct2.filePackedSize / (float)ct2.fileSize;
    		
        		return ratio2.CompareTo(ratio1);    		
        	}            
   		}	
		#endregion
	
		public string GetFile(string password)
		{
			string tempPath = Utils.GetTempPath() + @"CAKE3\";
			string tempFile = tempPath + fileName;
			if (Utils.FileExists(tempFile))
				if (Utils.GetFileInfo(tempFile).LastWriteTime == fileTime)
					return tempFile;			
			
			Utils.MakeDirectory(tempPath);
			
			Cakdir3 c3 = new Cakdir3(archiveName);			
			c3.Extract(fileName, tempPath, password, true, true);
			c3 = null;
			
			return tempFile;
		}
		
		public string GetFile()
		{
			return GetFile("");
		}
		
		public Stream GetStream()
		{
			string tempFile = GetFile();
			
			FileStream fs = new FileStream(tempFile, FileMode.Open);						
			BinaryReader br = new BinaryReader(fs);
			
			MemoryStream ms = new MemoryStream((int)fs.Length);
			BinaryWriter bw = new BinaryWriter(ms);
			
			try
			{							
				byte[] buffer;				
				buffer = br.ReadBytes(10240);
				bw.Write(buffer);
								
				while (buffer.Length > 0)
				{
					buffer = br.ReadBytes(10240);
					bw.Write(buffer);
				}
			}
			finally
			{
				fs.Close();
				br.Close();	
				ms.Position = 0; //Reset MemoryStream
//				Utils.RemoveFile(tempFile);
			}
			
			return (Stream)ms;
			
		}
	}
	
}
