/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 20/11/2006
 * Time: 21:02
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

namespace Cake3
{	
	#region Structures	
	/// <summary>
	/// Represent the options for extract file(s) from archive.
	/// </summary>
	public class ExtractOptions
	{
		/// <summary>
		/// Represent how to handle if file already exist when extracting.		
		/// </summary>
		public enum overwriteMode { 
			/// <summary>
			/// Ask user to overwrite or not.
			/// </summary>
			ask,
			/// <summary>
			/// Do not overwrite.
			/// </summary>
			skip,
			/// <summary>
			/// Overwrite existing file.
			/// </summary>
			overwrite,
			/// <summary>
			/// Rename old file.
			/// </summary>
			renameOld,
			/// <summary>
			/// Compare datetime and replace if current file is newer.
			/// </summary>
			compareDate
			}
		/// <summary>
		/// Gets of sets the filename of the archive.
		/// </summary>
		public string archiveName;								

		/// <summary>
		/// Gets or sets the directory to extract to.
		/// </summary>
		public string extractFolder;
		
		public string ExtractFolderParsed
		{
			get { string retVal = extractFolder.Replace("%archive%",Utils.RemoveFileExt(archiveName));
				  	retVal = retVal.Replace("%desktop%",Utils.GetDesktopPath());
					retVal = retVal.Replace("%mydocuments%",Utils.GetMydocuPath());				
					retVal = retVal.Replace("%programs%",Utils.GetProgramPath());
					retVal = retVal.Replace("%temp%",Utils.GetTempPath());
					while (retVal.IndexOf(@"\\") != -1)
						retVal = retVal.Replace(@"\\",@"\");
					return retVal;
			}
		}
		/// <summary>
		/// Gets or sets the archive password for extract. 
		/// </summary>
		public string password;
		/// <summary>
		/// Gets or sets an array of file to extract, mask (e.g. *) is supported.
		/// </summary>
		public string[] extractItem;	
		/// <summary>
		/// Gets or sets a value indicating how to handle when file already exists.
		/// </summary>
		public overwriteMode extractOverwrite;
		/// <summary>
		/// Gets or sets a value indicating whether cake overwrite original if the file exist already.
		/// </summary>
		public bool allowOverwrite 
		{ 
			get { return extractOverwrite == overwriteMode.overwrite; } 
			set { if (value)
					extractOverwrite = ExtractOptions.overwriteMode.overwrite;
					else extractOverwrite = ExtractOptions.overwriteMode.ask;
			}
		}
		/// <summary>
		/// Gets or sets a value indicating whether cake extract with folder information.
		/// </summary>
		public bool allowFolder;
		
		public bool dialogless = true;	
		
		/// <summary>
		/// Reset the extractItem to all files (*).
		/// </summary>
		public void ResetExtractItem()
		{
			extractItem = new string[1] {"*"};
		}
				
		/// <summary>
		/// Initializes a new instance of the ExtractOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		public ExtractOptions(string anArchiveName)
		{
			archiveName = anArchiveName;	
			extractItem = new string[1] {"*"};			
			extractFolder = "";
			extractOverwrite = overwriteMode.ask;
			allowOverwrite = true;
			allowFolder = true;
			password = "";			
		}
		
		/// <summary>
		/// Initializes a new instance of the ExtractOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		/// <param name="anExtractFolder">Directory to extract to.</param>
		/// <param name="isAllowOverwrite">Specify to overwrite original or not.</param>
		public ExtractOptions(string anArchiveName, string anExtractFolder, bool isAllowOverwrite)
		{
			archiveName = anArchiveName;
			extractItem = new string[1] {"*"};
			extractFolder = anExtractFolder;
			allowFolder = true;
			
			extractOverwrite = overwriteMode.ask;
			if (isAllowOverwrite)
				extractOverwrite = overwriteMode.overwrite;			
			allowOverwrite = isAllowOverwrite;
			password = "";
		}
		
		/// <summary>
		/// Initializes a new instance of the ExtractOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		/// <param name="anExtractFolder">Directory to extract to.</param>		
		/// <param name="anExtractItem">An array to specify which file to extract, mask (e.g. *) is supported.</param>
		/// <param name="isAllowOverwrite">Specify to overwrite original or not.</param>
		/// <param name="isExtractFolder">Specify to extract with folder information.</param>
		public ExtractOptions(string anArchiveName, string anExtractFolder, string[] anExtractItem, bool isAllowOverwrite, bool isAllowFolder)
		{
			archiveName = anArchiveName;
			extractItem = anExtractItem;
			extractFolder = anExtractFolder;
			allowFolder = isAllowFolder;

			extractOverwrite = overwriteMode.ask;
			if (isAllowOverwrite)
				extractOverwrite = overwriteMode.overwrite;						
			allowOverwrite = isAllowOverwrite;
			password = "";
		}
		
		public ExtractOptions()
		{
			
		}
		
		public static void ToXml(ExtractOptions extrOption, string filename)
		{
			XmlSerializer serializer = new XmlSerializer(typeof(ExtractOptions));
			FileStream fs = File.OpenWrite(filename);			
			XmlTextWriter writer = new XmlTextWriter(fs, System.Text.Encoding.UTF8);
			writer.Formatting = Formatting.Indented;

			try
			{				
				serializer.Serialize(writer, extrOption);
			}
			finally
			{
				writer.Close();
			}
		}
		
		public static ExtractOptions FromXml(string filename)
		{
			ExtractOptions retVal = new ExtractOptions();
			
			XmlSerializer serializer = new XmlSerializer(typeof(ExtractOptions));
			FileStream fs = File.OpenRead(filename);					
			
			try
			{				
				retVal = (ExtractOptions)serializer.Deserialize(fs);
			}
			finally
			{
				fs.Close();				
			}
			
			return retVal;
		}
	}
	
	/// <summary>
	/// Represent the options for add file(s) to archive.
	/// </summary>
	public class AddOptions : ICloneable
	{
		/// <summary>
		/// Represent how folder information is stored in archive.		
		/// </summary>
		public enum folderMode { 
			/// <summary>
			/// Store folder information relative to  baseFolder.
			/// </summary>
			relative, 
			/// <summary>
			/// Store full folder information.
			/// </summary>
			full, 
			/// <summary>
			/// Store no folder information.
			/// </summary>
			none }
		
		/// <summary>
		/// Represent how file is added to archive.
		/// </summary>
		public enum updateMode {
			/// <summary>
			/// Add all specified files to the archive.
			/// </summary>
			add, 
			/// <summary>
			/// Update older files in the archive and add files that are new to the archive.
			/// </summary>
			refresh, 
			/// <summary>
			/// Update specified files in the archive that are older than the selected disk files.
			/// </summary>
			update, 
			/// <summary>
			/// Replace specified files only if added files are newer. Always add those files, 
			/// which are not present in the archive. Delete from archive those files, 
			/// which are not present on the disk.
			/// </summary>
			synchronize
		}
		/// <summary>
		/// Gets of sets the filename of the archive.
		/// </summary>
		public string archiveName;
		/// <summary>
		/// Get or sets a list of file to be added to the archive.
		/// </summary>
		public string[] addFile;
		/// <summary>
		/// Get or sets the compress level.
		/// </summary>
		public UInt16 addCompressLevel;
		/// <summary>
		/// Gets or sets the archive password for add. 
		/// </summary>
		public string password;
		/// <summary>
		/// Get or sets how folder information is stored in archive.
		/// </summary>
		public folderMode addFolder;
		/// <summary>
		/// Get or sets the base folder. (require addFolder = folderMode.relative)
		/// </summary>
		public string baseFolder;
		public updateMode addMode;
		
		public bool dialogless;		
		/// <summary>
		/// Get or sets archiver based options.
		/// </summary>
		/// 	
		public AddOptionsEx addOptionsEx;
		
		public void defaultOptions()
		{
			archiveName = "";
			addFile = new string[0];
			addCompressLevel = 9;
			addFolder = folderMode.relative;
			password = "";
			baseFolder = "";
			dialogless = true;
			addMode = updateMode.add;
		}
		
		/// <summary>
		/// Initializes a new instance of the AddOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		/// <param name="anAddFile">File(s) add to the archive.</param>
		/// <param name="aCompressLv">Compress level.</param>
		/// <param name="isAddFolder">Folder storage information.</param>		
		public AddOptions(string anArchiveName, string[] anAddFile, UInt16 aCompressLv, folderMode isAddFolder, string aPassword, bool isDialogless, AddOptionsEx anOptionsEx)
		{
			defaultOptions();
			archiveName = anArchiveName;
			addFile = anAddFile;
			addCompressLevel = aCompressLv;
			addFolder = isAddFolder;
			password = aPassword;	
			baseFolder = "";
			dialogless = isDialogless;
			addOptionsEx = anOptionsEx;			
		}	
		public AddOptions(string anArchiveName, string[] anAddFile, UInt16 aCompressLv, folderMode isAddFolder, string aPassword, bool isDialogless)
		{
			defaultOptions();
			archiveName = anArchiveName;
			addFile = anAddFile;
			addCompressLevel = aCompressLv;
			addFolder = isAddFolder;
			password = aPassword;	
		}	
		public AddOptions(string anArchiveName, string[] anAddFile, UInt16 aCompressLv, folderMode isAddFolder, string aPassword)
		{
			defaultOptions();
			archiveName = anArchiveName;
			addFile = anAddFile;
			addCompressLevel = aCompressLv;
			addFolder = isAddFolder;
			password = aPassword;				
		}
		
		public AddOptions(string anArchiveName, string[] anAddFile, UInt16 aCompressLv, folderMode isAddFolder)
		{
			defaultOptions();
			archiveName = anArchiveName;
			addFile = anAddFile;
			addCompressLevel = aCompressLv;
			addFolder = isAddFolder;
		}
			
		
		public AddOptions(string anArchiveName, string[] anAddFile, UInt16 aCompressLv)
		{
			defaultOptions();
			archiveName = anArchiveName;
			addFile = anAddFile;
			addCompressLevel = aCompressLv;			
		}
		
		public AddOptions(string anArchiveName, string[] anAddFile)
		{
			defaultOptions();			
			archiveName = anArchiveName;
			addFile = anAddFile;
		}
					
		/// <summary>
		/// Initializes a new instance of the AddOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		public AddOptions(string anArchiveName)
		{
			defaultOptions();
			archiveName = anArchiveName;
		}
		
		public AddOptions()
		{
			defaultOptions();
		}
		
		/// <summary>
		/// Return a list of file that match the masks
		/// </summary>
		/// <param name="addFile">A list of addfile mask.</param>
		/// <param name="subdir">Include subdirectory or not.</param>
		/// <returns></returns>
		internal static List<string> ParseAddFileMask(string[] addFile, bool subdir)
		{			
			List<string> retVal = new List<string>();
			foreach (string item in addFile)
				retVal.AddRange(Utils.PollFileList(item, subdir));
			
			return retVal;
		}
		
		internal string ParseAddFile(string addFile)
		{
			return Utils.getPath(addFile, baseFolder, addFolder);
		}
		
		public object Clone()
		{
			AddOptions retVal = new AddOptions(archiveName, addFile, addCompressLevel, addFolder);
			retVal.addFile =  (string[])addFile.Clone();
			retVal.baseFolder = baseFolder;
			retVal.password = password;
			retVal.addMode = addMode;
			return retVal;
		}

		public static void ToXml(AddOptions addOption, string filename)
		{
			XmlSerializer serializer = new XmlSerializer(typeof(AddOptions));
			FileStream fs = File.OpenWrite(filename);			
			XmlTextWriter writer = new XmlTextWriter(fs, System.Text.Encoding.UTF8);
			writer.Formatting = Formatting.Indented;

			try
			{				
				serializer.Serialize(writer, addOption);
			}
			finally
			{
				writer.Close();
			}
		}
		
		public static AddOptions FromXml(string filename)
		{
			AddOptions retVal = new AddOptions();
			
			XmlSerializer serializer = new XmlSerializer(typeof(AddOptions));
			FileStream fs = File.OpenRead(filename);					
			
			try
			{				
				retVal = (AddOptions)serializer.Deserialize(fs);
			}
			finally
			{
				fs.Close();				
			}
			
			return retVal;
		}
	}
	
	/// <summary>
	/// Represent extra options for add file(s) to archive.
	/// </summary>
	public class AddOptionsEx
	{
	}
	
	/// <summary>
	/// Represent the options for delete file(s) from archive.
	/// </summary>
	public class DeleteOptions
	{
		/// <summary>
		/// Gets of sets the filename of the archive.
		/// </summary>
		public string archiveName;
		/// <summary>
		/// Get or sets a list of file to be deleted from the archive.
		/// </summary>
		public string[] deleteFile;
		/// <summary>
		/// Initializes a new instance of the DeleteOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		/// <param name="aDeleteFile">File to be deleted.</param>
		public DeleteOptions(string anArchiveName, string[] aDeleteFile)
		{
			archiveName = anArchiveName;
			deleteFile = aDeleteFile;
		}
		/// <summary>
		/// Initializes a new instance of the DeleteOptions class.
		/// </summary>
		/// <param name="anArchiveName">Filename of the archive.</param>
		public DeleteOptions(string anArchiveName)
		{
			archiveName = anArchiveName;			
		}
	}
	
	/// <summary>
	/// Represent the options for Creating SFX.
	/// </summary>
	public class SFXOptions
	{
		public string archiveName;
		
		public SFXOptions(string anArchiveName)
		{
			archiveName = anArchiveName;
		}
	}
	
	/// <summary>
	/// Represent an error message type.
	/// </summary>
	public class ErrorMessage
	{	
		private Int32 errorCode;
		private string errorName;
		private string errorDescription;
		/// <summary>
		/// Internal code of the error.
		/// </summary>
		public Int32 ErrorCode 			{ get { return errorCode; } }
		/// <summary>
		/// Short description of the error, for internal use.
		/// </summary>
		public string ErrorName 		{ get { return errorName; } }
		/// <summary>
		/// Long description the error.
		/// </summary>
		public string ErrorDescription	{ get { return errorDescription; } set { errorDescription = value; } }
		/// <summary>
		/// Initializes a new instance of the ErrorMessage class.
		/// </summary>
		/// <param name="anErrorCode">Internal code.</param>
		/// <param name="anErrorName">Short description.</param>
		/// <param name="anErrorDescription">Long description.</param>
		public ErrorMessage(Int32 anErrorCode, string anErrorName, string anErrorDescription)
		{
			errorCode = anErrorCode;
			errorName = anErrorName;
			errorDescription = anErrorDescription;
		}
	}
				
 	public class ArchiveType : IComparable
	{		
		string archiver;
		string extension;
		string additionalextensions = "";
		string additionaldescription = "";
		Int32 archivegroup;
		bool canList;
		bool canExtract;
		bool canAdd;
		bool canUpdate;
				
		public bool CanList { get { return canList; } }
		public bool CanExtract { get { return canExtract; } }
		public bool CanAdd { get { return canAdd; } }
		public bool CanUpdate { get { return canUpdate; } }
		public string Archiver { get { return archiver; } }
		public string MainExtension { get { return extension; } }
		public string Extensions { get { return (extension + "," + additionalextensions).Trim(','); } }
		public string SupportedExtensions { get { return "." + Extensions.Replace(",", ";."); } }
		public string ExtensionMasks { get { return "*." + Extensions.Replace(",", ";*."); } }
		public string Filter { get { return string.Format("|{0} ({1})|{2}",Description, Extensions.Replace(","," "), ExtensionMasks); } }
		public string Description { get { 
				string retVal = extension + " file";
				if (Defines.ArchiveDescriptionList.ContainsKey(extension)) 					
				{
					string[] descs = ((string)Defines.ArchiveDescriptionList[extension]).Split(',');
					if (descs.Length > archivegroup) 
						retVal = descs[archivegroup];
					else retVal = descs[0]; 
				}
				return retVal + additionaldescription;
				} }
		
		public int CompareTo (Object value)
		{
			int retVal = Description.CompareTo((((ArchiveType)value).Description));
			if (retVal == 0) retVal = -1 * CanUpdate.CompareTo((((ArchiveType)value).CanUpdate));
			if (retVal == 0) retVal = -1 * CanAdd.CompareTo((((ArchiveType)value).CanAdd));
			if (retVal == 0) retVal = -1 * CanExtract.CompareTo((((ArchiveType)value).CanExtract));
			if (retVal == 0) retVal = -1 * CanList.CompareTo((((ArchiveType)value).CanList));
			return retVal;
		}
		
		internal class sortbyextension : IComparer<ArchiveType>
    	{
    		public int Compare(ArchiveType x, ArchiveType y)
        	{
    			ArchiveType ct1 = (ArchiveType)(x);
    			ArchiveType ct2 = (ArchiveType)(y);
    			return ct1.extension.CompareTo(ct2.extension);      		
        	}            
   		}
		
		public static IComparer<ArchiveType> SortByExtension
        {
            get
            {
                return( new sortbyextension());
            }
        }
			
		public void InitArchiveType(string anArchiver, string anExtension, Int32 anArchiveGroup,  
		                            bool isCanList, bool isCanExtract, bool isCanAdd, bool isCanUpdate, string anAddDescription)
		{
			int idx = anExtension.IndexOf(",");
			if (idx == -1)
				extension = anExtension;
			else
			{
				extension = anExtension.Substring(0,idx);
				additionalextensions = anExtension.Substring(idx+1);
			}
			
			archiver = anArchiver;
			additionaldescription = anAddDescription;
			archivegroup = anArchiveGroup;
			canList = isCanList;
			canExtract = isCanExtract;
			canAdd = isCanAdd;
			canUpdate = isCanUpdate;
		}
		
		public ArchiveType(string anArchiver, string anExtension, Int32 anArchiveGroup, bool isCanList, 
		                   bool isCanExtract, bool isCanAdd, bool isCanUpdate)
		{
			InitArchiveType(anArchiver, anExtension, anArchiveGroup,isCanList, isCanExtract, isCanAdd, isCanUpdate, "");
		}
		
		public ArchiveType(string anArchiver, string anExtension, Int32 anArchiveGroup, bool isCanList, 
		                   bool isCanExtract, bool isCanAdd, bool isCanUpdate, string anAddDescription)
		{
			InitArchiveType(anArchiver, anExtension, anArchiveGroup, isCanList, isCanExtract, isCanAdd, isCanUpdate, anAddDescription);
		}
		
	}
	#endregion
	
	#region Consts	
	/// <summary>
	/// Defined constants.
	/// </summary>
	public static class Defines
	{
		/// <summary>
		/// Defines a list of Error and progress messages.
		/// </summary>
		public static ErrorMessage[] ErrorMessages = {
		  #region Error codes		  
			new ErrorMessage(0000,"NOERROR","OK, No error detected"),
			new ErrorMessage(9999,"UNDEFERROR","Unknown error"),
			new ErrorMessage(5001,"MALFUNCARC","Malfunctional Archive for detail please see - http://www.quickzip.org/forums/viewtopic.php?t=1328"),
			new ErrorMessage(2999,"PROCESSFAIL","Processing {0}... Fail"),		
			new ErrorMessage(2000,"PROCESS","Processing {0}... "),
			new ErrorMessage(2001,"PROCESSOK","Processing {0}... OK"),
			new ErrorMessage(2100,"PROCESSLIST","Listing {0}"),
			new ErrorMessage(2101,"PROCESSEXTR","Extracting {0}"),
			new ErrorMessage(2102,"PROCESSTEST","Testing {0}"),
			new ErrorMessage(2103,"PROCESSADD","Compressing {0}"),
			new ErrorMessage(2104,"PROCESSREADD","Re-Compressing {0}"),
			new ErrorMessage(2105,"PROCESSDEL","Deleting {0}"),
			new ErrorMessage(2106,"PROCESSREPAIR","Repairing {0}"),
			new ErrorMessage(2107,"PROCESSCOMMENT","Updating Comment of {0}"),
			new ErrorMessage(2107,"PROCESSSFX","Creating SFX from {0}"),
			new ErrorMessage(3001,"HOTEDIT","File with path, cannot HotEdit"),
			new ErrorMessage(1001,"DLLERROR","Internal error (related to dll)"),
			new ErrorMessage(1002,"NOTEXIST","File does not exist"),
			new ErrorMessage(1003,"PASSWORD","Wrong password"),
			new ErrorMessage(1004,"SFXHEADER","Problem copying SFX header"),
			new ErrorMessage(1005,"NOSUPPORT","Method not support"),
			new ErrorMessage(1006,"LISTERROR","Could not open {0} for listing."),
			new ErrorMessage(1007,"DLLNOTFOUND","Internal error (required dll not found)"),
			new ErrorMessage(1008,"CANTREAD","Can`t Read"),
			new ErrorMessage(1009,"EXIST","File already exist"),
			new ErrorMessage(1010,"USERSKIP","User Skip"),
			new ErrorMessage(1011,"NOMEMORY","Not enough memory"),
			new ErrorMessage(1012,"NOTSPECIFY","No file specified."),
			new ErrorMessage(1013,"NOTFOUNDARC","Cant find Archive file"),
			new ErrorMessage(1014,"NODISKSPACE","Out of Disk space"),
			new ErrorMessage(1015,"CANTWRITE","Cannot write "),
			new ErrorMessage(1016,"CRC","CRC Error"),
			new ErrorMessage(1017,"READERROR","Error reading from archive."),
			new ErrorMessage(1018,"OPENED","File Opened"),
			new ErrorMessage(1019,"VOLUMECHANGE","Volume change {0}"),
			new ErrorMessage(1020,"READONLY","Read Only"),
			new ErrorMessage(1021,"UNKTYPE","Unknown File type"),
			new ErrorMessage(1022,"LONGFN","This archive type dont support long filename"),
			new ErrorMessage(1023,"WRONGVER","Wrong Version"),
			new ErrorMessage(1024,"NEWER","Current file are newer"),
			new ErrorMessage(1025,"TOOMANYFILE","Too Many files"),
			new ErrorMessage(1026,"MAKEDIR","Require make directory (Error)"),
			new ErrorMessage(1027,"HUFFAN","Huffan code"),
			new ErrorMessage(1028,"HEADER","Problem reading comment header"),
			new ErrorMessage(1029,"CRCHEADER","Problem reading CRC header"),
			new ErrorMessage(1030,"HEADERBROKE","Header broken"),
			new ErrorMessage(1031,"NOTARC","Not an archive"),
			new ErrorMessage(1032,"WRONGTYPE","Wrong file style"),
			new ErrorMessage(1033,"WRONGCMD","Wrong command name"),
			new ErrorMessage(1034,"MOREHEAP","More heap memory"),
			new ErrorMessage(1035,"RUNNING","Already Running"),
			new ErrorMessage(1036,"HARC","HARC havent opened"),
			new ErrorMessage(1037,"SEARCH","Not in search mode"),
			new ErrorMessage(1038,"TIMESTAMP","Wrong timestamp"),
			new ErrorMessage(1039,"ARCREADONLY","Archive read only"),
			new ErrorMessage(1040,"TMPOPEN","Unknown error ERROR_TMP_OPEN"),
			new ErrorMessage(1041,"SAMENAME","File with same name"),
			new ErrorMessage(1042,"NORESPONSE","Unknown error ERROR_RESPONSE_READ"),
			new ErrorMessage(1043,"NOTVALID","Not valid filename'"),
			new ErrorMessage(1044,"COPYTEMP","Unable to copy to temp"),
			new ErrorMessage(1045,"EOF","End of file"),
			new ErrorMessage(1046,"CANTCREATE","File could not be created."),
			new ErrorMessage(1047,"DISKFULL","Disk Full"),
			new ErrorMessage(1048,"AVERROR","AV of archive invalid or does not match to yours"), //Ace related error					
			new ErrorMessage(1049,"CANTOPEN","Cannot Open {0}"), //Ace related error	
			new ErrorMessage(1050,"UNKREQ","Unknown Request"), //Ace related error
			new ErrorMessage(1051,"CANTCLOSE","File close error."), //Rar related error
		  #endregion	
		};
		
		#region Locate ErrorMessage		
		/// <summary>
		/// Return a error message based on error code.
		/// </summary>
		/// <param name="code">Error code.</param>
		/// <returns>ErrorMessage based on the code.</returns>
		public static ErrorMessage ErrorMessage(Int32 code)
		{
			foreach (ErrorMessage err in ErrorMessages)
			{
				if (err.ErrorCode == code)
					return err;
			}
			return ErrorMessage(9999); //Unknown error
		}
		/// <summary>
		/// Return a error message based on error name.
		/// </summary>
		/// <param name="name">Error name.</param>
		/// <returns>ErrorMessage based on the name.</returns>
		public static ErrorMessage ErrorMessage(string name)
		{
			name = name.ToUpper();
			foreach (ErrorMessage err in ErrorMessages)
			{
				if (err.ErrorName == name)
					return err;
			}
			return ErrorMessage(9999); //Unknown error
		}		
		#endregion
		
		private static Hashtable cacheArcDescList = null;
		public static Hashtable ArchiveDescriptionList
		{
			get
			{
			if (cacheArcDescList != null) 
				return cacheArcDescList;
			
			cacheArcDescList = new Hashtable();
			#region Generate List (Not All supported!!)
			cacheArcDescList.Add("7z", "SevenZip archive");
			cacheArcDescList.Add("7zip", "SevenZip archive");			
			cacheArcDescList.Add("ace", "Winace archive");
			cacheArcDescList.Add("arc", "Arc archive");
			cacheArcDescList.Add("arj", "Arj archive");
			cacheArcDescList.Add("asd", "Asd archive");
			cacheArcDescList.Add("b64", "B64 Encoded (MIME)");
			cacheArcDescList.Add("bel", "Belon archive");
			cacheArcDescList.Add("bin", "Disk Image");
			cacheArcDescList.Add("bsz", "BS Player skin");
			cacheArcDescList.Add("bz2", "BZIP2");
			cacheArcDescList.Add("bza", "BZIP2 archive");
			cacheArcDescList.Add("bh", "Blackhole archive");
			cacheArcDescList.Add("cab", "Microsoft Cabinet,Pocket PC Install Cabinet,Install Shield Setup Cabinet");
			cacheArcDescList.Add("cpio","Unix archive");
			cacheArcDescList.Add("cpt","Macintosh Compressed Pro");
			cacheArcDescList.Add("czip","Crypto Zip");
			cacheArcDescList.Add("deb","Debian Unix archive");
			cacheArcDescList.Add("dmg","Macintosh Disk Image file");
			cacheArcDescList.Add("ear", "Java archive");
			cacheArcDescList.Add("f", "Freeze Unix archive");
			cacheArcDescList.Add("gca", "G Compression Archive");			
			cacheArcDescList.Add("gz", "Tar archive");
			cacheArcDescList.Add("hog", "Descent Hog");
			cacheArcDescList.Add("hqx", "Macintosh BinHex");			
			cacheArcDescList.Add("icl", "Icon Library");
			cacheArcDescList.Add("ico", "Icon");
			cacheArcDescList.Add("img", "Disk Image");
			cacheArcDescList.Add("imp", "WinIMP archive");	
			cacheArcDescList.Add("ishield", "Install Shield Setup");				
			cacheArcDescList.Add("iso", "Disk Image");
			cacheArcDescList.Add("jam", "Jam archive");
			cacheArcDescList.Add("jar", "Java archive");
			cacheArcDescList.Add("lha", "Lha archive");
			cacheArcDescList.Add("lzh", "Lha archive");
			cacheArcDescList.Add("macbin", "StuffIt archive");			
			cacheArcDescList.Add("mskin", "Maxthon skin");			
			cacheArcDescList.Add("nrg", "Nero Disk Image");
			cacheArcDescList.Add("pak", "Quake Pak/Wad");
			cacheArcDescList.Add("pit", "Macintosh PackIt");			
			cacheArcDescList.Add("pk3", "Quake 3 Pak");
			cacheArcDescList.Add("psf", "PA skin");
			cacheArcDescList.Add("qze", "Quick Zip Encoded");
			cacheArcDescList.Add("r00", "Rar archive");
			cacheArcDescList.Add("r01", "Rar archive");
			cacheArcDescList.Add("rar", "Rar archive");
			cacheArcDescList.Add("rs", "Resource archive");	
			cacheArcDescList.Add("rpm","Redhat package");
			cacheArcDescList.Add("shar","Unix archive");
			cacheArcDescList.Add("sqx", "Sqx archive");
			cacheArcDescList.Add("tar", "Tar archive");
			cacheArcDescList.Add("tgz", "Tar archive");
			cacheArcDescList.Add("uue", "Uue Encoded");
			cacheArcDescList.Add("wad", "Quake Pak/Wad");
			cacheArcDescList.Add("wal", "WinAmp Skin");
			cacheArcDescList.Add("war", "Java archive");
			cacheArcDescList.Add("wmz", "Windows Media Player Skin");
			cacheArcDescList.Add("wsz", "Zip archive");
			cacheArcDescList.Add("xpi", "Firefox extension");
			cacheArcDescList.Add("xxe", "Xxe Encoded");
			cacheArcDescList.Add("yenc", "yEnc encoded file");
			cacheArcDescList.Add("yz1", "DeepFreezer Compressed Archive");
			cacheArcDescList.Add("z", "Tar archive");
			cacheArcDescList.Add("zip", "Zip archive");
			cacheArcDescList.Add("zoo", "Zoo archive");
			#endregion			
			return cacheArcDescList;
			}
		}			
	}
	
	public class ShellClipboardFormats
	{
		public const string CFSTR_SHELLIDLIST = "Shell IDList Array";
		public const string CFSTR_SHELLIDLISTOFFSET = "Shell Object Offsets";
		public const string CFSTR_NETRESOURCES = "Net Resource";
		public const string CFSTR_FILEDESCRIPTORA = "FileGroupDescriptor";
		public const string CFSTR_FILEDESCRIPTORW = "FileGroupDescriptorW";
		public const string CFSTR_FILECONTENTS = "FileContents";
		public const string CFSTR_FILENAMEA = "FileName";
		public const string CFSTR_FILENAMEW = "FileNameW";
		public const string CFSTR_PRINTERGROUP = "PrinterFreindlyName";
		public const string CFSTR_FILENAMEMAPA = "FileNameMap";
		public const string CFSTR_FILENAMEMAPW = "FileNameMapW";
		public const string CFSTR_SHELLURL = "UniformResourceLocator";
		public const string CFSTR_INETURLA = CFSTR_SHELLURL;
		public const string CFSTR_INETURLW = "UniformResourceLocatorW";
		public const string CFSTR_PREFERREDDROPEFFECT = "Preferred DropEffect";
		public const string CFSTR_PERFORMEDDROPEFFECT = "Performed DropEffect";
		public const string CFSTR_PASTESUCCEEDED = "Paste Succeeded";
		public const string CFSTR_INDRAGLOOP = "InShellDragLoop";
		public const string CFSTR_DRAGCONTEXT = "DragContext";
		public const string CFSTR_MOUNTEDVOLUME = "MountedVolume";
		public const string CFSTR_PERSISTEDDATAOBJECT = "PersistedDataObject";
		public const string CFSTR_TARGETCLSID = "TargetCLSID";
		public const string CFSTR_LOGICALPERFORMEDDROPEFFECT = "Logical Performed DropEffect";
		public const string CFSTR_AUTOPLAY_SHELLIDLISTS = "Autoplay Enumerated IDList Array";

	}
	#endregion

	#region Event Handler	
	/// <summary>
	/// ItemListEventArgs contain a event data related to item listing.
	/// </summary>
	public class ItemListEventArgs : EventArgs
	{
           private CakArchiver pArchiver;
           private ContentType pContent;
           /// <summary>
           /// Gets the calling archiver.
           /// </summary>
           public CakArchiver Archiver { get { return pArchiver; } }
           /// <summary>
           /// Gets current listed file in archive.
           /// </summary>
           public ContentType Content { get { return pContent; } }
           
           /// <summary>
           /// Initalize a new instance of ItemListEventArgs type.
           /// </summary>
           /// <param name="anArchiver">Calling archiver</param>
           /// <param name="aContent">File.</param>
           public ItemListEventArgs(CakArchiver anArchiver, ContentType aContent)
           {
               this.pArchiver = anArchiver;
               this.pContent = aContent;
           }
    }
	
	/// <summary>
	/// MessageEventArgs contain a event data related to progress message.
	/// </summary>
	public class MessageEventArgs : EventArgs
	{
           private CakArchiver pArchiver;
           private string pMessage;
           
           /// <summary>
           /// Gets the calling archiver.
           /// </summary>
           public CakArchiver Archiver { get { return pArchiver; } }
           /// <summary>
           /// Gets the progress Message.
           /// </summary>
           public string Message { get { return pMessage; } }
           
           /// <summary>
           /// Initalize a new instance of MessageEventArgs type.
           /// </summary>
           /// <param name="anArchiver">Calling archiver.</param>
           /// <param name="aMessage">Message.</param>
           public MessageEventArgs(CakArchiver anArchiver, string aMessage)
           {
               this.pArchiver = anArchiver;
               this.pMessage = aMessage;
           }
    }
	
	/// <summary>
	/// ErrorEventArgs contain a event data related to error message.
	/// </summary>
	public class ErrorEventArgs : EventArgs
	{
		private CakArchiver pArchiver;
		private ErrorMessage pError;
		
		/// <summary>
		/// Gets the calling archiver.
		/// </summary>
		public CakArchiver Archiver { get { return pArchiver; } }
		/// <summary>
		/// Gets the error message.
		/// </summary>
		public ErrorMessage Error { get { return pError; } }		
		/// <summary>
		/// Initalize a new instance of ErrorEventArgs type.
		/// </summary>
		/// <param name="anArchiver">Calling archiver.</param>
		/// <param name="anError">Error message.</param>
		public ErrorEventArgs(CakArchiver anArchiver, ErrorMessage anError)
		{
			this.pArchiver = anArchiver;
			this.pError = anError;
		}
		/// <summary>
		/// Initalize a new instance of ErrorEventArgs type.
		/// </summary>
		/// <param name="anArchiver">Calling archiver.</param>
		/// <param name="anError">Error code.</param>
		public ErrorEventArgs(CakArchiver anArchiver, Int32 anError)
		{
			this.pArchiver = anArchiver;
			this.pError = Defines.ErrorMessage(anError);
		}
		/// <summary>
		/// Initalize a new instance of ErrorEventArgs type.
		/// </summary>
		/// <param name="anArchiver">Calling archiver.</param>
		/// <param name="anError">Error text</param>
		public ErrorEventArgs(CakArchiver anArchiver, string anError)
		{
			this.pArchiver = anArchiver;
			this.pError = Defines.ErrorMessage(anError);
		}
    }
	
	/// <summary>
	/// ProgressEventArgs contain a event data related to progress message.
	/// </summary>
	public class ProgressEventArgs : EventArgs
	{
		private CakArchiver pArchiver;
		private UInt16 pPercent;
		private string pFilename;
		
		/// <summary>
        /// Gets the calling archiver.
        /// </summary>
		public CakArchiver Archiver { get { return pArchiver; } }
		/// <summary>
		/// Get percentage completed.
		/// </summary>
        public UInt16 Percent { get { return pPercent; } }      
        /// <summary>
        /// Get current processing file name.
        /// </summary>
        public string Filename { get { return pFilename; } }
        
        /// <summary>
        /// Initalize a new instance of ProgressEventArgs type.
        /// </summary>
        /// <param name="anArchiver">Calling archiver.</param>
        /// <param name="aPercent">Percentage completed</param>
        /// <param name="aFilename">File name.</param>
        public ProgressEventArgs(CakArchiver anArchiver, UInt16 aPercent, string aFilename)
        {
        	this.pArchiver = anArchiver;
        	this.pPercent = aPercent;
        	this.pFilename = aFilename;
        }
	}
	
	/// <summary>
	/// OverwriteEventArgs contain a event data related to overwrite confirmation.
	/// </summary>
	public class OverwriteEventArgs : EventArgs
	{
		private CakArchiver pArchiver;		
		private string pFilename;		
		
		/// <summary>
        /// Gets the calling archiver.
        /// </summary>
		public CakArchiver Archiver { get { return pArchiver; } }        
		/// <summary>
		/// Gets the file name that need to be confirmed.
		/// </summary>
        public string Filename { get { return pFilename; } }
        /// <summary>
        /// Gets or sets whether file should be overwrited.
        /// </summary>
        public bool Overwrite;
        
        /// <summary>
        /// Inisalize a new instance of OverwriteEventArgs type.
        /// </summary>
        /// <param name="anArchiver">Calling archiver.</param>
        /// <param name="aFilename">Processing file name.</param>
        public OverwriteEventArgs(CakArchiver anArchiver, string aFilename)
        {
        	this.pArchiver = anArchiver;        	
        	this.pFilename = aFilename;
        	this.Overwrite = false;
        }
        
        /// <summary>
        /// Inisalize a new instance of OverwriteEventArgs type.
        /// </summary>
        /// <param name="anArchiver">Calling archiver.</param>
        /// <param name="aFilename">Processing file name.</param>
        /// <param name="anOverwrite">Default overwrite mode.</param>
        public OverwriteEventArgs(CakArchiver anArchiver, string aFilename, bool anOverwrite)
        {
        	this.pArchiver = anArchiver;        	
        	this.pFilename = aFilename;
        	this.Overwrite = anOverwrite;
        }
	}
	
	/// <summary>
	/// PasswordEventArgs contain a event data related to password request.
	/// </summary>
	public class PasswordEventArgs : EventArgs
	{
		private CakArchiver pArchiver;		
		private string pFilename;		
		
		/// <summary>
        /// Gets the calling archiver
        /// </summary>
		public CakArchiver Archiver { get { return pArchiver; } }     
		/// <summary>
		/// Gets the file name that is encrypted.
		/// </summary>
        public string Filename { get { return pFilename; } }
        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        public string Password;
        
        /// <summary>
        /// Inistalize a new instance of PasswordEventArgs type.
        /// </summary>
        /// <param name="anArchiver">Calling archiver.</param>
        /// <param name="aFilename">File name of the encrypted file.</param>
        public PasswordEventArgs(CakArchiver anArchiver, string aFilename)
        {
        	this.pArchiver = anArchiver;        	
        	this.pFilename = aFilename;
        	this.Password = "";
        }
        
        /// <summary>
        /// Inistalize a new instance of PasswordEventArgs type.
        /// </summary>
        /// <param name="anArchiver">Calling archiver.</param>
        /// <param name="aFilename">File name of the encrypted file.</param>
        /// <param name="aPassword">Default password</param>
        public PasswordEventArgs(CakArchiver anArchiver, string aFilename, string aPassword)
        {
        	this.pArchiver = anArchiver;        	
        	this.pFilename = aFilename;
        	this.Password = "";
        }
	}
	
	/// <summary>
	/// Represents the method that will handle an event that list an item. 
	/// </summary>
	public delegate void ItemListEventHandler( Object sender, ItemListEventArgs e);
	/// <summary>
	/// Represents the method that will handle an event that output a message. 
	/// </summary>
	public delegate void MessageEventHandler( Object sender, MessageEventArgs e);
	/// <summary>
	/// Represents the method that will handle an event that report an error. 
	/// </summary>
	public delegate void ErrorEventHandler( Object sender, ErrorEventArgs e);
	/// <summary>
	/// Represents the method that will handle an event that report a progress. 
	/// </summary>
	public delegate void ProgressEventHandler( Object sender, ProgressEventArgs e);	
	/// <summary>
	/// Represents the method that will handle an event that request overwrite confirm. 
	/// </summary>
	public delegate void OverwriteEventHandler( Object sender, ref OverwriteEventArgs e);	
	/// <summary>
	/// Represents the method that will handle an event that request file password. 
	/// </summary>
	public delegate void PasswordEventHandler( Object sender, ref PasswordEventArgs e);					
	#endregion	
	
	}
