/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 16/8/2007
 * Time: 17:40
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.IconLib;
using System.Drawing.IconLib.EncodingFormats;
using System.IO;
using Cake3;
using System.Diagnostics;

namespace Cake3.IconLib 
{
	/// <summary>
	/// Description of ICLArchiver.
	/// </summary>
	public class ICLArchiver : CakArchiver
	{
		internal override string name {	get { return "iconlib"; }}
		MultiIcon   mIcon;		
		private static string DllName = "IconLib.dll";
		
			
		public ICLArchiver() : base()
		{
		}
			
		public override List<ArchiveType> SupportedArchiveTypes()
		{
			List<ArchiveType> retVal = new List<ArchiveType>();			
			retVal.Add(new ArchiveType(name, "icl,ico", 0, true, true, false, false));
			return retVal;
		}
		
		public override bool Open()
		{
			if (archiveOpened) 
				return false;
			
			mIcon = new MultiIcon();
			mIcon.Load(ArchiveName);
				
			archiveOpened = true;
			return true;

		}
		
		public override bool Close()
		{
			if (!archiveOpened) 
				return false;									
			
			mIcon = null;
			
			archiveOpened = false;
			
			return true;
		}
		
		private string GetFriendlyBitDepth(PixelFormat pixelFormat)
        {
            switch(pixelFormat)
            {
                case PixelFormat.Format1bppIndexed:
                    return "1-bit B/W";
                case PixelFormat.Format24bppRgb:
                    return "24-bit True Colors";
                case PixelFormat.Format32bppArgb:
                case PixelFormat.Format32bppRgb:
                    return "32-bit Alpha Channel";
                case PixelFormat.Format8bppIndexed:
                    return "8-bit 256 Colors";
                case PixelFormat.Format4bppIndexed:
                    return "4-bit 16 Colors";
            }
            return "Unknown";
        }
						
		
		public string GetImageFilename(SingleIcon singleIcon, IconImage iconImage)
		{
			return singleIcon.Name + " (" + iconImage.Size.Width + "x" + 
						iconImage.Size.Height + " " + GetFriendlyBitDepth(iconImage.PixelFormat) + ").ico";
		}

		
		public override bool List(string mask)
		{			
			if (mIcon == null) return false;
			DispMessage(Defines.ErrorMessage("PROCESSLIST"), archiveName);	
			DateTime fileTime = Utils.GetFileInfo(ArchiveName).LastWriteTime;
			
			foreach(SingleIcon singleIcon in mIcon)
			{				
				string filename1 = "\\" + singleIcon.Name + ".ico";	
				MemoryStream ms = new MemoryStream();
				singleIcon.Icon.Save(ms);				
				
				OnItemList(this, new ItemListEventArgs(this,
					                 new ContentType(ArchiveName, (UInt32)ms.Length, (UInt32)ms.Length,
				                     fileTime, (long)Utils.GetCRC(ms), filename1, false) ));
				ms.Close();
			}
			return true;
		}
		
		internal override void setArchiveName(string anArchive)
		{
			base.setArchiveName(anArchive);
			
			canList = true;
			canExtract = true;
			canAlarmOverwrite = true;
		}
				
		public override bool Extract(ExtractOptions options)
		{
			Open();
			if (mIcon == null) return false;		
			DispMessage(Defines.ErrorMessage("PROCESSEXTR"), archiveName);	
			
			foreach(SingleIcon singleIcon in mIcon)
			{
				string file1 = singleIcon.Name + ".ico";	
				string filename1 = "\\" + file1;
				if (Utils.MatchMasks(filename1, options.extractItem))
				{
					string destname1 = Utils.AppendSlash(options.ExtractFolderParsed) + file1;
					if (options.allowFolder)
					    destname1 = Utils.AppendSlash(options.ExtractFolderParsed) + "\\" + file1;
					Utils.MakeDirectory(Utils.ExtractFilePath(destname1));
					try
					{
						if (Utils.FileExists(destname1))
						{
							OverwriteEventArgs e = new OverwriteEventArgs(this, filename1);
							if (OnOverwrite != null)
								OnOverwrite(this, ref e);
							if (e.Overwrite)
								Utils.RemoveFile(destname1);
						}
						
						FileStream fs1 = new FileStream(destname1, FileMode.CreateNew);
						singleIcon.Icon.Save(fs1);
//						singleIcon.Icon.ToBitmap().Save(@"c:\temp\out"".bmp");
						
						fs1.Close();
					}
					catch (Exception ex)
					{
						Debug.WriteLine(ex.Message);
						return false;
					}
				}
			}
			Close();
			return true;
			
		}								
	}
}
