using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Text;
using System.Reflection;

namespace QuickZip.InstallBuilder
{
    public enum Processor
    {
        #region Processor List
        Unknown = 0,
        SHx_SH3 = 103,
        SHx_SH4 = 104,
        Intel_386 = 386,
        Intel_486 = 486,
        Intel_586 = 586,
        Intel_Pentium = 586,
        Emulator = 686,
        PowerPC_601 = 601,
        PowerPC_603 = 603,
        PowerPC_604 = 604,
        PowerPC_620 = 620,
        Motorola_821 = 821,
        ARM_720 = 1824,
        ARM_820 = 2080,
        ARM_920 = 2336,
        StrongArm = 2577,
        MIPS_R2000 = 2000,
        MIPS_R3000 = 3000,
        MIPS_R4000 = 4000,
        Hitachi_SH3 = 10003,
        Hitachi_SH3E = 10004,
        Hitachi_SH4 = 10005,
        Alpha_21064 = 21064,
        ARM_7TDMI = 700001
        #endregion
    }
    public enum Handheld_Shortcut
    {
        #region Shortcut List
        pf=1,                  // \Program Files
        win=2,                 // \Windows
        desktop=3,             // \Windows\Desktop
        startup=4,             // \Windows\StartUp
        mydocu=5,              // \My Documents
        accessories=6,         // \ProgramFiles\Accessories
        communications=7,      // \ProgramFiles\Communications
        games=8,               // \ProgramFiles\Games
        outlook=9,             // \ProgramFiles\Pocket Outlook
        office=10,             // \ProgramFiles\Office
        group=11,              // \Windows\Programs
        winaccessories=12,     // \Windows\Programs\Accessories
        wincommunications=13,  // \Windows\Programs\Communications
        wingames=14,           // \Windows\Programs\Games
        fonts=15,              // \Windows\Fonts
        recent=16,             // \Windows\Recent
        favorites=17           // \Windows\Favorites
        #endregion
    }
    public enum PPC1020
    {
        #region Shortcut List
        pf = 1,                  // \Program Files
        win = 2,                 // \Windows        
        startup = 4,             // \Windows\StartUp
        mydocu = 5,              // \My Documents
        accessories = 6,         // \ProgramFiles\Accessories
        communications = 7,      // \ProgramFiles\Communications
        games = 8,               // \ProgramFiles\Games
        group = 11,              // \Windows\StartMenu\Programs 
        grp_accessories = 12,    // \Windows\StartMenu\Accessories
        grp_communications = 13, // \Windows\StartMenu\Communications 
        grp_games = 14,          // \Windows\StartMenu\Games 
        fonts = 15,              // \Windows\Fonts
        startmenu = 17           // \Windows\StartMenu         
        //desktop=3,             // \Windows\Desktop
        //outlook=9,             // \ProgramFiles\Pocket Outlook
        //office=10,             // \ProgramFiles\Office
        //recent=16,             // \Windows\Recent
        #endregion    }
    }
    public enum PPC30
    {
        #region Shortcut List
        app = 0,                 // %InstallDir%
        pf = 1,                  // \Program Files
        win = 2,                 // \Windows
        startup = 4,             // \Windows\StartUp
        mydocu = 5,              // \My Documents
        group = 11,              // \Windows\StartMenu\Programs            
        fonts = 15,              // \Windows\Fonts
        startmenu = 17               // \Windows\StartMenu         
        //desktop=3,             // \Windows\Desktop
        //accessories = 6,       // \ProgramFiles\Accessories
        //communications = 7,    // \ProgramFiles\Communications
        //games = 8,             // \ProgramFiles\Games
        //outlook=9,               // \ProgramFiles\Pocket Outlook
        //office=10,               // \ProgramFiles\Office
        //grp_accessories = 12,    // \Windows\StartMenu\Accessories
        //grp_communications = 13, // \Windows\StartMenu\Communications 
        //grp_games = 14,          // \Windows\StartMenu\Games 
        //recent=16,               // \Windows\Recent
        #endregion    }
    }

    public enum RegistryRoot
    {
        HKCR=1,
        HKCU=2,
        HKLM=3,
        HKU=4
    }

    public enum RegistryDataType
    {
        TYPE_DWORD = 0x00010001,
        TYPE_SZ    = 0x00000000,
        TYPE_MULTI_SZ = 0x00001000,
        TYPE_BINARY = 0x00000001,
        NOCLOBBER = 0x00000002
    }
    public enum LinkType
    {
        directory=0,
        file=1
    }

    public enum KeyDataType
    {
        _dword = 0x00010001,
        _string = 0x00000000,
        _multistring = 0x00001000,
        _binary = 0x00000001
    }

    public enum SetupFileType { Encoded, Xml, EncodedXml }

    public class Tools
    {
        public static SetupFileType Str2SetupFileType(string value)
        {
            String Proc = value.ToLower();
            if (Proc == "encodedxml") { return SetupFileType.EncodedXml; }
            if (Proc == "encoded") { return SetupFileType.Encoded; }
            if (Proc == "xml") { return SetupFileType.Xml; }

            return SetupFileType.EncodedXml;
        }
        public static Processor Str2ProcessorType(string value)
        {
            String Proc = value.ToLower();
            if (Proc == "unknown") { return Processor.Unknown; }
            if (Proc == "strongarm") { return Processor.StrongArm; }
            if (Proc == "emulator") { return Processor.Emulator; }
            if (Proc == "intel_386") { return Processor.Intel_386; }
            if (Proc == "intel_486") { return Processor.Intel_486; }
            if (Proc == "intel_586") { return Processor.Intel_586; }
            if (Proc == "intel_pentium") { return Processor.Intel_586; }
            if (Proc == "intel_686") { return Processor.Emulator; }
            if (Proc == "hitachi_sh3") { return Processor.Hitachi_SH3; }
            if (Proc == "hitachi_sh4") { return Processor.Hitachi_SH4; }
            if (Proc == "mips_r2000") { return Processor.MIPS_R2000; }
            if (Proc == "mips_r3000") { return Processor.MIPS_R3000; }
            if (Proc == "mips_r4000") { return Processor.MIPS_R4000; }
            if (Proc == "arm_720") { return Processor.ARM_720; }            
            if (Proc == "arm_820") { return Processor.ARM_820; }
            if (Proc == "arm_920") { return Processor.ARM_920; }
            if (Proc == "arm_7tdmi") { return Processor.ARM_7TDMI; }
            if (Proc == "hitachi_sh3") { return Processor.Hitachi_SH3; }
            if (Proc == "hitachi_sh3e") { return Processor.Hitachi_SH3E; }
            if (Proc == "hitachi_sh4") { return Processor.Hitachi_SH4; }
            if (Proc == "alpha_21064") { return Processor.Alpha_21064; }
            return Processor.Unknown;
        }
        public static RegistryRoot Str2RegistryRoot(String value)
        {
            String Root = value.ToUpper();
            if (Root == "HKCR") { return RegistryRoot.HKCR; }
            if (Root == "HKCU") { return RegistryRoot.HKCU; }
            if (Root == "HKLM") { return RegistryRoot.HKLM ; }
            if (Root == "HKU") { return RegistryRoot.HKU; }
            return RegistryRoot.HKCU;
        }
        public static Int32 Str2KeyDataType(String value)
        {
            String Root = value.ToUpper();
            if (Root == "DWORD") { return 0x00010001; }
            if (Root == "STRING") { return 0x00000000; }
            if ((Root == "MULTI_SZ") || (Root == "MULTIPLESTRING")) { return 0x00001000; }
            if (Root == "BINARY") { return 0x00000001; }
            throw (new FormatException(value + " is not a valid data type. (dword, string, multiplestring and binary)"));
        }

        public static String GetDosFilename(String filename, Int16 FileNumber)
        {
            String fn = filename.Substring(0, filename.IndexOf("."));
            if (fn.Length > 6)
            {
                fn = fn.Substring(0, 6);
            }

            return System.String.Format("{0,6:000000}~1.{1,3:000}", fn, FileNumber).Replace(" ", "0");
        }

        public static String ExtractFileName(String filepath)
        {
            Int32 pos = filepath.LastIndexOf("\\");
            if ((pos == -1) || (pos == filepath.Length))
            {
                return filepath;
            }
            else
            {
                return filepath.Substring(pos + 1);
            }
        }

        public static String ExtractFilePath(String filepath)
        {
            Int32 pos = filepath.LastIndexOf("\\");
            if ((pos == -1) || (pos == filepath.Length))
            {
                return "";
            }
            else
            {
                return filepath.Substring(0,pos);
            }
        }

        public static String Appendslash(String filepath)
        {
            if (filepath == "")
            {
                return "";
            }

            if (filepath[filepath.Length - 1] == '\\')
            {
                return filepath;
            }
            else
            {
                return filepath + '\\';
            }
        }

        public static String AppPath
        {
            get
            {
                String path = System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase);
                path = path.Replace(@"file:\", "");
                return Appendslash(path);
            }

        }

        public static Color Binary2Color(Byte[] value)
        {
            if ((value.Length == 4) && (value[3] == 0))
            {
                Int32 R = value[0];
                Int32 G = value[1];
                Int32 B = value[2];

                return Color.FromArgb(R, G, B);                
            }

            return Color.Black;
        }

        public static Byte[] Color2Binary(Color value)
        {
            Byte[] retVal = new Byte[4];
            retVal[0] = value.R;
            retVal[1] = value.G;
            retVal[2] = value.B;
            retVal[3] = 0;

            return retVal;            
        }

        public static void Test()
        {
            BinaryReader reader = new BinaryReader(new FileStream(@"C:\PPC\NightLake VGA\KMIPAQ~1.000", FileMode.Open));
            Encoded_INF project = new Encoded_INF();
            project.Read(reader);
            Describe_INF describe = new Describe_INF(project.Data);
            describe.DebugClass();
            reader.Close();
        }


    }
 
}
