using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.Xml;
using QuickZip.InstallBuilder;
using QuickZip.ThemeBuilder;

namespace QuickZip.InstallBuilder.Reader
{   
    /// <summary>
    /// Main Reader class
    /// </summary>
    public class InfReader
    {
        private String pPath;
        private Encoded_INF InstallData;
        
        public InfReader(string path)
        {
            pPath = path;
            InstallData = new Encoded_INF();

        }
        

        public bool Read()
		{
            try
            {
                FileStream fs = new FileStream(pPath, System.IO.FileMode.Open);
                BinaryReader reader = new BinaryReader(fs, Encoding.ASCII);

                InstallData.Read(reader);
                InstallData.DebugClass();
                              

                Describe_INF DescribeData = new Describe_INF(InstallData.Data);
                DescribeData.DebugClass();

                Debug.WriteLine("fs.Position="+fs.Position);
                Debug.Flush();
                return true;
            }
            catch
            {
                //return false;
            }
            return true;
            
		}
        public void Write(String path)
        {
            FileStream fs = new FileStream(path, System.IO.FileMode.Create);
            BinaryWriter writer = new BinaryWriter(fs, Encoding.ASCII);

            InstallData.Write(writer);

            writer.Flush();
            writer.Close();
            fs.Close();


            fs = new FileStream(path+".inno", System.IO.FileMode.Create);
            StreamWriter writer1 = new StreamWriter(fs);
            InnoProject inno = new InnoProject(InstallData.Data);
            inno.Write(writer1);
            writer1.Flush();
            writer1.Close();
            fs.Close();
        }

    }

    public class InnoReader
    {
        private String pPath;
        private InnoProject InstallData;

        public InnoReader(string path)
        {
            pPath = path;
            InstallData = new InnoProject();           
        }

        public bool Read()
        {
            FileStream fs = new FileStream(pPath, System.IO.FileMode.Open);;
            try
            {                
                StreamReader reader = new StreamReader(fs, Encoding.ASCII);

                InstallData.Read(reader);
                InstallData.DebugClass();

                //Encoded_INF instData = new Encoded_INF(InstallData.Data);
                //instData.DebugClass();

                //Describe_INF DescribeData = new Describe_INF(InstallData.Data);
                //DescribeData.DebugClass();               
            }
            finally
            {
                if (fs != null)
                    fs.Close();
            }
            return true;
        }

        

        public void ThemeWriteInno(string path)
        {
            BinaryReader reader;
            FileStream fs;

            Encoded_INF InstData = new Encoded_INF();
            fs = new FileStream(path, System.IO.FileMode.Open);
            reader = new BinaryReader(fs, Encoding.ASCII);

            InstData.Read(reader);
            InstallData.Data = InstData.Data;
            ThemeData();
            reader.Close();
            fs.Close();

            fs = new FileStream(@"c:\temp\theme.inno", System.IO.FileMode.Create);
            StreamWriter writer = new StreamWriter(fs, Encoding.ASCII);
            InstallData.Write(writer);
            writer.Flush();
            fs.Close();

        }

        public void Write(String path)
        {            
            FileStream fs = new FileStream(path, System.IO.FileMode.Create);
            BinaryWriter writer = new BinaryWriter(fs);
            Encoded_INF instData = new Encoded_INF(InstallData.Data);
            
            instData.Write(writer);
            writer.Flush();
            writer.Close();
            fs.Close();
        }

        public void WriteInno(string path)
        {
            FileStream fs = new FileStream(path, System.IO.FileMode.Create);
            StreamWriter writer = new StreamWriter(fs, Encoding.ASCII);
            InstallData.Write(writer);
            writer.Flush();
            fs.Close();

        }

        public void WriteXML(String path)
        {
            FileStream fs = new FileStream(path, System.IO.FileMode.Create);
            XmlWriterSettings settings = new XmlWriterSettings();
            XmlTextWriter writer = new XmlTextWriter(fs, new UTF8Encoding(false));
            
            XmlProject xmlData = new XmlProject(InstallData.Data);
            
            xmlData.Write(writer);
            
            writer.Flush();
            writer.Close();            

            fs.Close();
        }

        public void CopyTempFiles(String path)
        {
            for (Int16 i = 0; i < InstallData.Data.Files.Count; i++)
            {
                FileEntry entry = InstallData.Data.Files.File(i);
                String FromFile = entry.SourcePath;
                String ToFile = entry.ShortPath;
                if (!File.Exists(FromFile))
                {
                    throw new FileNotFoundException(FromFile + " Not Found!", FromFile);
                }

                File.Delete(Tools.Appendslash(path) + ToFile);
                File.Copy(FromFile, Tools.Appendslash(path) + ToFile);                
            }

        }

        public void DelTempFiles(String path, String setup, String XmlSetup)
        {
            for (Int16 i = 0; i < InstallData.Data.Files.Count; i++)
            {
                FileEntry entry = InstallData.Data.Files.File(i);
                String DelFile = entry.ShortPath;
                if (File.Exists(Tools.Appendslash(path) + DelFile))
                {
                    File.Delete(Tools.Appendslash(path) + DelFile);
                }
            }

            if (File.Exists(Tools.Appendslash(path) + setup))
            {
                File.Delete(Tools.Appendslash(path) + setup);
            }
            if (File.Exists(Tools.Appendslash(path) + XmlSetup))
            {
                File.Delete(Tools.Appendslash(path) + XmlSetup);
            }
        }

        public void CreateCab(String path, String setup, String setupXML, String cab)
        {
            Process CabArc = new Process();
            String flist = "";
            if (InstallData.SetupFile != SetupFileType.Xml) flist = " \"" + Tools.Appendslash(path) + setup + "\"";
            if (InstallData.SetupFile != SetupFileType.Encoded) flist = flist + " \"" + Tools.Appendslash(path) + setupXML + "\"";

            for (Int16 i = 0; i < InstallData.Data.Files.Count; i++)
            {
                FileEntry entry = InstallData.Data.Files.File(i);
                flist += " \"" + Tools.Appendslash(path) + entry.ShortPath + "\"";  
            }

            if (!File.Exists(Tools.AppPath + "Cabarc.exe"))
            {
                throw new FileNotFoundException("Cabarc.exe Not Found!, please download it from web.");
            }

            CabArc.StartInfo.FileName = Tools.AppPath + "Cabarc.exe";
            CabArc.StartInfo.Arguments = String.Format("-m NONE n \"{0}\" {1}", cab, flist);
            CabArc.Start();
            CabArc.WaitForExit();

            if (!File.Exists(cab))
            {
                throw new FileNotFoundException("Cannot create cabinet properly!");
            }
        }

        public void WriteIni(String path, String cab, String description)
        {
            FileStream fs = new FileStream(path + "Setup.ini", System.IO.FileMode.Create);
            StreamWriter writer = new StreamWriter(fs);

            writer.WriteLine("[CEAppManager]");
            writer.WriteLine("Version=1.0");
            writer.WriteLine(String.Format("Component={0}", InstallData.ProductName));
            writer.WriteLine(String.Format("[{0}]", InstallData.ProductName));
            writer.WriteLine(String.Format("Description={0}", description));
            writer.WriteLine(String.Format("Uninstall={0}", InstallData.ProductName));
            writer.WriteLine(";Because there are multiple .cab files specific to a CPU type,");
            writer.WriteLine(";these files are relative to the installation directory.");
            writer.WriteLine(";But Install builder do support only StrongARM PPC,");
            writer.WriteLine(";so there's only one cab here.");
            writer.WriteLine(String.Format("Cabfiles={0}", Tools.ExtractFileName(cab)));

            writer.Flush();
            writer.Close();
            fs.Close();

        }

        public void WriteBatch(String path, String setup, String cab )
        {
            FileStream fs = new FileStream(path, System.IO.FileMode.Create);
            StreamWriter writer = new StreamWriter(fs);
            String AddList = "";

            for (Int16 i = 0; i < InstallData.Data.Files.Count; i++)
            {
                FileEntry entry = InstallData.Data.Files.File(i);
                String renameCmd = System.String.Format(
                    "Rename {0} {1}", entry.SourcePath, entry.ShortPath);
                writer.WriteLine(renameCmd);
                AddList += " " + entry.ShortPath;
            }

            writer.WriteLine("del " + cab);
            writer.WriteLine(String.Format(
                "cabarc -m NONE n {0} {1} {2}",cab, setup, AddList));
            writer.WriteLine("pause");
            
            writer.Flush();
            writer.Close();
            fs.Close();            
        }

        public void Debug()
        {
            Describe_INF describe = new Describe_INF(InstallData.Data);
            describe.DebugClass();
        }

        public String ProductName
        {
            get
            {
                return InstallData.ProductName;
            }
        }

        public String Description
        {
            get
            {
                return InstallData.Description;
            }
        }

        public ThemeData ThemeData()
        {
            ThemeData retVal = new ThemeData();
            Color[] themeColors = new Color[41];
            String defBackGround = QuickZip.ThemeBuilder.ThemeData.tdywater;

            for (short i = 0; i < InstallData.Data.Files.Count; i++)
            {
                string sf = "", rf = "", dd = "", df = "";
                InstallData.FileInfo(i, out sf, out rf, out dd, out df);
                if ((df.ToLower() == defBackGround) && (dd == "%CE2%"))
                {
                    retVal.backGroundBitmap = sf;
                    break;
                }
            }

            for (short i = 0; i < InstallData.Data.RegKeys.Count; i++)
            {
                RegKeyEntry re = InstallData.Data.RegKeys.RegKey(i);
                RegHiveEntry rhe = InstallData.Data.RegHives.ID(re.HiveID);
                String rPath = InstallData.Data.Strings.ParseString(rhe.Value);

                if ((rhe.HiveRoot == RegistryRoot.HKLM) && (rPath.ToLower() == @"software\microsoft\color"))
                {
                    Int32 id = Convert.ToInt32(re.KeyName);
                    if (!(QuickZip.ThemeBuilder.ThemeData.ValidEntry(id)))
                    {
                        Console.WriteLine(re.KeyName + "-" + Convert.ToString(Tools.Binary2Color(re.Value_Binary)));
                    }
                    else
                    {
                        themeColors[id] = Tools.Binary2Color(re.Value_Binary);;
                    }
                }                                    
            }

            retVal.SetThemeColors(themeColors);
            return retVal;
            
        }

        public void SetThemeData(ThemeData data)
        {
            String defBackGround = QuickZip.ThemeBuilder.ThemeData.tdywater;
            if (InstallData.Data.Files.File(@"{app}\" + defBackGround) != null)
                InstallData.Data.Files.File(@"{app}\" + defBackGround).SourcePath = data.backGroundBitmap;

            for (short i = 0; i < InstallData.Data.RegKeys.Count; i++)
            {
                RegKeyEntry re = InstallData.Data.RegKeys.RegKey(i);
                RegHiveEntry rhe = InstallData.Data.RegHives.ID(re.HiveID);
                String rPath = InstallData.Data.Strings.ParseString(rhe.Value);

                if ((rhe.HiveRoot == RegistryRoot.HKLM) && (rPath.ToLower() == @"software\microsoft\color"))
                {
                    
                    Int32 id = Convert.ToInt32(re.KeyName);
                    InstallData.Data.RegKeys.RegKey(i).Value_Binary = Tools.Color2Binary(data.themeColors[id]);
                }                                    
            }
        }
    }
}
