using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using System.Diagnostics;

namespace QuickZip.InstallBuilder
{
    /// <summary>
    /// Either STRINGS, DIRS, FILES, REGHIVES, REGKEYS, LINKS section.
    /// Abstract class.
    /// </summary>
    internal class Section
    {
        public Int16 EntriesCount;
        public Int32 Offset;
        protected ArrayList Items;
       
        public Section()
        {
            Items = new ArrayList();
        }

        public Int32 Count
        {
            get
            {
                return Items.Count;
            }
        }

        public virtual void Clear()
        {
            Items.Clear();
        }

        public void Add(object item)
        {
            Items.Add(item);
        }

        public Int32 MinLength()
        {
            Int32 len = 0;
            for (int i = 0; i < Items.Count; i++)
            {
                len += ((Entry)Items[i]).MinLength();
            }
            return len;
        }

        public void DebugClass(String name)
        {
            Debug.WriteLine(String.Format("{0,-13} = Entries({1}), Offset({2})"
                , name, EntriesCount, Offset));

            for (int i = 0; i < Items.Count; i++)
            {
                ((Entry)Items[i]).DebugClass();
            }
        }
    }
    /// <summary>
    /// Represent STRINGS section
    /// </summary>
    internal class StringSection : Section
    {
        private ProjectData Data;
        public StringSection(ProjectData pd)
        {
            Data = pd;
        }
        public StringEntry String(Int16 Item)
        {
            return (StringEntry)Items[Item];
        }

        public StringEntry ID(Int16 Item)
        {
            for (int i = 0; i < Items.Count; i++)
            {
                if (((StringEntry)Items[i]).StringID == Item)
                    return (StringEntry)Items[i];
            }
            return null;
        }

        public Int16 MaxID
        {
            get
            {
                Int16 maxid = 1;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (String((Int16)i).StringID > maxid)
                    {
                        maxid = String((Int16)i).StringID;
                    }
                }
                return maxid;
            }
        }

        public String KeyString(Int16 item)
        {
            if (item == 0)
                return "";

            StringEntry se = ID(item);
            if (se != null) 
            {
                return se.Value;
            }
            else
            {
                return "";                
            }
        }      
       
        public String ParseString(Int16[] value)
        {
            String temp = "";
            for (int i = 0; i < value.Length; i++)
            {
                if (value[i] != 0)
                {
                    temp += ("\\" + KeyString(value[i]));
                }
            }
            if (temp != "")
                { return temp.Substring(1); }
            return "";
        }

        public Int16 AddString(String value)
        {
            String input = DirEntry.EncodeDirectory(value, Data.InstallDir);
            
            for (int i = 0; i < Items.Count; i++)
            {
                if (String((Int16)i).Value == input)
                {
                    return (String((Int16)i).StringID);
                }
            }

            if (value == "%CE0%") { return 1; }

            StringEntry newEntry = new StringEntry((Int16)(MaxID + 1), input);
            Add(newEntry);
            EntriesCount += 1;
            return (Int16)(MaxID);
        }

        public Int16[] AddStrings(String value)
        {
            ArrayList outputList = new ArrayList();
            String[] Split = value.Split(new Char[] {'\\'});


            foreach (string s in Split)
            {
                if (s.Trim() != "")
                    outputList.Add((Int16)AddString(s));
            }

            //if (outputList.Count > 1)
            outputList.Add((Int16)0);                        

            return (Int16[])(outputList.ToArray(typeof(Int16)));
        }

        
    }
    /// <summary>
    /// Represent DIRS Section
    /// </summary>
    internal class DirSection : Section
    {
        private ProjectData Data;
        public DirSection(ProjectData pd)
        {
            Data = pd;
        }
        public DirEntry Dir(Int16 Item)
        {
            return (DirEntry)Items[Item];
        }

        public DirEntry ID(Int16 Item)
        {
            for (int i = 0; i < Count; i++)
            {
                DirEntry de = Dir((Int16)i);
                if (de.DirID == Item)
                {
                    return de;
                }
            }
            return new DirEntry();
        }

        public Int16 MaxID
        {
            get
            {
                Int16 maxid = 1;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (Dir((Int16)i).DirID > maxid)
                    {
                        maxid = Dir((Int16)i).DirID;
                    }
                }
                return maxid;
            }
        }

        public Int16 AddDir(String value)
        {
            String input = DirEntry.EncodeDirectory(value, Data.InstallDir);
            Int16 [] DirArray = Data.Strings.AddStrings(input);

            for (int i = 0; i < Items.Count; i++)
            {
                Int16[] CurrentArray = Dir((Int16)(i)).Value;
                if (Data.Strings.ParseString(CurrentArray) == Data.Strings.ParseString(DirArray))
                {
                    return (Dir((Int16)i).DirID);
                }
            }           
           
            DirEntry newEntry = new DirEntry((Int16)(MaxID+1), DirArray);
            Add(newEntry);
            EntriesCount += 1;
            return (Int16)(newEntry.DirID);
        }

    }

    /// <summary>
    /// Represent FILE Section
    /// </summary>
    internal class FileSection : Section
    {
        private ProjectData Data;        
        public FileSection(ProjectData pd)
        {
            Data = pd;
        }

        public FileEntry File(Int16 Item)
        {
            return (FileEntry)Items[Item];
        }

        public FileEntry ID(Int16 Item)
        {
            for (int i = 0; i < Count; i++)
            {
                FileEntry fe = File((Int16)i);
                if (fe.FileID == Item)
                {
                    return fe;
                }
            }
            return new FileEntry();
        }

        public FileEntry File(String filename)
        {
            filename = DirEntry.EncodeDirectory(filename, Data.InstallDir);
            for (Int16 i = 0; i < Items.Count; i++)
            {      
                FileEntry fe = File(i);
                DirEntry de = Data.Dirs.ID(fe.DirID);
                String path = Data.Strings.ParseString(de.Value) + @"\" + fe.Value;
                if (Data.InstallDir != "")
                	path = path.Replace(Data.InstallDir, "%CE0%");
                if (path.ToLower() == filename.ToLower())
                    return File(i);
            }
            return null;
        }

        public FileEntry Source(String sourcename)
        {
            for (Int16 i = 0; i < Items.Count; i++)
            {
                if (File(i).SourcePath.ToLower() == sourcename.ToLower())
                    return File(i);
            }
            return null;
        }

        public Int16 MaxID
        {
            get
            {
                Int16 maxid = 0;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (File((Int16)i).FileID > maxid)
                    {
                        maxid = File((Int16)i).FileID;
                    }
                }
                return maxid;
            }
        }

        public Int16 MaxFileNumber
        {
            get
            {
                Int16 maxnum = 000;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (File((Int16)i).FileNumber > maxnum)
                    {
                        maxnum = File((Int16)i).FileNumber;
                    }
                }
                return maxnum;
            }
        }
        

        public Int16 AddFile(String directory, ref String filename)
        {
            Int16 DirID = Data.Dirs.AddDir(directory);
            UInt32 Flags = 0;

            FileEntry newEntry = new FileEntry((Int16)(MaxID + 1), DirID, 
                (Int16)(MaxFileNumber+1),Flags, filename);
            Add(newEntry);
            EntriesCount += 1;
            filename = Tools.GetDosFilename(filename, newEntry.FileNumber);
            return newEntry.FileID;
        }

        public Int16 AddFile(String directory, ref String filename, UInt32 flags)
        {
            Int16 DirID = Data.Dirs.AddDir(directory);            

            FileEntry newEntry = new FileEntry((Int16)(MaxID + 1), DirID,
                (Int16)(MaxFileNumber + 1), flags, filename);
            Add(newEntry);
            EntriesCount += 1;
            filename = Tools.GetDosFilename(filename, newEntry.FileNumber);
            return newEntry.FileID;
        }

        public Int16 AddFile(String directory, ref String filename, String source, UInt32 flags)
        {
            Int16 DirID = Data.Dirs.AddDir(directory);

            FileEntry newEntry = new FileEntry((Int16)(MaxID + 1), DirID,
                (Int16)(MaxFileNumber + 1), flags, filename, source);
            Add(newEntry);
            EntriesCount += 1;
            filename = Tools.GetDosFilename(filename, newEntry.FileNumber);
            return newEntry.FileID;
        }
    }

    /// <summary>
    /// Represent REGHIVES Section
    /// </summary>
    internal class RegHivesSection : Section
    {
        private ProjectData Data;
        public RegHivesSection(ProjectData pd)
        {
            Data = pd;
        }
        public RegHiveEntry RegHive(Int16 Item)
        {
            return (RegHiveEntry)Items[Item];
        }

        public RegHiveEntry ID(Int16 Item)
        {
            for (int i = 0; i < Count; i++)
            {
                RegHiveEntry re = RegHive((Int16)i);
                if (re.HiveID == Item)
                {
                    return re;
                }
            }
            return new RegHiveEntry();
        }

        public Int16 MaxID
        {
            get
            {
                Int16 maxid = 0;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (RegHive((Int16)i).HiveID > maxid)
                    {
                        maxid = RegHive((Int16)i).HiveID;
                    }
                }
                return maxid;
            }
        }

        public Int16 AddRegHive(RegistryRoot root, String path)
        {
            Int16[] PathArray = Data.Strings.AddStrings(path);

            for (int i = 0; i < Items.Count; i++)
            {
                Int16[] CurrentArray = RegHive((Int16)(i)).Value;
                RegistryRoot CurrentRoot = RegHive((Int16)(i)).HiveRoot;
                if (Data.Strings.ParseString(CurrentArray) == Data.Strings.ParseString(PathArray)) 
                {
                    if (CurrentRoot == root)
                    {
                        return (RegHive((Int16)i).HiveID);
                    }
                }
            }

            RegHiveEntry newEntry = new RegHiveEntry((Int16)(MaxID + 1), root, PathArray);
            Add(newEntry);
            EntriesCount += 1;
            return (Int16)(newEntry.HiveID);
        }

    }

    /// <summary>
    /// Represent REGKEYS Section
    /// </summary>
    internal class RegKeysSection : Section
    {
        private ProjectData Data;
        public RegKeysSection(ProjectData pd)
        {
            Data = pd;
        }
        public RegKeyEntry RegKey(Int16 Item)
        {
            return (RegKeyEntry)Items[Item];
        }

        public RegKeyEntry ID(Int16 Item)
        {
            for (int i = 0; i < Count; i++)
            {
                RegKeyEntry re = RegKey((Int16)i);
                if (re.KeyID == Item)
                {
                    return re;
                }
            }
            return new RegKeyEntry();
        }

        public Int16 MaxID
        {
            get
            {
                Int16 maxid = 0;
                for (int i = 0; i < Items.Count; i++)
                {
                    if (RegKey((Int16)i).KeyID > maxid)
                    {
                        maxid = RegKey((Int16)i).KeyID;
                    }
                }
                return maxid;
            }
        }

        public Int16 AddRegKey(RegistryRoot root, String path,
                String key, Int32 datatype, Int32 value_dword, 
                String value_string, String[] value_multi_string, 
                Byte[] value_binary)
        {
            Int16 HiveID = Data.RegHives.AddRegHive(root, path);
            
            RegKeyEntry newEntry;
            value_string = value_string.Replace("{app}", "%InstallDir%");
            switch (datatype)
            {
                case 0x00010001: //DWORD 
                    newEntry = new RegKeyEntry((Int16)(MaxID + 1), HiveID, key, value_dword);
                    break;
                case 0x00000000: //SZ 
                    newEntry = new RegKeyEntry((Int16)(MaxID + 1), HiveID, key, value_string);
                    break;
                case 0x00001000: //MULTI_SZ 
                    newEntry = new RegKeyEntry((Int16)(MaxID + 1), HiveID, key, value_multi_string);
                    break;
                case 0x00000001:  //Binary
                    newEntry = new RegKeyEntry((Int16)(MaxID + 1), HiveID, key, value_binary);
                    break;
                default: 
                    throw new FormatException("Unrecognized datatype in AddRegKey");
            }    
        
            Add(newEntry);
            EntriesCount += 1;
            return (Int16)(newEntry.KeyID);
        }
 
    }
        /// <summary>
        /// Represent LINKS Section
        /// </summary>
        internal class LinksSection : Section
        {

            private ProjectData Data;
            public LinksSection(ProjectData pd)
            {
            Data = pd;
            }
            public LinkEntry Link(Int16 Item)
            {
                return (LinkEntry)Items[Item];
            }

            public LinkEntry ID(Int16 Item)
            {
                for (int i = 0; i < Count; i++)
                {
                    LinkEntry le = Link((Int16)i);
                    if (le.LinkID == Item)
                    {
                        return le;
                    }
                }
                return new LinkEntry();
            }

            public Int16 MaxID
            {
                get
                {
                    Int16 maxid = 0;
                    for (int i = 0; i < Items.Count; i++)
                    {
                        if (Link((Int16)i).LinkID > maxid)
                        {
                            maxid = Link((Int16)i).LinkID;
                        }
                    }
                    return maxid;
                }
            }

            public Int16 AddLink(PPC30 BaseDir, LinkType linktype, Int16 target , string linkname)
            {
                Int16[] LinkArray = Data.Strings.AddStrings(linkname);

                LinkEntry newEntry = new LinkEntry((Int16)(MaxID + 1), target, linktype, BaseDir, LinkArray);
                Add(newEntry);
                EntriesCount += 1;
                return (Int16)(newEntry.LinkID);
            }

        }

}
