/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 2/7/2007
 * Time: 2:31
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Threading;
using Cake3;
using System.Diagnostics;

namespace Cake3.Queue
{			
	public class CakdirWorkItem
	{
		public enum WorkItemStatus { wsAbort, wsAborting, wsCompleted, wsRunning, wsWaiting }
		public enum ActionType { atUnknown, atAdd, atDelete, atExtract, atDebug };
		
		private WorkItemStatus status = WorkItemStatus.wsWaiting;
		private ActionType action = ActionType.atUnknown;
		private Thread thread = null;
		private DateTime startTime = DateTime.MinValue;
		private DateTime endTime = DateTime.MaxValue;
		private string name;
		private string archiveName;
		private ExtractOptions extrOptions;
		private AddOptions addOptions;
		private string addto = "";
		private DeleteOptions delOptions;
		private bool retVal;
		private List<string> messages;
		
		internal CakdirThreadQueue queue;
		internal ThreadPriority priority;
		
		public WorkItemStatus Status { get { return status; } }
		public ActionType Action { get { return action; } }
		public string ThreadStatus { get { return action.ToString(); } }
		public string ArchiveName { get { return archiveName; } }
		public DateTime StartTime { get { return startTime; } }
		public DateTime EndTime { get { return endTime; } }
		public string[] ReturnMessage { get { return messages.ToArray(); } }
		public string LastMessage { get { if (messages.Count > 0) return messages[messages.Count-1]; else return ""; } }
		#region EventHandler
		/// <summary>
		/// Occurs when there is an message
		/// </summary>
		public MessageEventHandler OnMessage;
		/// <summary>
		/// Occurs when there is an error.
		/// </summary>
		public ErrorEventHandler OnError;
		/// <summary>
		/// Occurs when progress update.
		/// </summary>
		public ProgressEventHandler OnProgress;
		/// <summary>
		/// Occurs when confirm overwrite.
		/// </summary>
		public OverwriteEventHandler OnOverwrite;
		/// <summary>
		/// Occurs when request password.
		/// </summary>
		public PasswordEventHandler OnPassword;
		/// <summary>
		/// Occurs when start archive work.
		/// </summary>
		public EventHandler OnStartWorking;
		/// <summary>
		/// Occurs when finished archive work.
		/// </summary>
		public EventHandler OnStopWorking;
		
		public EventHandler OnItemChanged;
		
		public void CakdirOnMessage(object sender, MessageEventArgs e)
		{			
			messages.Add(e.Message);
		}
		public void CakdirOnError(object sender, ErrorEventArgs e)
		{			
			messages.Add(e.Error.ErrorDescription);
		}
		
		public void AttachEventHandlers(Cakdir3 c3)
		{
			if (OnMessage != null) c3.OnMessage += OnMessage;
			if (OnError != null) c3.OnError += OnError;
			if (OnProgress != null) c3.OnProgress += OnProgress;
			if (OnOverwrite != null) c3.OnOverwrite += OnOverwrite;
			if (OnPassword != null) c3.OnPassword += OnPassword;
			if (OnStartWorking != null) c3.OnStartWorking += OnStartWorking;
			if (OnStopWorking != null) c3.OnStopWorking += OnStopWorking;			
			c3.OnMessage += new MessageEventHandler(CakdirOnMessage);
			c3.OnError += new ErrorEventHandler(CakdirOnError);
		}
		#endregion
		#region Constructor
		public CakdirWorkItem(string DebugName)
		{
			name = DebugName;
			archiveName = "Debug";
			action = ActionType.atDebug;
			messages = new List<string>();
		}
		
		public CakdirWorkItem(ExtractOptions Options)
		{
			extrOptions = Options;			
			archiveName = Options.archiveName;
			name = String.Format("extract-from-{0}",Utils.ExtractFileName(Options.archiveName));
			action = ActionType.atExtract;
			messages = new List<string>();
		}
		
		public CakdirWorkItem(AddOptions Options)
		{
			addOptions = Options;
			archiveName = Options.archiveName;
			name = String.Format("Add-to-{0}",Utils.ExtractFileName(Options.archiveName));
			action = ActionType.atAdd;
			messages = new List<string>();
		}
		
		public CakdirWorkItem(AddOptions Options, string Addto)
		{
			addOptions = Options;
			archiveName = Options.archiveName;
			addto = Addto;
			name = String.Format("Add-to-{0}",Utils.ExtractFileName(Options.archiveName));
			action = ActionType.atAdd;
			messages = new List<string>();
		}
		
		public CakdirWorkItem(DeleteOptions Options)
		{
			delOptions = Options;
			archiveName = Options.archiveName;
			name = String.Format("Delete-{0}",Utils.ExtractFileName(Options.archiveName));
			action = ActionType.atDelete;
			messages = new List<string>();
		}
		#endregion
		#region Raise Event
		void RaiseRunning()
		{
			startTime = DateTime.Now;
			status = WorkItemStatus.wsRunning;
			Debug.WriteLine(String.Format("{0} : {1} Running", DateTime.Now.ToString(), name));
			RaiseQueueChanged();
		}
		void RaiseCompleted()
		{
			endTime = DateTime.Now;
			status = WorkItemStatus.wsCompleted;
			Debug.WriteLine(String.Format("{0} : {1} Completed", DateTime.Now.ToString(), name));
			RaiseQueueChanged();
		}
		void RaiseAborting()
		{
			status = WorkItemStatus.wsAborting;
			Debug.WriteLine(String.Format("{0} : {1} Aborting", DateTime.Now.ToString(), name));
			RaiseQueueChanged();
		}
		void RaiseAborted()
		{
			status = WorkItemStatus.wsAbort;
			Debug.WriteLine(String.Format("{0} : {1} Aborted", DateTime.Now.ToString(), name));
			RaiseQueueChanged();
		}
		#endregion
		#region Thread Operations		
		public void Run()
		{
			Cakdir3 c3 = null;
			try
			{
				switch (action)
				{
					case ActionType.atDebug:
						Thread.Sleep(1000);
						retVal = true;
						break;
					case ActionType.atExtract:
						c3 = new Cakdir3(extrOptions.archiveName);
						AttachEventHandlers(c3);
						c3.ExtractOptions = extrOptions;
						retVal = c3.Extract();
						break;
					case ActionType.atAdd:
						c3 = new Cakdir3(addOptions.archiveName);
						AttachEventHandlers(c3);
						c3.AddOptions = addOptions;
						retVal = c3.AddToFolder(addto);
						break;
					case ActionType.atDelete:	
						c3 = new Cakdir3(delOptions.archiveName);
						AttachEventHandlers(c3);
						c3.DeleteOptions = delOptions;
						retVal = c3.Delete();
						break;
				}
				
				if (OnItemChanged != null)
					OnItemChanged(this, new EventArgs());
			}
			catch (ThreadAbortException ex)
			{
				if (c3 != null)
				{
					//Not Implemented
//					c3.Stop();
				}
				RaiseAborted();
				return;
			}
			RaiseCompleted();
		}
		
		public CakdirWorkItem Start()
		{
			if (status == WorkItemStatus.wsWaiting)
			{
				RaiseRunning();
				thread = new System.Threading.Thread
					( new System.Threading.ThreadStart ( Run ) ) ;
				thread.Name = name;
				thread.IsBackground = true;
				thread.Start();
			}
			return this;
		}
		public void Abort()
		{
			if (status == WorkItemStatus.wsRunning)
				if ( thread.IsAlive )
			{
				RaiseAborting();
				thread.Abort();
				int timeout = 1000;
				while ((thread.IsAlive) && (timeout > 0))
				{
					System.Threading.Thread.Sleep (100) ;
					timeout -= 100;
				}
				RaiseAborted();
			}
			
			if (status == WorkItemStatus.wsWaiting)
				RaiseAborted();
		}
		
		public void RaiseQueueChanged()
		{
			if ((queue != null) && (queue.WorkItemExists(this)))
				queue.RaiseQueueChanged();
		}
		#endregion
		
	}
	
	/// <summary>
	/// Description of CakdirThreadPool.
	/// </summary>
	public class CakdirThreadQueue : System.IDisposable
	{
		private List<CakdirWorkItem> queue;
		private List<CakdirWorkItem> fullQueue;
		private ThreadPriority priority;
		private bool paused = false;
		
		#region Properties
		public int Count { get { return queue.Count; } }
		public bool isEmpty { get { return queue.Count == 0; } }
		public EventHandler OnQueueChanged;
		public bool Paused { get { return paused; } set { paused = value; } }		
		public ThreadPriority Priority { get { return priority; }
			set { priority = value; foreach (CakdirWorkItem workItem in queue) workItem.priority = priority; }
		}		
		public List<CakdirWorkItem> FullQueue { get { return fullQueue; } }
		public List<CakdirWorkItem> Queue { get { return queue; } }
		public List<string> QueueStatus { 
			get { List<string> retVal = new List<string>();				
				foreach (CakdirWorkItem workItem in fullQueue)
					retVal.Add(workItem.ThreadStatus); 
				return retVal;} }
		#endregion
						
		public CakdirThreadQueue()
		{
			queue = new List<CakdirWorkItem>();
			fullQueue = new List<CakdirWorkItem>();
		}
		
		public void RaiseQueueChanged()
		{			
			if (paused) return;
			
			if (OnQueueChanged != null)
				OnQueueChanged(this, new EventArgs());
			CheckQueue();
		}
		
		public void CheckQueue()
		{
			while (Paused)
				Thread.Sleep(100);
				
			
			if (!isEmpty)
			{
				CakdirWorkItem firstItem = queue[0];
				switch (firstItem.Status)
				{
					case CakdirWorkItem.WorkItemStatus.wsWaiting :
						firstItem.Start();
						break;
					case CakdirWorkItem.WorkItemStatus.wsCompleted :
						Dequeue(firstItem);
						break;
					case CakdirWorkItem.WorkItemStatus.wsAbort :
						Dequeue(firstItem);
						break;
				}
			}
		}
		
		public bool WorkItemExists(CakdirWorkItem workItem)
		{
			return (queue.IndexOf(workItem) != -1);
		}
		
		public const string queuemsg = "Work queued ({0})";
		public void Enqueue(CakdirWorkItem workItem)
		{
			lock (queue)
			{
				workItem.queue = this;
				workItem.priority = priority;
				if (!isEmpty)
					if (workItem.OnMessage != null)
					workItem.OnMessage(this, new MessageEventArgs(null, String.Format(queuemsg, queue.Count)));
				queue.Add(workItem);
				fullQueue.Add(workItem);
				RaiseQueueChanged();				
			}
			
			CheckQueue();
		}
		
		public void Dequeue(CakdirWorkItem workItem)
		{
			lock (queue)
			{
				if (queue.IndexOf(workItem) != -1)
				{
					if (workItem.Status == CakdirWorkItem.WorkItemStatus.wsRunning)
						workItem.Abort();
					queue.Remove(workItem);
					RaiseQueueChanged();
				}
			}
			
			CheckQueue();
		}
		
		public void AbortAll()
		{
			lock (queue)
			{				
				paused = true;
				for (int i = queue.Count -1; i >= 0; i--)
					queue[i].Abort();
				paused = false;				
			}
			RaiseQueueChanged();
		}
		
//		public void ClearAll()
//		{
//			lock (queue)
//			{
//				queue.Clear();
//				RaiseQueueChanged();
//			}
//		}
		
		public void Cleanup()
		{
			lock (fullQueue)
			{
				for (int i = fullQueue.Count -1; i >= 0; i--)
					if ((fullQueue[i].Status == CakdirWorkItem.WorkItemStatus.wsCompleted) ||
					    (fullQueue[i].Status == CakdirWorkItem.WorkItemStatus.wsAbort))
							fullQueue.Remove(fullQueue[i]);								
			}
			RaiseQueueChanged();
		}
		
		public void Dispose()
		{
			AbortAll();			
		}
		
		
	}
}
