/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 3/7/2007
 * Time: 17:38
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;

namespace Cake3.Queue
{
	/// <summary>
	/// Description of ThreadQueueMonitor.
	/// </summary>
	public partial class ThreadQueueMonitor : UserControl
	{
		public ThreadQueueMonitor()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			listView.TileSize = new Size(Width-30, 50);
			//
			// TODO: Add constructor code after the InitializeComponent() call.
			//
		}
		
		private CakdirThreadQueue ctq = null;
		public void RegisterThreadQueue(CakdirThreadQueue threadQueue)
		{
			threadQueue.OnQueueChanged += new EventHandler(ThreadQueueChanged);
			ctq = threadQueue;
			ReloadList();
		}
		
		public void UnregisterThreadQueue()
		{
			if (ctq != null)
				ctq.OnQueueChanged = null;
			ctq.AbortAll();
			ctq.Dispose();
			ctq = null;
		}
		
		void ThreadQueueChanged(object sender, EventArgs e)
		{
			if (listView.InvokeRequired)
				this.Invoke(new EventHandler(ThreadQueueChanged), new object[] { sender, e });
			else
			ReloadList();
		}
		
		void ReloadList()
		{		
			if (!Utils.isXPUp())
				listView.View = View.Details;
			
			
			timer1.Enabled = false;
			int selIdx = -1;
			if (listView.SelectedItems.Count == 1)
				selIdx = listView.SelectedItems[0].Index;
				
			listView.BeginUpdate();			
			listView.Items.Clear();
			
			if (ctq == null) return;
			for (int i = 0; i < ctq.FullQueue.Count; i++)
			{
				CakdirWorkItem workItem = ctq.FullQueue[i];
				ListViewItem lvItem = new ListViewItem();
				string ext = Utils.ExtractFileExt(workItem.ArchiveName) + " ";
				if (!imageL.Images.ContainsKey(ext))
					imageL.Images.Add(ext, Utils.GetLargeFileIcon("abc" + ext.Trim()).ToBitmap());
				
				lvItem.Tag = i;
				lvItem.ImageKey = ext;				
				lvItem.Text = workItem.Action.ToString().Substring(2) + " " + 
							  Utils.ExtractFileName(workItem.ArchiveName);
				
				string spendTimeStr = "";
				TimeSpan spendTime;					
				switch (workItem.Status)
				{
					case CakdirWorkItem.WorkItemStatus.wsCompleted:
						spendTime = (workItem.EndTime - workItem.StartTime);
						spendTimeStr = String.Format(" ({0}:{1})", spendTime.Minutes, spendTime.Seconds);
						break;
					case CakdirWorkItem.WorkItemStatus.wsRunning:
						spendTime = (DateTime.Now - workItem.StartTime);
						spendTimeStr = String.Format(" ({0}:{1})", spendTime.Minutes, spendTime.Seconds);
						break;
					
				}
																														
				lvItem.SubItems.Add("  " + workItem.Status.ToString().Substring(2) + spendTimeStr  );				
				lvItem.SubItems.Add("");
				lvItem.SubItems.Add(workItem.ArchiveName);
				listView.Items.Add(lvItem);					
			}
			if ((selIdx != -1) && (listView.Items.Count > selIdx))
				listView.SelectedIndices.Add(selIdx);			
			listView.EndUpdate();
			timer1.Enabled = (ctq.Queue.Count > 0);
		}
		
//		void ListBoxMeasureItem(object sender, MeasureItemEventArgs e)
//		{
//			e.ItemHeight = 50;
//		}
//		
//		void ListBoxDrawItem(object sender, DrawItemEventArgs e)
//		{
//			if (e.Index == -1) return;
//			int X = 10;
//			int Y = (int)((e.Bounds.Top + e.Bounds.Bottom) / 2);
//			
//			if (e.State == DrawItemState.Selected)
//				e.Graphics.FillRectangle(new SolidBrush(Color.YellowGreen), e.Bounds);
//			else e.Graphics.FillRectangle(new SolidBrush(SystemColors.Window), e.Bounds);
//			e.Graphics.DrawString((string)listBox.Items[e.Index], listBox.Font, System.Drawing.Brushes.Black, X, Y);
//		}
				
		
		void ThreadQueueMonitorSizeChanged(object sender, EventArgs e)
		{
			listView.TileSize = new Size(Math.Max(1,Width-30), 50);
		}
		
		void Abort1Click(object sender, EventArgs e)
		{
			if (listView.SelectedItems.Count == 1)
				ctq.FullQueue[listView.SelectedItems[0].Index].Abort();
		}
		
		public string[] SelectionList()
		{			
			string[] result = new string[listView.SelectedItems.Count];			

			for (int i = 0; i < listView.SelectedItems.Count; i++)
			{
				result[i] = listView.SelectedItems[i].SubItems[3].Text;
//				Debug.WriteLine(result[i]);
			}
			return result;
		}
		
		void ListViewItemDrag(object sender, ItemDragEventArgs e)
		{
			string[] fileNames = SelectionList();
			
			DataObject dataObj = new DataObject(fileNames);
			//Add the list of files to the data object
			dataObj.SetData(DataFormats.FileDrop, fileNames);
//            //Set the preferred drop effect
//            dataObj.SetData(ShellClipboardFormats.CFSTR_PREFERREDDROPEFFECT, DragDropEffects.Move);
//            //Indicate that we are in a drag loop
//            dataObj.SetData(ShellClipboardFormats.CFSTR_INDRAGLOOP, 1);
            //Initiate the drag operation
            listView.DoDragDrop(dataObj, DragDropEffects.Move | DragDropEffects.Copy);
		}
		
		void ListViewQueryContinueDrag(object sender, QueryContinueDragEventArgs e)
		{
			//ESC pressed
			if (e.EscapePressed)
			{
				e.Action = DragAction.Cancel;
				return;
			}
			//Drop!
			if (e.KeyState == 0)
			{				
				e.Action = DragAction.Drop;				
				return;				
			}
			e.Action = DragAction.Continue;
		}
		
		void Timer1Tick(object sender, EventArgs e)
		{
			ReloadList();
		}
		
		void ListViewItemMouseHover(object sender, ListViewItemMouseHoverEventArgs e)
		{
			string[] msg = ctq.FullQueue[(int)e.Item.Tag].ReturnMessage;
			int startFrom = 0;
			string tooltip = ctq.FullQueue[(int)e.Item.Tag].ArchiveName;
			if (msg.Length > 10) 
				startFrom = msg.Length - 10;
			while (startFrom < msg.Length)
			{
				tooltip += "\r\n" + msg[startFrom];
				startFrom++;
			}
			
			e.Item.ToolTipText = tooltip;;
		}
		
		void ListViewDoubleClick(object sender, EventArgs e)
		{
			if (listView.SelectedIndices.Count == 1)
			{
				int idx = listView.SelectedIndices[0];
			if (ctq.FullQueue[(int)idx].Status == CakdirWorkItem.WorkItemStatus.wsCompleted)
				if (Utils.FileExists(ctq.FullQueue[(int)idx].ArchiveName))
				Utils.Run(ctq.FullQueue[(int)idx].ArchiveName, "");
			}
		}
	}
}
