/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 22/5/2007
 * Time: 18:39
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Schematrix;
using Cake3;

namespace Cake3.Rar
{
	/// <summary>
	/// Description of RarArchiver.
	/// </summary>
	public class RarArchiver :CakArchiver
	{
		internal override string name {	get { return "rardllwrap"; }}
		private Unrar arc;
		private static string DllName = "unrar.dll";
		
		private static string ModifySlashToCak(string input)
		{
			return Utils.AppendFrontSlash( input );
		}
		
		private static string ModifySlashFromCak(string input)
		{
			return Utils.RemoveFrontSlash( input );
		}
				
		private void AttachHandlers(Unrar unrar)
        {
            unrar.ExtractionProgress+=new ExtractionProgressHandler(unrar_ExtractionProgress);
            unrar.MissingVolume+=new MissingVolumeHandler(unrar_MissingVolume);
            unrar.PasswordRequired+=new PasswordRequiredHandler(unrar_PasswordRequired);
        }
		
		private void HandleException(Exception ex)
		{
			if ((OnError == null)) return;
			ErrorMessage ec = Defines.ErrorMessage("NOERROR");
			
			string filename = "";
			if (arc.CurrentFile != null)
				filename = arc.CurrentFile.FileName;	
			
			switch (ex.Message)
			{
				case "Unknown archive format.":
					DispError(Defines.ErrorMessage("UNKTYPE"), filename);
					break;
				case "File CRC Error":
					DispError(Defines.ErrorMessage("CRC"), filename);
					break;
				case "File is not a valid archive.":
					DispError(Defines.ErrorMessage("UNKTYPE"), filename);
					break;
				case "File could not be opened.":
					DispError(Defines.ErrorMessage("LISTERROR"), filename);
					break;
				case "File could not be created.":
					DispError(Defines.ErrorMessage("CANTCREATE"), filename);
					break;
				case "File close error.":
					DispError(Defines.ErrorMessage("CANTCLOSE"), filename);
					break;
				case "File read error.":
					DispError(Defines.ErrorMessage("CANTREAD"), filename);
					break;
				case "File write error.":
					DispError(Defines.ErrorMessage("CANTWRITE"), filename);
					break;
			}
				
		}
		
		private void unrar_ExtractionProgress(object sender, ExtractionProgressEventArgs e)
        {
			//TODO: Workout the percent complete.
			if ((OnProgress != null))
				OnProgress(this, new ProgressEventArgs(this, (ushort)e.PercentComplete, e.FileName));
								
			DispMessage(Defines.ErrorMessage("PROCESS"), e.FileName);	
        }

        private void unrar_MissingVolume(object sender, MissingVolumeEventArgs e)
        {
        	OpenFileDialog dialog=new OpenFileDialog();
        	dialog.FileName = e.VolumeName;
        	dialog.Title = "Volume " + e.VolumeName + " is missing.";
        	if ((dialog.ShowDialog() == DialogResult.OK) && (Utils.FileExists(dialog.FileName)))
        	{
        		e.VolumeName = dialog.FileName;
        		e.ContinueOperation = true;
        	}
        	else
        		e.ContinueOperation = false;
        }

        private void unrar_PasswordRequired(object sender, PasswordRequiredEventArgs e)
        {
        	if ((this.OnPassword != null))
        	{
        		PasswordEventArgs pe = new PasswordEventArgs(this, arc.CurrentFile.FileName);
        		OnPassword(this, ref pe);        		
        		e.Password = pe.Password;        		
        		e.ContinueOperation = true;
        		return;
        	}
        	e.ContinueOperation = false;        
        }
        
		public RarArchiver() : base()
		{
		}
		
		internal override void setArchiveName(string anArchive)
		{			
			base.setArchiveName(anArchive);
			
			string extension = Cake3.Utils.ExtractFileExt(anArchive).ToLower();
			switch (extension)
			{
				case ".rar" :					
					break;
				default : throw new NotSupportedException();
			}
			
			canList = true;
			canExtract = true;
			canAdd = false;
		}
				
		public override List<ArchiveType> SupportedArchiveTypes()
		{
			List<ArchiveType> retVal = new List<ArchiveType>();			
			retVal.Add(new ArchiveType(name, "rar", 0, true, true, false, false));
			return retVal;
		}
		
		public override bool Open()
		{ 
			if (archiveOpened) 
				return false;
			
			try
			{
				arc = new Unrar();	
				AttachHandlers(arc);				
				archiveOpened = true;
				return true;
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
						
			return false;
		}
		
		public override bool Close()
		{
			if (!archiveOpened) 
				return false;
						
			arc.Close();
			
			archiveOpened = false;
			return true;
		}
		
		public override bool List(string mask) 
		{ 
			if(OnItemList == null) return false;
			DispMessage(Defines.ErrorMessage("PROCESSLIST"), archiveName);	
			
			arc.Open(archiveName, Unrar.OpenMode.List);					
			while(arc.ReadHeader())
			{
				if(!arc.CurrentFile.IsDirectory)
				{					
					OnItemList(this, new ItemListEventArgs(this,
			                                       new ContentType(ArchiveName, (UInt32)arc.CurrentFile.UnpackedSize, 
					                                                       (UInt32)arc.CurrentFile.PackedSize,
			                                       arc.CurrentFile.FileTime, arc.CurrentFile.FileCRC,
			                                       ModifySlashToCak(arc.CurrentFile.FileName), 
			                                      arc.CurrentFile.IsEncrypted) ));
												//06-24-07 Detect rar encryption.
				}
				arc.Skip();
			}
			
			return true;
		}
		
		public override bool Test()
		{			
			DispMessage(Defines.ErrorMessage("PROCESSTEST"), archiveName);	
			
			arc = new Unrar();
			AttachHandlers(arc);
			
			arc.Open(archiveName, Unrar.OpenMode.Extract);
			try
			{								
				while(arc.ReadHeader())
				{
					arc.Test();
				}				
			}
			catch(Exception ex)
			{
				HandleException(ex);					
				return false;				
			}			
			finally
			{
				arc.Close();
			}
			
			return true;
		}
			
		private bool MatchExtractItem(string filename, ExtractOptions options)
		{
			string file = ModifySlashToCak(filename);
			foreach (string item in options.extractItem)				
				if (Utils.MatchMask(file, item))
				    return true;
			return false;
		}
		
		public override bool Extract(ExtractOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSEXTR"), archiveName);	
			arc = new Unrar();
			AttachHandlers(arc);
			
			try
			{								
				arc.Open(archiveName, Unrar.OpenMode.Extract);
				arc.DestinationPath = Utils.AppendSlash(options.ExtractFolderParsed);
				arc.Password = options.password;
				while(arc.ReadHeader())
				{					
					if (MatchExtractItem(arc.CurrentFile.FileName, options))
					{
						string destFile;
						
						if (options.allowFolder) 
							destFile = arc.DestinationPath + Utils.RemoveFrontSlash(arc.CurrentFile.FileName);
						else
							destFile = arc.DestinationPath + Utils.ExtractFileName(arc.CurrentFile.FileName);
//						Debug.WriteLine(destFile);
						arc.Extract(destFile);
					}
					
					else
						arc.Skip();						
				}				
			}
			catch (Exception ex)
			{				
				HandleException(ex);
				return false;
			}
			finally
			{
				arc.Close();
			}
			
			return true;
		}			
		
	}
}
