/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 26/11/2006
 * Time: 23:36
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using Cake3;

namespace Cake3.Sqx
{
	/// <summary>
	/// Description of SqxArchiver.
	/// </summary>
	public class SqxArchiver : CakArchiver
	{
		#region DllHeader
		private enum ErrorCodes
		{
			SQX_ERR_SUCCESS = 0,
			SQX_ERR_ERROR,
			SQX_ERR_FILE_NOT_FOUND,
			SQX_ERR_PATH_NOT_FOUND,
			SQX_ERR_TOO_MANY_FILES,
			SQX_ERR_ACCESS_DENIED,
			SQX_ERR_INVALID_HANDLE,
			SQX_ERR_DISK_FULL,
			SQX_ERR_OUT_OF_MEMORY,
			SQX_ERR_CANT_ACCESS_TEMP_DIR,
			SQX_ERR_TEMP_DIR_FULL,
			SQX_ERR_USER_ABORT,
			SQX_ERR_INVALID_ARC_HANDLE,
			SQX_ERR_CANT_FIND_LANG_DATA,
			SQX_ERR_UNKNOWN_SUBSTREAM,
			SQX_ERR_BAD_SUBSTREAM_CRC,
			SQX_ERR_UNKNOWN_METHOD,
			SQX_ERR_FILE_ENCRYPTED,
			SQX_ERR_BAD_CRC,
			SQX_ERR_CANT_CREATE_FILE,
			SQX_ERR_BAD_FILE_FORMAT,
			SQX_ERR_FUNCTION_NOT_SUPPORTED,
			SQX_ERR_FUNC_NOT_SUPPORTED_BY_ARCHIVE,
			SQX_ERR_CANT_CREATE_ARC_DIR,
			SQX_ERR_INVALID_DIR_NAME,
			SQX_ERR_INVALID_FILE_NAME,
			SQX_ERR_TOO_MANY_BROKEN_FBLOCKS,
			SQX_ERR_ARCHIVE_OK_RDATA_NOT,
			SQX_ERR_RDATA_DAMAGED,
			SQX_ERR_RDATA_NEW_VERSION,
			SQX_ERR_RDATA_DOES_NOT_MATCH,
			SQX_ERR_CANT_FIND_RDATA,
			SQX_ERR_ARCHIVE_IS_LOCKED,
			SQX_ERR_CANT_ADD_TO_MV,
			SQX_ERR_CANT_DELETE_FROM_MV,
			SQX_ERR_NEED_1ST_VOLUME,
			SQX_ERR_MISSING_VOLUME,
			SQX_ERR_VOLUME_LIMIT_REACHED,
			SQX_ERR_SFXSTUB_NOT_INSTALLED,
			SQX_ERR_BACKUP_READ_ACCESS_DENIED,
			SQX_ERR_BACKUP_WRITE_ACCESS_DENIED,
			SQX_ERR_ACL_READ_ACCESS_DENIED,
			SQX_ERR_ACL_WRITE_ACCESS_DENIED,
			SQX_ERR_WRONG_ARCHIVER_VERSION,
			SQX_ERR_CANT_COPY_SOURCE_TO_SOURCE,
			SQX_ERR_VOLUMES_TOO_SMALL,
			SQX_ERR_ARCHIVE_VERSION_TOO_HIGH,
			SQX_ERR_EXT_RDATA_DOES_NOT_MATCH,
			SQX_ERR_BAD_PARAMETER,
			SQX_ERR_EQUAL_PASSWORDS,
			SQX_ERR_REQUIRES_ENCRYPTION,
			SQX_ERR_MISSING_HEADER_PASSWORD,
			SQX_ERR_MISSING_SQX_PRIVATE_KEY,
			SQX_ERR_MISSING_SQX_AVKEY,
			SQX_ERR_MISSING_EXTERNAL_AVKEY,
			SQX_ERR_INVALID_SQX_AVKEY,
			SQX_ERR_SQX_AVKEY_VERSION,
			SQX_ERR_SQX_AVDATA_VERSION,
			SQX_ERR_SQX_BROKEN_AVRECORD,
			SQX_ERR_RIJNDAEL_RSA,
			SQX_ERR_REQUIRES_NTFS,
			SQX_ERR_REQUIRES_WINNT,
			SQX_ERR_REQUIRES_W2K,
			SQX_ERR_REQUIRES_WINXP,
			SQX_ERR_REQUIRES_WINXP_SP1,
			SQX_ERR_REQUIRES_WINXP_SP2,
			SQX_ERR_REQUIRES_LONGHORN,
			SQX_ERR_NO_RESOURCES_FOUND,
			SQX_ERR_UNKNOWN_ICON_FORMAT,
			SQX_ERR_NO_MATCHING_ICON_SIZE,
			SQX_ERR_UNKNOWN_EXE_FORMAT,
			SQX_ERR_REQUIRES_SOURCE_PATH,
			SQX_ERR_FILE_DATA_NOT_EQUAL,
			SQX_ERR_COMMENT_BIGGER_4K,
			SQX_ERR_CANT_CREATE_SFX_FROM_MV
		}	
		
		private const string DllName = "sqx20.dll";		
		public enum FileFormat
		{
			SQX_FILEFORMAT_10 = 0,
			SQX_FILEFORMAT_20 = 1
		};
		public enum CompressRate
		{
			SQX_COMPRATE_STORED 	= 0,
			SQX_COMPRATE_FAST 		= 1,
			SQX_COMPRATE_NORMAL 	= 2,
			SQX_COMPRATE_HIGH 		= 3,
			SQX_COMPRATE_MAXIMIUM 	= 4,
			SQX_COMPRATE_ULTRA 		= 5			
		};
		
		public enum CompressMethod
		{
			SQX_METHOD_STORED	= 0,
			SQX_METHOD_NORMAL	= 1,
			SQX_METHOD_GOOD		= 2,
			SQX_METHOD_HIGH		= 3,
			SQX_METHOD_BEST		= 4,
			SQX_METHOD_AUDIO	= 5,
			SQX_METHOD_TEXT		= 6,
			SQX_METHOD_ULTRA	= 7,
		}
		
		public enum DictionarySize
		{
			SQX_DICTIONARY_32K 	= 3,
			SQX_DICTIONARY_64K 	= 4,
			SQX_DICTIONARY_128K = 5,
			SQX_DICTIONARY_256K	= 6,
			SQX_DICTIONARY_512K	= 7,
			SQX_DICTIONARY_1M	= 8,
			SQX_DICTIONARY_2M	= 9,
			SQX_DICTIONARY_4M	= 10,
			SQX_DICTIONARY_8M	= 11,
			SQX_DICTIONARY_16M	= 12,
			SQX_DICTIONARY_32M	= 13,
		};
		
		public enum FileNameEncode
		{
			SQX_FILENAME_ANSI	= 0,
			SQX_FILENAME_ASCII	= 1,
			SQX_FILENAME_UNICODE= 2
		};
		
		public enum SpecialCompressor
		{
			SQX_SPECIALCOMP_DISABLED	= 0, 
			SQX_SPECIALCOMP_AUTOMATIC	= 1, 
			SQX_SPECIALCOMP_FORCE		= 2
		}
		
		public enum Signal
		{
			SQX_SIGNAL_COMPRESS 		= 1,
			SQX_SIGNAL_UNCOMPRESS,
			SQX_SIGNAL_DELETE,
			SQX_SIGNAL_TEMP_ARC_COPY,
			SQX_SIGNAL_CREATE_AV_SIG,
			SQX_SIGNAL_TEST_AV_SIG,
			SQX_SIGNAL_ADD_MV_UNCOMPRESS,
			SQX_SIGNAL_CREATE_RDATA,
			SQX_SIGNAL_REPAIR_ARCHIVE,
			SQX_SIGNAL_STRIP_RDATA,
			SQX_SIGNAL_TEST_ARCHIVE,
			SQX_SIGNAL_CREATE_SFX_ARCHIVE,
			SQX_SIGNAL_ADD_MV_TEST,
			SQX_SIGNAL_EX_TEST_ARCHIVE,
			SQX_SIGNAL_ADD_MV_EX_TEST	= 15
		}
		
		public enum Callback
		{
			SQX_CALLBACK_FILENAME		= 0,
			SQX_CALLBACK_PROGRESS,
			SQX_CALLBACK_REPLACE,
			SQX_CALLBACK_PASSWORD,
			SQX_CALLBACK_PASSWORD_HEADER,
			SQX_CALLBACK_SKIP,
			SQX_CALLBACK_NEXTDISK,
			SQX_CALLBACK_SIGNAL
		}
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxInitArchive([MarshalAs(UnmanagedType.LPStr)]string pszArchiveName,
		                                           dSqxCallback pfnCallback,
		                                           IntPtr pParam,
		                                           ref IntPtr phArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneArchive(IntPtr hArchive);
				
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static IntPtr SqxInitFileList(IntPtr hArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneFileList(IntPtr hArchive, IntPtr hFileMaskList);

		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static IntPtr SqxInitArcFileList(IntPtr hArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneArcFileList(IntPtr hArchive, IntPtr pArcFileList);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxListFiles(IntPtr hArchive, IntPtr hFileMaskList, ref SQX_ARCLIST pArcFileList,ref SQX_ARCINFO pArchiveInfo);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxAppendFileList(IntPtr hArchive, IntPtr hFileMaskList, string pszFileName);
					
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxTestArchive(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxExtractFiles(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam, 
		                                            IntPtr hFileMaskList, 	
		                                            ref SQX_EXTRACTOPTIONS pExtractOptions); 		
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxCompressFiles(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam,
		                                             IntPtr hFileMaskList,
		                                             ref SQX_COMPRESSOPTIONS pCompressOptions);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxDeleteFiles(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam,
		                                             IntPtr hFileMaskList);
		
		#endregion
		
		internal override string name {	get { return "sqxwrap"; }}
		IntPtr harc;
		UInt32 totalFiles, totalProcessed;	
		dSqxCallback SqxCallBackProc;
		List<string> fileList = new List<string>();
		
		void resetCallBack()
		{
			totalProcessed = 0;
		}

		public int SqxCallback(IntPtr pParam, ref SQX_CALLBACKINFO CallbackInfo)
		{			
//			if (CallbackInfo.dwCallbackType == Callback.SQX_CALLBACK_FILENAME)
				
			switch (CallbackInfo.dwCallbackType)
			{
				case Callback.SQX_CALLBACK_FILENAME:
					if (CallbackInfo.pszSourceFileName != null)
					{
						
						totalProcessed++;
						
						UInt16 Ratio = 0;
						if (totalFiles != 0)
							Ratio = (UInt16)((float)totalProcessed/(float)totalFiles*100);
						
						if ((OnProgress != null))
							OnProgress(this, new ProgressEventArgs(this, Ratio, CallbackInfo.pszSourceFileName));
						
						DispMessage(Defines.ErrorMessage("PROCESSOK"), CallbackInfo.pszSourceFileName);
					};
					
					return 1;
				case Callback.SQX_CALLBACK_PASSWORD:
					if (OnPassword != null)
					{
						PasswordEventArgs e = new PasswordEventArgs(this, CallbackInfo.pszSourceFileName);
						OnPassword(this, ref e);
						if (e.Password != "")
						{
							CallbackInfo.szCryptKey = e.Password;							
							return 1;
						}						
					}
					return 0;
				case Callback.SQX_CALLBACK_PASSWORD_HEADER:
					if (OnPassword != null)
					{
						PasswordEventArgs e = new PasswordEventArgs(this, "Header");
						OnPassword(this, ref e);
						if (e.Password != "")
						{
							CallbackInfo.szCryptKey = e.Password;							
							return 1;
						}						
					}
					return 0;
			}
			
			return 1;
		}
		
		public SqxArchiver() : base()
		{			
		}
				
		
		internal override void setArchiveName(string anArchive)
		{			
			base.setArchiveName(anArchive);
			
			string extension = Cake3.Utils.ExtractFileExt(anArchive).ToLower();
			switch (extension)
			{
				case ".sqx" :					
					break;
				default : throw new NotSupportedException();
			}
			
			canList = true;
			canExtract = true;
			canAdd = true;
			
		}
				
		public override List<ArchiveType> SupportedArchiveTypes()
		{
			List<ArchiveType> retVal = new List<ArchiveType>();	
			if (Utils.verifyDllPath(DllName))
				retVal.Add(new ArchiveType(name, "sqx", 0, true, true, true, true));
			return retVal;
		}
		
		public override bool Open()
		{ 
			if (archiveOpened) 
				return false;
			SqxCallBackProc = new dSqxCallback(SqxCallback);		
			Int32 iStatus = SqxInitArchive(archiveName, SqxCallBackProc, IntPtr.Zero, ref harc);						
			
			archiveOpened = true;
			return (iStatus == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		public override bool Close()
		{
			if (!archiveOpened) 
				return false;									
			
			SqxDoneArchive(harc);		
			
			archiveOpened = false;
			
			return true;
		}
		
		private string ModifySlashToCak(string input)
		{
			return Utils.AppendFrontSlash( input );
		}
		
		private string ModifySlashFromCak(string input)
		{
			return Utils.RemoveFrontSlash( input );
		}
		
		void AddContents(SQX_ARCNODE node)
		{			
			if (Utils.ExtractFileName(node.pszFileName) == "") 
				return;
			totalFiles++;
			fileList.Add(node.pszFileName);
			OnItemList(this, new ItemListEventArgs(this,
			                                       new ContentType(ArchiveName, (UInt32)node.dwlSizeOrig, (UInt32)node.dwlSize,
			                                       Utils.DosDateToDateTime(node.dwDosFileTime), node.dwFileCRC,
			                                       ModifySlashToCak(node.pszFileName), node.fEncrypted) ));			
		}
				
		void AddContents(IntPtr node)
		{
			if (node != IntPtr.Zero)
				AddContents((SQX_ARCNODE)Marshal.PtrToStructure(node, typeof(SQX_ARCNODE)));
		}
		
		bool CheckError(Int32 errorCode)
		{
			Debug.WriteLine((ErrorCodes)(errorCode));
			return (errorCode == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		
		
		public override bool List(string mask) 
		{ 
			if(OnItemList == null) return false;				
			DispMessage(Defines.ErrorMessage("PROCESSLIST"), archiveName);				
			bool retVal = false;
			
			totalFiles = 0;
			fileList.Clear();
			
			IntPtr hFileMaskList = SqxInitFileList(harc);			
			IntPtr pArcFileList = SqxInitArcFileList(harc);
			try
			{								
				SQX_ARCLIST ArcFileList = (SQX_ARCLIST)Marshal.PtrToStructure(pArcFileList, typeof(SQX_ARCLIST));
				
				SQX_ARCINFO ArcInfo = new SQX_ARCINFO();
				ArcInfo.cbSize = (UInt32)System.Runtime.InteropServices.Marshal.SizeOf(typeof(SQX_ARCINFO));
				
				SqxAppendFileList(harc, hFileMaskList, "*.*");
				if (CheckError(SqxListFiles(harc, hFileMaskList, ref ArcFileList, ref ArcInfo)))
				{
					
					SQX_ARCLISTNODE aNode = (SQX_ARCLISTNODE)Marshal.PtrToStructure(ArcFileList.pHead, typeof(SQX_ARCLISTNODE));
					AddContents(aNode.pArcNode);
					
					for (Int32 i = 1; i < ArcFileList.dwItemCount; i++)
						if (aNode.pNext != IntPtr.Zero)
					{
						aNode = (SQX_ARCLISTNODE)Marshal.PtrToStructure(aNode.pNext, typeof(SQX_ARCLISTNODE));
						AddContents(aNode.pArcNode);
					}
					retVal = true;
				}
			}
			catch
			{
				retVal = false;
			}
			finally
			{				
				try
				{
					if (pArcFileList != IntPtr.Zero)
						SqxDoneArcFileList(harc, pArcFileList);
					if (hFileMaskList != IntPtr.Zero)
						SqxDoneFileList(harc, hFileMaskList);
				}	
				catch
				{
					
				}
			}										
			
			return retVal;
		}
		
		public override bool Test()
		{
			DispMessage(Defines.ErrorMessage("PROCESSTEST"), archiveName);
			resetCallBack();
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			
			if (Open())
			{
				iStatus = SqxTestArchive(harc, SqxCallBackProc, Marshal.StringToHGlobalAuto("Test"));			
			}
			 
				
			Close();
			
			return (iStatus == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		public override bool Extract(ExtractOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSEXTR"), archiveName);			
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			resetCallBack();
			if (Open())
			{								
				
				SQX_EXTRACTOPTIONS extractOptions = new SQX_EXTRACTOPTIONS();
								
				extractOptions.szOutputPath 	= options.ExtractFolderParsed;
				extractOptions.szPassword 		= options.password;
				extractOptions.fKeepBrokenFiles = true;
				extractOptions.fCreateFolders   = options.allowFolder;
				extractOptions.fOverwriteAlways = false;				
				
				extractOptions.cbSize = Marshal.SizeOf(typeof(SQX_EXTRACTOPTIONS));
								
				foreach (string item in options.extractItem)
				{
					IntPtr hFileMaskList = SqxInitFileList(harc);	
					SqxAppendFileList(harc, hFileMaskList, item);										
					
					extractOptions.cbSize = 1348;
					iStatus = SqxExtractFiles(harc, SqxCallBackProc, 
					                          Marshal.StringToHGlobalAuto("Extract"), 
					                          hFileMaskList, ref extractOptions);
										
					SqxDoneFileList(harc, hFileMaskList);	
																						
				}				
				
				Close();
			}
			return CheckError(iStatus);
		}
		
		string getFullPath(string filePath, ref string basePath)
		{
			basePath = Utils.GetFileDrive(filePath) + @":\";
			return Utils.RemoveFileDrive(filePath);
		}
		
		string getRelativePath(string filePath, ref string basePath)
		{			
			if (basePath == "")
			{
				getFullPath(filePath, ref basePath);
			}
							
			string retVal;
			basePath = Utils.RemoveSlash(basePath).ToLower();
			filePath = filePath.ToLower();
			string path = Utils.ExtractFilePath(filePath);
			string name = Utils.ExtractFileName(filePath);
			
			if (path.IndexOf(basePath) == 0)				//06-09-07
				retVal = path.Substring(basePath.Length);
			else retVal = path;
			
			if (retVal != "")
				retVal += @"\";
			
			return retVal + name;
		}
		
		
		public override bool Add(AddOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSADD"), archiveName);			
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			resetCallBack();
			if (Open())
			{												
				SQX_COMPRESSOPTIONS compressOptions = new SQX_COMPRESSOPTIONS();
								
				compressOptions.dwFileFormat = (int)FileFormat.SQX_FILEFORMAT_20;
				compressOptions.compOptions.dwCompRate = (int)CompressRate.SQX_COMPRATE_MAXIMIUM;
				compressOptions.compOptions.dwDictionarySize = (int)DictionarySize.SQX_DICTIONARY_2M;
				compressOptions.compOptions.fSolidFlag = false;
				compressOptions.szPassword = options.password;
				if (options.password != "")
					compressOptions.dwEncryption = 1;
				
				compressOptions.compOptions.dwExeCompression = (int)SpecialCompressor.SQX_SPECIALCOMP_AUTOMATIC;
        		compressOptions.compOptions.dwRgbCompression = (int)SpecialCompressor.SQX_SPECIALCOMP_AUTOMATIC;;
        		compressOptions.compOptions.dwTextCompression = (int)SpecialCompressor.SQX_SPECIALCOMP_AUTOMATIC;;
        		compressOptions.compOptions.dwAudioCompression = (int)SpecialCompressor.SQX_SPECIALCOMP_AUTOMATIC;;
        		compressOptions.compOptions.dwMultimediaCompression = (int)SpecialCompressor.SQX_SPECIALCOMP_AUTOMATIC;;
				compressOptions.fRetainFolderStructure = true;
				compressOptions.cbSize = Marshal.SizeOf(typeof(SQX_COMPRESSOPTIONS));			
								
				List<string> AddList = AddOptions.ParseAddFileMask(options.addFile, true);
				foreach (string item in AddList)
				{					
					IntPtr hFileMaskList = SqxInitFileList(harc);	
					
					string baseFolder = options.baseFolder;
					string filename = item;
					
					switch (options.addFolder)
					{
						case (AddOptions.folderMode.relative):
							filename = getRelativePath(item, ref baseFolder);
							break;
						case (AddOptions.folderMode.full):
							filename = getFullPath(item, ref baseFolder);
							break;
						case (AddOptions.folderMode.none):
							baseFolder = Utils.ExtractFilePath(item);
							filename = Utils.ExtractFileName(item);
							break;
					}
					
					SqxAppendFileList(harc, hFileMaskList, filename);
					compressOptions.szInputPath = baseFolder;										
									
					//TODO: Figure out why Callback crash.					
					iStatus = SqxCompressFiles(harc, /*SqxCallBackProc*/ null, 
					                          Marshal.StringToHGlobalAuto("Compress"), 
					                          hFileMaskList, ref compressOptions);					
					
					SqxDoneFileList(harc, hFileMaskList);		
					
					Close();
					Open();
				}				
				
				Close();
			}
			return CheckError(iStatus);
		}
		
		public override bool Delete(DeleteOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSDEL"), archiveName);				
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			resetCallBack();
			if (Open())
			{											
				List("*.*");
				
				IntPtr hFileMaskList = SqxInitFileList(harc);
				
				foreach (string item in options.deleteFile)
				{		
					if (item.IndexOf("*") == -1)
						SqxAppendFileList(harc, hFileMaskList, ModifySlashFromCak(item));
					else
					foreach (string file in fileList)
						if (Utils.MatchMask(file, ModifySlashFromCak(item)))
						{
							SqxAppendFileList(harc, hFileMaskList, file);							
						}
				}	
				
				iStatus = SqxDeleteFiles(harc, SqxCallBackProc, 
					                          Marshal.StringToHGlobalAuto("Delete"), 	
					                          hFileMaskList);
				SqxDoneFileList(harc, hFileMaskList);
				;
				Close();			
			}
			
			return CheckError(iStatus);
		}
				
		
		
	}
}
