/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 2/10/2007
 * Time: 2:07
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */


using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;


namespace Cake3.WcxPlugins
{
	internal class Kernel
	{
		public const Int32 MAXPATH = 260;
	}
	internal class Win32API
    {
        [DllImport("kernel32.dll")]
        internal static extern IntPtr LoadLibrary(string ArchiverName);
        [DllImport("kernel32.dll")]
        internal static extern bool FreeLibrary(IntPtr hModule);
        [DllImport("kernel32.dll")]
        internal static extern IntPtr GetProcAddress(IntPtr hModule, string procname);
        [DllImport("kernel32.dll")]
        internal static extern bool DosDateTimeToFileTime(ushort wFatDate, ushort wFatTime, out Int64 lpFileTime);
    }
	
	internal static class WcxDefs
	{
		#region Constants		
		//ErrorCodes
		public enum ErrorCodes
		{
			E_END_ARCHIVE=     10,	//{No more files in archive}
			E_NO_MEMORY=       11,  //{Not enough memory}
			E_BAD_DATA=        12, 	//{Data is bad}
			E_BAD_ARCHIVE=     13,  //{CRC error in archive data}
			E_UNKNOWN_FORMAT=  14,  //{Archive format unknown}
			E_EOPEN=           15,  //{Cannot open existing file}

			E_ECREATE=         16,  //{Cannot create file}
			E_ECLOSE=          17,  //{Error closing file}
			E_EREAD=           18,  //{Error reading from file}
			E_EWRITE=          19,  //{Error writing to file}
			E_SMALL_BUF=       20,  //{Buffer too small}
			E_EABORTED=        21,  //{Function aborted by user}
			E_NO_FILES=        22,  //{No files found}
			E_TOO_MANY_FILES=  23,  //{Too many files to pack}
			E_NOT_SUPPORTED=   24   //{Function not supported}			
		}		
        
		public const int PK_OM_LIST = 0;
		public const int PK_OM_EXTRACT = 1;
  		
		//{Flags for ProcessFile}
		public const int PK_SKIP = 0;	//{Skip file (no unpacking)}
		public const int PK_TEST = 1;	//{Test file integrity}
		public const int PK_EXTRACT = 2;//{Extract file to disk}
		
		//{Flags passed through ChangeVolProc}
		public const int PK_VOL_ASK = 0; 			//{Ask user for location of next volume}
		public const int PK_VOL_NOTIFY = 1; 		//{Notify app that next volume will be unpacked}
		
  		//{For PackFiles}
  		public const int PK_PACK_MOVE_FILES = 1;    //{Delete original after packing}
		public const int PK_PACK_SAVE_PATHS = 2;	//{Save path names of files}
  
		//{Returned by GetPackCaps}
		public const int PK_CAPS_NEW = 1;			//{Can create new archives}
		public const int PK_CAPS_MODIFY = 2;		//{Can modify exisiting archives}
		public const int PK_CAPS_MULTIPLE = 4;		//{Archive can contain multiple files}
		public const int PK_CAPS_DELETE = 8;		//{Can delete files}
		public const int PK_CAPS_OPTIONS = 16;		//{Supports the options dialogbox}
		public const int PK_CAPS_MEMPACK = 32;		//{Supports packing in memory}
		public const int PK_CAPS_BY_CONTENT = 64;	//{Detect archive type by content}
		public const int PK_CAPS_SEARCHTEXT = 128;	//{Allow searching for text in archives created with this plugin}
		public const int PK_CAPS_HIDE = 256;		//{ Show as normal files (hide packer icon)  
		                                            //{ open with Ctrl+PgDn, not Enter }}
		                                            
  		//{Flags for packing in memory}
  		public const int MEM_OPTIONS_WANTHEADERS = 1;	//{Return archive headers with packed data}
  		
  		// {Errors returned by PackToMem}
  		public const int MEMPACK_OK = 0; 	//{Function call finished OK, but there is more data}
  		public const int MEMPACK_DONE = 1;  //{Function call finished OK, there is no more data}
  		#endregion    		
  		
  		#region Structures
  		[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Ansi)]
  		public struct tHeaderData
  		{
  			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]public string ArcName;
  			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]public string FileName;
  			public Int32 Flags;
  			public Int32 PackSize;
  			public Int32 UnpSize;
  			public Int32 HostOS;
  			public Int32 FileCRC;
  			public Int32 FileTime;
  			public Int32 UnpVer;
  			public Int32 Method;
  			public Int32 FileAttr;
  			[MarshalAs(UnmanagedType.LPStr)]public string CmtBuf;
  			public Int32 CmtBufSize;
  			public Int32 CmtSize;
  			public Int32 CmtState;
  		}
  		
  		[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Ansi)]
  		public struct tOpenArchiveData
  		{
  			[MarshalAs(UnmanagedType.LPStr)]public string ArcName;
  			public Int32 OpenMode;
  			public Int32 OpenResult;
  			[MarshalAs(UnmanagedType.LPStr)]public string CmtBuf;
  			public Int32 CmtBufSize;
  			public Int32 CmtSize;
  			public Int32 CmtState;
  		}
  		
  		[StructLayout(LayoutKind.Sequential, Pack=1, CharSet=CharSet.Ansi)]
  		public struct PackDefaultParamStruct
  		{
  			public Int64 PluginInterfaceVersionLow;
  			public Int64 PluginInterfaceVersionHi;
  			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = Kernel.MAXPATH)]
  			public string DefaultIniName;
  		}
  		#endregion  		
	}
	
	internal class WcxPluginInfo
	{
		string wcxPath;
		string extensions;
		string description;
		public string WcxPath { get { return wcxPath; } }
		public string Extensions { get { return extensions; } }
		public string Description { get { return description; } }
		
		internal WcxPluginInfo(string aWcxPath, string anExtensions, string aDescription)
		{
			wcxPath = aWcxPath;			
			
			description = aDescription;
			if (anExtensions != "")
				extensions = "." + anExtensions.ToLower().Replace(",",",.");
		}
		
		public bool Support(string ext)
		{
			return (extensions.IndexOf(ext.ToLower()) != -1);
		}
	}
	
}
