/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 2/10/2007
 * Time: 2:11
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using System.IO;
using Cake3;

namespace Cake3.WcxPlugins
{
	/// <summary>
	/// Description of WcxArchiver.
	/// Place the plugins in {app}\\plugins\\wcx\\{plugin} folder, 
	/// pluginst.inf must exist in that folder, with at least "file" and "defaultextension" defined.
	/// </summary>
	public class WcxArchiver : CakArchiver
	{
		#region Header
		internal delegate IntPtr WcxOpenArchive(ref WcxDefs.tOpenArchiveData ArchiveData); 
		internal delegate Int32 WcxCloseArchive(IntPtr hArcData); 
		internal delegate Int32 WcxReadHeader(IntPtr hArcData, out WcxDefs.tHeaderData HeaderData);		
		internal delegate Int32 WcxProcessFile(IntPtr hArcData, Int32 Operation, 
		                                     [MarshalAs(UnmanagedType.LPStr)]string DestPath,
		                                     [MarshalAs(UnmanagedType.LPStr)]string DestName
		                                    );	
		internal delegate Int32 WcxPackFiles([MarshalAs(UnmanagedType.LPStr)]string PackedFile,
		                                     [MarshalAs(UnmanagedType.LPStr)]string SubPath,
		                                     [MarshalAs(UnmanagedType.LPStr)]string SrcPath,
		                                     [MarshalAs(UnmanagedType.LPStr)]string AddList,
		                                     Int32 Flags);
		internal delegate Int32 WcxDeleteFiles([MarshalAs(UnmanagedType.LPStr)]string PackedFile,
		                                       [MarshalAs(UnmanagedType.LPStr)]string DeleteList);
		
//		internal delegate void WcxSetProcessDataProc(IntPtr hArcData, ref WcxDefs.tProcessDataProc pProcessDataProc);
		
		private WcxOpenArchive OpenArchive;
		private WcxCloseArchive CloseArchive;
		private WcxReadHeader ReadHeader;
		private WcxProcessFile ProcessFile;
		private WcxPackFiles PackFiles;
		private WcxDeleteFiles DeleteFiles;			
		
		private IntPtr hArchiver;
		
		public void LoadWcx(string dllPath)
		{
			hArchiver = Win32API.LoadLibrary(dllPath);
			if (hArchiver == IntPtr.Zero) 
            	{ throw new Exception("Cannot load " + dllPath); }
			LoadFunction(dllPath);
		}
		
		public void LoadFunction(string dllPath)
		{
			IntPtr procaddr;
			procaddr = Win32API.GetProcAddress(hArchiver, "OpenArchive");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { throw new Exception("Cannot load command OpenArchive"); }
			OpenArchive = (WcxOpenArchive)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxOpenArchive));
			
			procaddr = Win32API.GetProcAddress(hArchiver, "CloseArchive");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { throw new Exception("Cannot load command CloseArchive"); }
			CloseArchive = (WcxCloseArchive)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxCloseArchive));
			
			procaddr = Win32API.GetProcAddress(hArchiver, "ReadHeader");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { throw new Exception("Cannot load command ReadHeader"); }
			ReadHeader = (WcxReadHeader)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxReadHeader));
			
			procaddr = Win32API.GetProcAddress(hArchiver, "ProcessFile");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { throw new Exception("Cannot load command ProcessFile"); }
			ProcessFile = (WcxProcessFile)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxProcessFile));
			
			procaddr = Win32API.GetProcAddress(hArchiver, "PackFiles");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { canAdd = false; }
			else PackFiles = (WcxPackFiles)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxPackFiles));
			
			procaddr = Win32API.GetProcAddress(hArchiver, "DeleteFiles");
			if ((hArchiver == IntPtr.Zero) || (procaddr == IntPtr.Zero)) { canAdd = false; }
			else DeleteFiles = (WcxDeleteFiles)Marshal.GetDelegateForFunctionPointer(procaddr, typeof(WcxDeleteFiles));
		}
		
		#endregion
		
		internal override string name {	get { return "wcxplugin"; }}
		WcxPluginInfo ActivePlugin = null;
		
		public WcxArchiver() : base()
		{			
		}
				
		
		internal override void setArchiveName(string anArchive)
		{			
			base.setArchiveName(anArchive);
			
			ActivePlugin = null;
			string ext = Utils.ExtractFileExt(anArchive);
			
			foreach (WcxPluginInfo info in WcxPluginList)
				if (info.Support(ext))
					ActivePlugin = info;
			
			canList = true;
			canExtract = true;	
//			canAdd = true;
			
			if (ActivePlugin != null)
				LoadWcx(Utils.GetProgramPath() + ActivePlugin.WcxPath);
		
		}
		
		private Int32 ProcessDataCallback([MarshalAs(UnmanagedType.LPStr)]string FileName, Int32 Size)
		{
			DispProgress(0, FileName);
			
			return 1;
			//0 = cancel
		}
		
		private bool ReadPluginst(string dirpath, out WcxPluginInfo pluginInfo)
		{
			string extensions = "";		
			string description = Utils.ExtractFileName(Utils.RemoveSlash(dirpath));
			string wcxpath = Utils.AppendSlash(dirpath).Replace(Utils.GetProgramPath(), "");
			string wcxfile = "";
			pluginInfo = new WcxPluginInfo("","","");
			
			string infPath = Utils.AppendSlash(dirpath) + "pluginst.inf";			
			if (!Utils.FileExists(infPath)) return false;
			
			FileStream fs = new FileStream(infPath, FileMode.Open);
			StreamReader sr = new StreamReader(fs, Encoding.ASCII);
			
			
			string line = sr.ReadLine();
			while (line != null)
			{
				string[] sections = line.Split('=');
				if (sections.Length == 2)
				{
					string section = sections[0];
					string value = sections[1];
					
					switch (section)
					{
						case "defaultextension":
							extensions = value;
							break;
						case "file":
							wcxfile = value;
							break;
						case "description":
							description = value;
							break;
//						case "defaultdir":
//							if (wcxpath.IndexOf("\\") == -1)
//								wcxpath = "plugins\\wcx\\" + value + "\\";
//							else wcxpath = Utils.AppendSlash(value);
//							break;
					}					
				}
				line = sr.ReadLine();
			}
			
			sr.Close();
			fs.Close();
			
			pluginInfo = new WcxPluginInfo(wcxpath + wcxfile, extensions, description);
			
			return (extensions != "");			
		}

		private WcxPluginInfo[] WcxPluginList
		{
			get {
				List<WcxPluginInfo> pluginList = new List<WcxPluginInfo>();
				
				List<string> DirList = Utils.PollDirList(Utils.GetProgramPath() + @"\plugins\wcx\", false);
				WcxPluginInfo info;
				
				foreach (string dir in DirList)
					if (ReadPluginst(dir, out info))
						pluginList.Add(info);
				
				return pluginList.ToArray();
			}
		}
				
		public override List<ArchiveType> SupportedArchiveTypes()
		{
			List<ArchiveType> retVal = new List<ArchiveType>();
			List<string> DirList = Utils.PollDirList(Utils.GetProgramPath() + "\\plugins\\wcx\\", false);
			foreach (WcxPluginInfo info in WcxPluginList)
				retVal.Add(new ArchiveType(name, info.Extensions.Replace(".",""), 0, true, true, false, false, "/w" ));			
			return retVal;
		}
		
		public override bool Open()
		{ 
			if (archiveOpened) 
				return false;
			
			WcxDefs.tOpenArchiveData thd = new WcxDefs.tOpenArchiveData();
			thd.ArcName = ArchiveName;
			thd.OpenMode = WcxDefs.PK_OM_LIST;
			hArchiver = OpenArchive(ref thd);

			return (hArchiver != IntPtr.Zero);
		}
		
		public override bool Close()
		{			
			if (!archiveOpened) 
				return false;									
			
			CheckError(CloseArchive(hArchiver), ArchiveName);
			
			archiveOpened = false;			
			return true;
		}
		
		private string ModifySlashToCak(string input)
		{
			return Utils.AppendFrontSlash( input );
		}
		
		private string ModifySlashFromCak(string input)
		{
			return Utils.RemoveFrontSlash( input );
		}
		
		public override bool List(string mask) 
		{ 
			if(OnItemList == null) return false;				
			DispMessage(Defines.ErrorMessage("PROCESSLIST"), archiveName);							
			
			WcxDefs.tHeaderData data;
			while (ReadHeader(hArchiver, out data) != (int)WcxDefs.ErrorCodes.E_END_ARCHIVE)
			{				
				if (data.FileAttr != 16) //Dir
				OnItemList(this, new ItemListEventArgs(this,
			                                       new ContentType(ArchiveName, (UInt32)data.UnpSize, (UInt32)data.PackSize,
			                                       Utils.DosDateToDateTime(data.FileTime), data.FileCRC,
			                                       ModifySlashToCak(data.FileName), false) ));	
				ProcessFile(hArchiver, WcxDefs.PK_SKIP, null, null);
			}								
			
			return true;
		}
		
		public bool CheckError(Int32 errorCode, string fileName)
		{
		switch (errorCode)
		{
		case 0 : return true;
		case (int)WcxDefs.ErrorCodes.E_END_ARCHIVE :    return true;
		case (int)WcxDefs.ErrorCodes.E_NO_MEMORY : 		DispError(Defines.ErrorMessage("NOMEMORY"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_BAD_DATA :  		DispError(Defines.ErrorMessage("UNKTYPE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_BAD_ARCHIVE : 	DispError(Defines.ErrorMessage("CRC"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_UNKNOWN_FORMAT : DispError(Defines.ErrorMessage("UNKTYPE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_ECREATE : 		DispError(Defines.ErrorMessage("CANTCREATE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_EOPEN : 			DispError(Defines.ErrorMessage("LISTERROR"), ArchiveName); return false;
		case (int)WcxDefs.ErrorCodes.E_ECLOSE : 		DispError(Defines.ErrorMessage("CANTCLOSE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_EREAD :   		DispError(Defines.ErrorMessage("CANTREAD"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_EWRITE :   		DispError(Defines.ErrorMessage("CANTWRITE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_SMALL_BUF :   	DispError(Defines.ErrorMessage("DISKFULL"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_EABORTED :   	DispError(Defines.ErrorMessage("USERSKIP"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_NO_FILES :   	DispError(Defines.ErrorMessage("NOTFOUNDARC"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_TOO_MANY_FILES : DispError(Defines.ErrorMessage("TOOMANYFILE"), fileName); return false;
		case (int)WcxDefs.ErrorCodes.E_NOT_SUPPORTED :  DispError(Defines.ErrorMessage("NOSUPPORT"), fileName); return false;
		}
		
			return true;
		}
		
		public override bool Extract(ExtractOptions options)
		{
			DispMessage(Defines.ErrorMessage("PROCESSEXTR"), archiveName);
			string outFolder = Utils.AppendSlash(options.extractFolder);
			if (Open())
			{	
				WcxDefs.tHeaderData data;				
				while (ReadHeader(hArchiver, out data) != (int)WcxDefs.ErrorCodes.E_END_ARCHIVE)				
				{					
					if ((data.FileAttr != 16) && (Utils.MatchMasks(ModifySlashToCak(data.FileName), options.extractItem))) //Dir
					{
						string outName = outFolder + data.FileName;
						if (!options.allowFolder) 
							outName = outFolder + Utils.ExtractFileName(data.FileName);
						//TODO: OnProgress support
						Utils.MakeDirectory(Utils.ExtractFilePath(outName));
						if (CheckError(ProcessFile(hArchiver, WcxDefs.PK_EXTRACT, null, outName), data.FileName))
							DispMessage(Defines.ErrorMessage("PROCESSOK"), data.FileName);
					} else ProcessFile(hArchiver, WcxDefs.PK_SKIP, null, null);
										
				}
				Close();
			}
			
			return true;
		}
		
		public override bool Add(AddOptions options)
		{			
			Close();
			
			foreach (string file in options.addFile)
			{
				string addFile = "";
				string addPath = "";
				switch (options.addFolder)
				{
					case AddOptions.folderMode.full : 
						addFile = Utils.RemoveFileDrive(file);
						addPath = Utils.GetFileDrive(file) + @":\";
						break;
					case AddOptions.folderMode.none : 
						addFile = Utils.ExtractFileName(file);
						addPath = Utils.ExtractFilePath(file);
						break;
					case AddOptions.folderMode.relative :						
						addFile = Utils.getRelativePath(file, options.baseFolder);
						addPath = options.baseFolder;
						break;
				}
				
				if (CheckError(PackFiles(ArchiveName, null,Utils.AppendSlash(addPath), Utils.RemoveFrontSlash(addFile) + '\0', WcxDefs.PK_PACK_SAVE_PATHS), file))
					DispMessage(Defines.ErrorMessage("PROCESSOK"), file);
			}
			
			return true;
			
		}
		
		public override bool Delete(DeleteOptions options)
		{
			Close();
			
			foreach (string file in options.deleteFile)
			{
				
				
				if (CheckError(DeleteFiles(ArchiveName, file), file))
					DispMessage(Defines.ErrorMessage("PROCESSOK"), file);
			}
			
			return true;
		}
	}
}
