/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 26/11/2006
 * Time: 23:36
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using Cake3;

namespace Cake3.Sqx
{
	/// <summary>
	/// Description of SqxArchiver.
	/// </summary>
	public class SqxArchiver : CakArchiver
	{
		#region DllHeader
		private enum ErrorCodes
		{
			SQX_ERR_SUCCESS = 0,
			SQX_ERR_ERROR,
			SQX_ERR_FILE_NOT_FOUND,
			SQX_ERR_PATH_NOT_FOUND,
			SQX_ERR_TOO_MANY_FILES,
			SQX_ERR_ACCESS_DENIED,
			SQX_ERR_INVALID_HANDLE,
			SQX_ERR_DISK_FULL,
			SQX_ERR_OUT_OF_MEMORY,
			SQX_ERR_CANT_ACCESS_TEMP_DIR,
			SQX_ERR_TEMP_DIR_FULL,
			SQX_ERR_USER_ABORT,
			SQX_ERR_INVALID_ARC_HANDLE,
			SQX_ERR_CANT_FIND_LANG_DATA,
			SQX_ERR_UNKNOWN_SUBSTREAM,
			SQX_ERR_BAD_SUBSTREAM_CRC,
			SQX_ERR_UNKNOWN_METHOD,
			SQX_ERR_FILE_ENCRYPTED,
			SQX_ERR_BAD_CRC,
			SQX_ERR_CANT_CREATE_FILE,
			SQX_ERR_BAD_FILE_FORMAT,
			SQX_ERR_FUNCTION_NOT_SUPPORTED,
			SQX_ERR_FUNC_NOT_SUPPORTED_BY_ARCHIVE,
			SQX_ERR_CANT_CREATE_ARC_DIR,
			SQX_ERR_INVALID_DIR_NAME,
			SQX_ERR_INVALID_FILE_NAME,
			SQX_ERR_TOO_MANY_BROKEN_FBLOCKS,
			SQX_ERR_ARCHIVE_OK_RDATA_NOT,
			SQX_ERR_RDATA_DAMAGED,
			SQX_ERR_RDATA_NEW_VERSION,
			SQX_ERR_RDATA_DOES_NOT_MATCH,
			SQX_ERR_CANT_FIND_RDATA,
			SQX_ERR_ARCHIVE_IS_LOCKED,
			SQX_ERR_CANT_ADD_TO_MV,
			SQX_ERR_CANT_DELETE_FROM_MV,
			SQX_ERR_NEED_1ST_VOLUME,
			SQX_ERR_MISSING_VOLUME,
			SQX_ERR_VOLUME_LIMIT_REACHED,
			SQX_ERR_SFXSTUB_NOT_INSTALLED,
			SQX_ERR_BACKUP_READ_ACCESS_DENIED,
			SQX_ERR_BACKUP_WRITE_ACCESS_DENIED,
			SQX_ERR_ACL_READ_ACCESS_DENIED,
			SQX_ERR_ACL_WRITE_ACCESS_DENIED,
			SQX_ERR_WRONG_ARCHIVER_VERSION,
			SQX_ERR_CANT_COPY_SOURCE_TO_SOURCE,
			SQX_ERR_VOLUMES_TOO_SMALL,
			SQX_ERR_ARCHIVE_VERSION_TOO_HIGH,
			SQX_ERR_EXT_RDATA_DOES_NOT_MATCH,
			SQX_ERR_BAD_PARAMETER,
			SQX_ERR_EQUAL_PASSWORDS,
			SQX_ERR_REQUIRES_ENCRYPTION,
			SQX_ERR_MISSING_HEADER_PASSWORD,
			SQX_ERR_MISSING_SQX_PRIVATE_KEY,
			SQX_ERR_MISSING_SQX_AVKEY,
			SQX_ERR_MISSING_EXTERNAL_AVKEY,
			SQX_ERR_INVALID_SQX_AVKEY,
			SQX_ERR_SQX_AVKEY_VERSION,
			SQX_ERR_SQX_AVDATA_VERSION,
			SQX_ERR_SQX_BROKEN_AVRECORD,
			SQX_ERR_RIJNDAEL_RSA,
			SQX_ERR_REQUIRES_NTFS,
			SQX_ERR_REQUIRES_WINNT,
			SQX_ERR_REQUIRES_W2K,
			SQX_ERR_REQUIRES_WINXP,
			SQX_ERR_REQUIRES_WINXP_SP1,
			SQX_ERR_REQUIRES_WINXP_SP2,
			SQX_ERR_REQUIRES_LONGHORN,
			SQX_ERR_NO_RESOURCES_FOUND,
			SQX_ERR_UNKNOWN_ICON_FORMAT,
			SQX_ERR_NO_MATCHING_ICON_SIZE,
			SQX_ERR_UNKNOWN_EXE_FORMAT,
			SQX_ERR_REQUIRES_SOURCE_PATH,
			SQX_ERR_FILE_DATA_NOT_EQUAL,
			SQX_ERR_COMMENT_BIGGER_4K,
			SQX_ERR_CANT_CREATE_SFX_FROM_MV
		}	
		
		private const string DllName = "sqx20.dll";		
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxInitArchive([MarshalAs(UnmanagedType.LPStr)]string pszArchiveName,
		                                           dSqxCallback pfnCallback,
		                                           IntPtr pParam,
		                                           ref IntPtr phArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneArchive(IntPtr hArchive);
				
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static IntPtr SqxInitFileList(IntPtr hArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneFileList(IntPtr hArchive, IntPtr hFileMaskList);

		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static IntPtr SqxInitArcFileList(IntPtr hArchive);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxDoneArcFileList(IntPtr hArchive, IntPtr pArcFileList);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxListFiles(IntPtr hArchive, IntPtr hFileMaskList, ref SQX_ARCLIST pArcFileList,ref SQX_ARCINFO pArchiveInfo);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static void SqxAppendFileList(IntPtr hArchive, IntPtr hFileMaskList, string pszFileName);
					
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxTestArchive(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam);
		
		[DllImport(DllName, CharSet=CharSet.Ansi)]
		private extern static Int32 SqxExtractFiles(IntPtr hArchive, dSqxCallback pfnCallback, IntPtr pParam, 
		                                            IntPtr hFileMaskList, 	
		                                            ref SQX_EXTRACTOPTIONS pExtractOptions);  //pExtractOptions		
		
		#endregion
		
		IntPtr harc;
		UInt32 totalFiles, totalProcessed;	
		string ActionCode;
		
		void resetCallBack()
		{
			totalProcessed = 0;
		}

		public int SqxCallback(IntPtr pParam, ref SQX_CALLBACKINFO CallbackInfo)
		{
			if (CallbackInfo.pszSourceFileName != null)
			{
				totalProcessed++;
				
				UInt16 Ratio = (UInt16)((float)totalProcessed/(float)totalFiles*100);
			
				if ((OnProgress != null))
					OnProgress(this, new ProgressEventArgs(this, Ratio, CallbackInfo.pszSourceFileName));
				
				if ((OnMessage != null) && (ActionCode != ""))
				{
					string ProgressStr = Defines.ErrorCode(ActionCode).errorDescription;
					OnMessage(this, new MessageEventArgs(this, string.Format(ProgressStr, CallbackInfo.pszSourceFileName)));
				}
			}									
			
			return 1;
		}
		
		public SqxArchiver()
		{
		}
				
		
		public override void setArchiveName(string anArchive)
		{			
			base.setArchiveName(anArchive);
			
			string extension = Cake3.Utils.ExtractFileExt(anArchive).ToLower();
			switch (extension)
			{
				case ".sqx" :					
					break;
				default : throw new NotSupportedException();
			}
		}
		
		public override List<string> SupportedExtensions()
		{ 
			List<string> retVal = new List<string>();
			retVal.Add(".sqx");		
			return retVal;		
		}						
		
		public override bool Open()
		{ 
			if (archiveOpened) 
				return false;
			dSqxCallback SqxCallBackProc = new dSqxCallback(SqxCallback);		
			Int32 iStatus = SqxInitArchive(archiveName, SqxCallBackProc, IntPtr.Zero, ref harc);						
			
			archiveOpened = true;
			return (iStatus == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		public override bool Close()
		{
			if (!archiveOpened) 
				return false;									
			
			SqxDoneArchive(harc);		
			
			archiveOpened = false;
			ActionCode = "";
			
			return true;
		}
		
		private string ModifySlashToCak(string input)
		{
			return Utils.AppendFrontSlash( input );
		}
		
		private string ModifySlashFromCak(string input)
		{
			return Utils.RemoveFrontSlash( input );
		}
		
		void AddContents(SQX_ARCNODE node)
		{
			totalFiles++;
			
			OnItemList(this, new ItemListEventArgs(this,
			                                       new ContentType((UInt32)node.dwlSizeOrig, (UInt32)node.dwlSize,
			                                       Utils.DosDateToDateTime(node.dwDosFileTime), node.dwFileCRC,
			                                       ModifySlashToCak(node.pszFileName), node.fEncrypted) ));			
		}
				
		void AddContents(IntPtr node)
		{
			if (node != IntPtr.Zero)
				AddContents((SQX_ARCNODE)Marshal.PtrToStructure(node, typeof(SQX_ARCNODE)));
		}
		
		bool CheckError(Int32 errorCode)
		{
			Debug.WriteLine((ErrorCodes)(errorCode));
			return (errorCode == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		public override bool List(string mask) 
		{ 
			if(OnItemList == null) return false;			
			bool retVal = false;
			
			totalFiles = 0;
			
			IntPtr hFileMaskList = SqxInitFileList(harc);			
			IntPtr pArcFileList = SqxInitArcFileList(harc);
			try
			{								
				SQX_ARCLIST ArcFileList = (SQX_ARCLIST)Marshal.PtrToStructure(pArcFileList, typeof(SQX_ARCLIST));
				
				SQX_ARCINFO ArcInfo = new SQX_ARCINFO();
				ArcInfo.cbSize = (UInt32)System.Runtime.InteropServices.Marshal.SizeOf(typeof(SQX_ARCINFO));
				
				SqxAppendFileList(harc, hFileMaskList, "*.*");
				if (CheckError(SqxListFiles(harc, hFileMaskList, ref ArcFileList, ref ArcInfo)))
				{
					
					SQX_ARCLISTNODE aNode = (SQX_ARCLISTNODE)Marshal.PtrToStructure(ArcFileList.pHead, typeof(SQX_ARCLISTNODE));
					AddContents(aNode.pArcNode);
					
					for (Int32 i = 1; i < ArcFileList.dwItemCount; i++)
						if (aNode.pNext != IntPtr.Zero)
					{
						aNode = (SQX_ARCLISTNODE)Marshal.PtrToStructure(aNode.pNext, typeof(SQX_ARCLISTNODE));
						AddContents(aNode.pArcNode);
					}
					retVal = true;
				}
			}
			catch
			{
				retVal = false;
			}
			finally
			{				
				try
				{
					if (pArcFileList != IntPtr.Zero)
						SqxDoneArcFileList(harc, pArcFileList);
					if (hFileMaskList != IntPtr.Zero)
						SqxDoneFileList(harc, hFileMaskList);
				}	
				catch
				{
					
				}
			}										
			
			return retVal;
		}
		
		public override bool Test()
		{
			ActionCode = "PROCESSTEST";
			resetCallBack();
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			
			if (Open())
			{
				dSqxCallback SqxCallBackProc = new dSqxCallback(SqxCallback);				
				iStatus = SqxTestArchive(harc, SqxCallBackProc, Marshal.StringToHGlobalAuto("Test"));			
			}
			 
				
			Close();
			
			return (iStatus == (Int32)ErrorCodes.SQX_ERR_SUCCESS);
		}
		
		public override bool Extract(ExtractOptions options)
		{
			ActionCode = "PROCESSEXTR";
			Int32 iStatus = (Int32)ErrorCodes.SQX_ERR_ERROR;
			resetCallBack();
			if (Open())
			{								
				dSqxCallback SqxCallBackProc = new dSqxCallback(SqxCallback);
				
				SQX_EXTRACTOPTIONS extractOptions = new SQX_EXTRACTOPTIONS();
								
				extractOptions.szOutputPath 	= options.extractTo;
				extractOptions.szPassword 		= options.password;
				extractOptions.fKeepBrokenFiles = true;
				extractOptions.fCreateFolders   = options.extractFolder;
				
				extractOptions.cbSize = Marshal.SizeOf(typeof(SQX_EXTRACTOPTIONS));
								
				foreach (string item in options.extractItem)
				{
					IntPtr hFileMaskList = SqxInitFileList(harc);	
					SqxAppendFileList(harc, hFileMaskList, item);										
					
					extractOptions.cbSize = 1348;
					iStatus = SqxExtractFiles(harc, SqxCallBackProc, 
					                          Marshal.StringToHGlobalAuto("Extract"), 
					                          hFileMaskList, ref extractOptions);
										
					SqxDoneFileList(harc, hFileMaskList);	
																						
				}				
				
				Close();
			}
			return CheckError(iStatus);
		}
		
		
	}
}
